@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
+  Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
 
   The MySQL Connector/ODBC is licensed under the terms of the GPLv2
   <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
@@ -384,7 +384,7 @@ SQLDriverConnect(SQLHDBC hdbc, SQLHWND hwnd, SQLCHAR *in, SQLSMALLINT in_len,
   {
     uint errors;
     /* Now we have to convert SQLWCHAR back into a SQLCHAR. */
-    *out_len= sqlwchar_as_sqlchar_buf(default_charset_info, out, out_max,
+    *out_len= (SQLSMALLINT)sqlwchar_as_sqlchar_buf(default_charset_info, out, out_max,
                                       outw, *out_len, &errors);
     /* TODO what to do with errors? */
 
@@ -837,7 +837,8 @@ SQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT type, SQLPOINTER value,
 
   if (char_value)
   {
-    if (dbc->ansi_charset_info->number != dbc->cxn_charset_info->number)
+    if (dbc->ansi_charset_info && dbc->cxn_charset_info
+      && dbc->ansi_charset_info->number != dbc->cxn_charset_info->number)
     {
       char_value= sqlchar_as_sqlchar(dbc->cxn_charset_info,
                                      dbc->ansi_charset_info,
@@ -1501,15 +1502,24 @@ SQLSetDescRec(SQLHDESC hdesc, SQLSMALLINT record, SQLSMALLINT type,
 }
 
 
-#ifdef NOT_IMPLEMENTED_YET
+/**
+  Discover and enumerate the attributes and attribute values required to
+  connect.
+
+  @return Always returns @c SQL_ERROR, because the driver does not support this.
+
+  @since ODBC 1.0
+*/
 SQLRETURN SQL_API
 SQLBrowseConnect(SQLHDBC hdbc, SQLCHAR *in, SQLSMALLINT in_len,
                  SQLCHAR *out, SQLSMALLINT out_max, SQLSMALLINT *out_len)
 {
-  NOT_IMPLEMENTED;
+  return set_conn_error(hdbc,MYERR_S1000,
+                        "Driver does not support this API", 0);
 }
 
 
+#ifdef NOT_IMPLEMENTED_YET
 //SQLDataSources
 
 
