@@ -86,7 +86,7 @@ static my_bool check_table_type(const SQLCHAR *TableType,
 }
 
 
-static MYSQL_ROW fix_fields_copy(STMT FAR *stmt,MYSQL_ROW row)
+static MYSQL_ROW fix_fields_copy(STMT *stmt,MYSQL_ROW row)
 {
     uint i;
     for ( i=0 ; i < stmt->order_count; ++i )
@@ -914,7 +914,7 @@ SQLRETURN mysql_foreign_keys(SQLHSTMT hstmt,
                            SQLCHAR    *szFkTableName,
                            SQLSMALLINT cbFkTableName)
 {
-  STMT FAR *stmt=(STMT FAR*) hstmt;
+  STMT *stmt=(STMT *) hstmt;
   uint row_count= 0;
 
   MEM_ROOT  *alloc;
@@ -1132,7 +1132,7 @@ mysql_primary_keys(SQLHSTMT hstmt,
                  SQLSMALLINT schema_len __attribute__((unused)),
                  SQLCHAR *table, SQLSMALLINT table_len)
 {
-    STMT FAR  *stmt= (STMT FAR*) hstmt;
+    STMT *stmt= (STMT *) hstmt;
     MYSQL_ROW row;
     char      **data;
     uint      row_count;
@@ -1451,7 +1451,7 @@ mysql_procedure_columns(SQLHSTMT hstmt,
         dynstr_append_os_quoted(&dynQuery, (char *)szColumnName, NullS);
       }
 
-      if(param_ordinal_position == 0)
+      if (param_ordinal_position == 0)
       {
         ptype= SQL_RETURN_VALUE;
       }
@@ -1459,10 +1459,17 @@ mysql_procedure_columns(SQLHSTMT hstmt,
       sprintf(param_type, "%d", ptype);
       data[mypcCOLUMN_TYPE]= my_strdup(param_type, MYF(0)); /* COLUMN_TYPE */
 
-      sprintf(param_sql_type, "%d", (int)type_map->sql_type);
+      if (!myodbc_strcasecmp(type_map->type_name, "bit") && param_size > 1)
+      {
+        sprintf(param_sql_type, "%d", SQL_BINARY);
+      }
+      else
+      {
+        sprintf(param_sql_type, "%d", (int)type_map->sql_type);
+      }
       data[mypcDATA_TYPE]= my_strdup(param_sql_type, MYF(0)); /* DATA_TYPE */
       
-      if(!myodbc_strcasecmp(type_map->type_name, "set") ||
+      if (!myodbc_strcasecmp(type_map->type_name, "set") ||
          !myodbc_strcasecmp(type_map->type_name, "enum"))
       {
         data[mypcTYPE_NAME]= my_strdup("char", MYF(0));
@@ -2134,6 +2141,7 @@ mysql_tables(SQLHSTMT hstmt,
       if (!row_count)
       {
         mysql_free_result(stmt->result);
+        stmt->result= NULL;
         goto empty_set;
       }
 
