@@ -39,6 +39,8 @@
 typedef BOOL (*PromptFunc)(SQLHWND, SQLWCHAR *, SQLUSMALLINT,
                            SQLWCHAR *, SQLSMALLINT, SQLSMALLINT *);
 
+const char *my_os_charset_to_mysql_charset(const char *csname);
+
 /**
   Get the connection flags based on the driver options.
 
@@ -213,6 +215,7 @@ SQLRETURN myodbc_do_connect(DBC *dbc, DataSource *ds)
     mysql_options(mysql, MYSQL_OPT_SSL_VERIFY_SERVER_CERT,
                   (const char *)&opt_ssl_verify_server_cert);
 
+  if (dbc->unicode)
   {
     /*
       Get the ANSI charset info before we change connection to UTF-8.
@@ -220,19 +223,37 @@ SQLRETURN myodbc_do_connect(DBC *dbc, DataSource *ds)
     MY_CHARSET_INFO my_charset;
     mysql_get_character_set_info(&dbc->mysql, &my_charset);
     dbc->ansi_charset_info= get_charset(my_charset.number, MYF(0));
-
+    /*
+      We always use utf8 for the connection, and change it afterwards if needed.
+    */
+    mysql_options(mysql, MYSQL_SET_CHARSET_NAME, "utf8");
+    dbc->cxn_charset_info= utf8_charset_info;
   }
+  else
+  {
+#ifdef _WIN32
+    char cpbuf[64];
+    const char *client_cs_name= NULL;
 
-  /*
-    We always use utf8 for the connection, and change it afterwards if needed.
-  */
-  mysql_options(mysql, MYSQL_SET_CHARSET_NAME, "utf8");
-  dbc->cxn_charset_info= utf8_charset_info;
+    my_snprintf(cpbuf, sizeof(cpbuf), "cp%u", GetACP());
+    client_cs_name= my_os_charset_to_mysql_charset(cpbuf);
+
+    if (client_cs_name)
+    {
+      mysql_options(mysql, MYSQL_SET_CHARSET_NAME, client_cs_name);
+      dbc->ansi_charset_info= dbc->cxn_charset_info= get_charset_by_csname(client_cs_name, MYF(MY_CS_PRIMARY), MYF(0));
+    }
+#else
+    MY_CHARSET_INFO my_charset;
+    mysql_get_character_set_info(&dbc->mysql, &my_charset);
+    dbc->ansi_charset_info= get_charset(my_charset.number, MYF(0));
+#endif
+}
 
 #if MYSQL_VERSION_ID >= 50610
   if (ds->can_handle_exp_pwd)
   {
-    //mysql_options(mysql, MYSQL_OPT_CAN_HANDLE_EXPIRED_PASSWORDS, (char *)&on);
+    mysql_options(mysql, MYSQL_OPT_CAN_HANDLE_EXPIRED_PASSWORDS, (char *)&on);
   }
 #endif
 
@@ -252,10 +273,19 @@ SQLRETURN myodbc_do_connect(DBC *dbc, DataSource *ds)
                           ds_get_utf8attr(ds->socket,   &ds->socket8),
                           flags))
   {
+    unsigned int native_error= mysql_errno(mysql);
+
+    /* Before 5.6.11 error returned by server was ER_MUST_CHANGE_PASSWORD(1820).
+       In 5.6.11 it changed to ER_MUST_CHANGE_PASSWORD_LOGIN(1862)
+       We must to change error for old servers in order to set correct sqlstate */
+    if (native_error == 1820 && ER_MUST_CHANGE_PASSWORD_LOGIN != 1820)
+    {
+      native_error= ER_MUST_CHANGE_PASSWORD_LOGIN;
+    }
 
 #if MYSQL_VERSION_ID < 50610
     /* In that special case when the driver was linked against old version of libmysql*/
-    if (mysql_errno(mysql) == ER_MUST_CHANGE_PASSWORD_LOGIN
+    if (native_error == ER_MUST_CHANGE_PASSWORD_LOGIN
       && ds->can_handle_exp_pwd)
     {
       /* The password has expired, application said it knows how to deal with
@@ -267,8 +297,9 @@ SQLRETURN myodbc_do_connect(DBC *dbc, DataSource *ds)
         "this functionlaity", 0);
     }
 #endif
-    set_dbc_error(dbc, "HY000", mysql_error(mysql), mysql_errno(mysql));
-    translate_error(dbc->error.sqlstate, MYERR_S1000, mysql_errno(mysql));
+    set_dbc_error(dbc, "HY000", mysql_error(mysql), native_error);
+
+    translate_error(dbc->error.sqlstate, MYERR_S1000, native_error);
 
     return SQL_ERROR;
   }
@@ -316,7 +347,7 @@ SQLRETURN myodbc_do_connect(DBC *dbc, DataSource *ds)
     dbc->query_log= init_query_log();
 
   /* Set the statement error prefix based on the server version. */
-  strxmov(dbc->st_error_prefix, MYODBC3_ERROR_PREFIX, "[mysqld-",
+  strxmov(dbc->st_error_prefix, MYODBC_ERROR_PREFIX, "[mysqld-",
           mysql->server_version, "]", NullS);
 
   /* This needs to be set after connection, or it doesn't stick.  */
@@ -754,27 +785,6 @@ error:
 }
 
 
-/**
-  Discover and enumerate the attributes and attribute values required to
-  connect.
-
-  @return Always returns @c SQL_ERROR, because the driver does not support this.
-
-  @since ODBC 1.0
-*/
-SQLRETURN SQL_API
-SQLBrowseConnect(SQLHDBC hdbc,
-                 SQLCHAR FAR *szConnStrIn __attribute__((unused)),
-                 SQLSMALLINT cbConnStrIn __attribute__((unused)),
-                 SQLCHAR FAR *szConnStrOut __attribute__((unused)),
-                 SQLSMALLINT cbConnStrOutMax __attribute__((unused)),
-                 SQLSMALLINT FAR *pcbConnStrOut __attribute__((unused)))
-{
-  return set_conn_error(hdbc,MYERR_S1000,
-                        "Driver does not support this API", 0);
-}
-
-
 /**
   Disconnect a connection.
 
@@ -788,7 +798,7 @@ SQLBrowseConnect(SQLHDBC hdbc,
 SQLRETURN SQL_API SQLDisconnect(SQLHDBC hdbc)
 {
   LIST *list_element, *next_element;
-  DBC FAR *dbc= (DBC FAR*) hdbc;
+  DBC *dbc= (DBC *) hdbc;
 
   for (list_element= dbc->statements; list_element; list_element= next_element)
   {
