@@ -41,12 +41,25 @@
 #include "driver.h"
 #include <locale.h>
 
+
+/* Sets affected rows everewhere where SQLRowCOunt could look for */
+void global_set_affected_rows(STMT * stmt, my_ulonglong rows)
+{
+  stmt->affected_rows= stmt->dbc->mysql.affected_rows= rows;
+
+  /* Dirty hack. But not dirtier than the one above */
+  if (ssps_used(stmt))
+  {
+    stmt->ssps->affected_rows= rows;
+  }
+}
+
+
 /*
   @type    : myodbc3 internal
   @purpose : returns the table used by this query and
   ensures that all columns are from the same table
 */
-
 static const char *find_used_table(STMT *stmt)
 {
     MYSQL_FIELD  *field, *end;
@@ -107,61 +120,56 @@ static const char *find_used_table(STMT *stmt)
 */
 char *check_if_positioned_cursor_exists(STMT *pStmt, STMT **pStmtCursor)
 {
-  if (pStmt->query && pStmt->query_end)
-  {
-    const char *pszQueryTokenPos= pStmt->query_end;
-    const char *pszCursor= mystr_get_prev_token(pStmt->dbc->ansi_charset_info,
-                                                (const char**)&pszQueryTokenPos,
-                                                pStmt->query);
+  const char * cursorName= get_cursor_name(&pStmt->query);
 
-    if (!myodbc_casecmp(mystr_get_prev_token(pStmt->dbc->ansi_charset_info,
-                                             &pszQueryTokenPos,
-                                             pStmt->query),"OF",2) &&
-        !myodbc_casecmp(mystr_get_prev_token(pStmt->dbc->ansi_charset_info,
-                                             &pszQueryTokenPos,
-                                             pStmt->query),"CURRENT",7) &&
-        !myodbc_casecmp(mystr_get_prev_token(pStmt->dbc->ansi_charset_info,
-                                             &pszQueryTokenPos,
-                                             pStmt->query),"WHERE",5) )
+  if (cursorName != NULL)
+  {
+    
+    LIST *list_element;
+    DBC  *dbc= (DBC *)pStmt->dbc;
+    char * wherePos= get_token(&pStmt->query, TOKEN_COUNT(&pStmt->query)- 4);
+
+    if (wherePos > GET_QUERY(&pStmt->query))
     {
-      LIST *list_element;
-      DBC  *dbc= (DBC *)pStmt->dbc;
+      /* Decrementing if do not point to the beginning of the string to
+         point to a character(?) before "where"*/
+      --wherePos;
+    }
+
+    /*
+      Scan the list of statements for this connection and see if we
+      can find the cursor name this statement is referring to - it
+      must have a result set to count.
+    */
+    for (list_element= dbc->statements;
+         list_element;
+         list_element= list_element->next)
+    {
+      *pStmtCursor= (HSTMT)list_element->data;
 
       /*
-        Scan the list of statements for this connection and see if we
-        can find the cursor name this statement is referring to - it
-        must have a result set to count.
+        Even if the cursor name matches, the statement must have a
+        result set to count.
       */
-      for (list_element= dbc->statements;
-           list_element;
-           list_element= list_element->next)
+      if ((*pStmtCursor)->result &&
+          (*pStmtCursor)->cursor.name &&
+          !myodbc_strcasecmp((*pStmtCursor)->cursor.name,
+                             cursorName))
       {
-        *pStmtCursor= (HSTMT)list_element->data;
-
-        /*
-          Even if the cursor name matches, the statement must have a
-          result set to count.
-        */
-        if ((*pStmtCursor)->result &&
-            (*pStmtCursor)->cursor.name &&
-            !myodbc_strcasecmp((*pStmtCursor)->cursor.name,
-                               pszCursor))
-        {
-          return (char *)pszQueryTokenPos;
-        }
+        return (char *)wherePos;
       }
-
-      /* Did we run out of statements without finding a viable cursor? */
-      if (!list_element)
-      {
-        char buff[200];
-        strxmov(buff,"Cursor '", pszCursor,
-                "' does not exist or does not have a result set.", NullS);
-        set_stmt_error(pStmt, "34000", buff, ER_INVALID_CURSOR_NAME);
-      }
-
-      return (char *)pszQueryTokenPos;
     }
+
+    /* Did we run out of statements without finding a viable cursor? */
+    if (!list_element)
+    {
+      char buff[200];
+      strxmov(buff,"Cursor '", cursorName,
+              "' does not exist or does not have a result set.", NullS);
+      set_stmt_error(pStmt, "34000", buff, ER_INVALID_CURSOR_NAME);
+    }
+
+    return (char *)wherePos;
   }
 
   return NULL;
@@ -283,26 +291,41 @@ static my_bool check_if_usable_unique_key_exists(STMT *stmt)
   @purpose : positions the data cursor to appropriate row
 */
 
-void set_current_cursor_data(STMT FAR *stmt,SQLUINTEGER irow)
+void set_current_cursor_data(STMT *stmt, SQLUINTEGER irow)
 {
-    long       nrow, row_pos;
-    MYSQL_RES  *result= stmt->result;
-    MYSQL_ROWS *dcursor= result->data->data;
+  long       nrow, row_pos;
+  MYSQL_RES  *result= stmt->result;
 
-    /*
-      If irow exists, then position the current row to point
-      to the rowsetsize+irow, this is needed for positioned
-      calls
-    */
-    row_pos= irow ? (long) (stmt->current_row+irow-1) : stmt->current_row;
-    if ( stmt->cursor_row != row_pos )
+  
+  /*
+    If irow exists, then position the current row to point
+    to the rowsetsize+irow, this is needed for positioned
+    calls
+  */
+  row_pos= irow ? (long) (stmt->current_row+irow-1) : stmt->current_row;
+
+  if ( stmt->cursor_row != row_pos )
+  {
+    if (ssps_used(stmt))
     {
-        for ( nrow= 0; nrow < row_pos; ++nrow )
-            dcursor= dcursor->next;
-
-        stmt->cursor_row= row_pos;
-        result->data_cursor= dcursor;
+       data_seek(stmt, row_pos);
+       fetch_row(stmt);
     }
+    else
+    {
+      MYSQL_ROWS *dcursor;
+      dcursor= result->data->data;
+
+      for ( nrow= 0; nrow < row_pos; ++nrow )
+      {
+        dcursor= dcursor->next;
+      }
+      
+      result->data_cursor= dcursor;
+    }
+
+    stmt->cursor_row= row_pos;
+  }
 }
 
 
@@ -312,7 +335,7 @@ void set_current_cursor_data(STMT FAR *stmt,SQLUINTEGER irow)
   explicitly by the application
 */
 
-static void set_dynamic_cursor_name(STMT FAR *stmt)
+static void set_dynamic_cursor_name(STMT *stmt)
 {
     stmt->cursor.name= (char*) my_malloc(MYSQL_MAX_CURSOR_LEN,MYF(MY_ZEROFILL));
     sprintf((char*) stmt->cursor.name,"SQL_CUR%d",stmt->dbc->cursor_count++);
@@ -324,7 +347,7 @@ static void set_dynamic_cursor_name(STMT FAR *stmt)
   @purpose : updates the stmt status information
 */
 
-static SQLRETURN update_status(STMT FAR *stmt, SQLUSMALLINT status)
+static SQLRETURN update_status(STMT *stmt, SQLUSMALLINT status)
 {
     if ( stmt->affected_rows == 0 )
         return set_error(stmt,MYERR_01S03,NULL,0);
@@ -354,13 +377,15 @@ static SQLRETURN update_status(STMT FAR *stmt, SQLUSMALLINT status)
   @purpose : updates the SQLSetPos status information
 */
 
-static SQLRETURN update_setpos_status(STMT FAR *stmt, SQLINTEGER irow,
+static SQLRETURN update_setpos_status(STMT *stmt, SQLINTEGER irow,
                                       my_ulonglong rows, SQLUSMALLINT status)
 {
-  stmt->affected_rows= stmt->dbc->mysql.affected_rows= rows;
+  global_set_affected_rows(stmt, rows);
 
   if (irow && rows > 1)
-      return set_error(stmt,MYERR_01S04,NULL,0);
+  {
+    return set_error(stmt,MYERR_01S04,NULL,0);
+  }
 
   /*
     If all rows successful, then only update status..else
@@ -393,7 +418,7 @@ static SQLRETURN update_setpos_status(STMT FAR *stmt, SQLINTEGER irow,
   @purpose : copy row buffers to statement
 */
 
-static SQLRETURN copy_rowdata(STMT FAR *stmt, DESCREC *aprec,
+static SQLRETURN copy_rowdata(STMT *stmt, DESCREC *aprec,
                               DESCREC *iprec, NET **net, SQLCHAR **to)
 {
     SQLRETURN rc;
@@ -405,7 +430,7 @@ static SQLRETURN copy_rowdata(STMT FAR *stmt, DESCREC *aprec,
     if ( !(*to= (SQLCHAR *) extend_buffer(*net,(char*) *to,length)) )
         return set_error(stmt,MYERR_S1001,NULL,4001);
 
-    rc= insert_param(stmt, (char**) to, stmt->apd, aprec, iprec, 0);
+    rc= insert_param(stmt, (uchar*) to, stmt->apd, aprec, iprec, 0);
     if (!SQL_SUCCEEDED(rc))
         return rc;
 
@@ -425,10 +450,10 @@ static SQLRETURN copy_rowdata(STMT FAR *stmt, DESCREC *aprec,
   @purpose : executes a statement query
 */
 
-static SQLRETURN exec_stmt_query(STMT FAR *stmt,char *query,
+static SQLRETURN exec_stmt_query(STMT *stmt,char *query,
                                  SQLUINTEGER len)
 {
-    DBC FAR *dbc= stmt->dbc;
+    DBC *dbc= stmt->dbc;
     SQLRETURN error= SQL_SUCCESS;
 
     MYLOG_QUERY(stmt, query);
@@ -449,48 +474,61 @@ static SQLRETURN exec_stmt_query(STMT FAR *stmt,char *query,
   @purpose : copies field data to statement
 */
 
-static my_bool insert_field(STMT FAR *stmt, MYSQL_RES *result,
+static my_bool insert_field(STMT *stmt, MYSQL_RES *result,
                             DYNAMIC_STRING *dynQuery,
                             SQLUSMALLINT nSrcCol)
 {
-    DESCREC aprec_, iprec_;
-    DESCREC *aprec= &aprec_, *iprec= &iprec_;
-    MYSQL_FIELD *field= mysql_fetch_field_direct(result,nSrcCol);
-    MYSQL_ROW   row_data= result->data_cursor->data + nSrcCol;
-    NET         *net=&stmt->dbc->mysql.net;
-    unsigned char *to= net->buff;
-    SQLLEN      length;
+  DESCREC aprec_, iprec_;
+  DESCREC *aprec= &aprec_, *iprec= &iprec_;
+  MYSQL_FIELD *field= mysql_fetch_field_direct(result,nSrcCol);
+  MYSQL_ROW   row_data;
+  NET         *net=&stmt->dbc->mysql.net;
+  unsigned char *to= net->buff;
+  SQLLEN      length;
+  char as_string[50], *dummy;
 
-    desc_rec_init_apd(aprec);
-    desc_rec_init_ipd(iprec);
+  if (ssps_used(stmt))
+  {
+    dummy= get_string(stmt, nSrcCol, NULL, &length, as_string);
+    row_data= &dummy;
+  }
+  else
+  {
+    row_data= result->data_cursor->data + nSrcCol;
+  }
 
-    /* Copy row buffer data to statement */
-    iprec->concise_type= get_sql_data_type(stmt, field, 0);
-    aprec->concise_type= SQL_C_CHAR;
+  desc_rec_init_apd(aprec);
+  desc_rec_init_ipd(iprec);
 
-    if ( row_data && *row_data )
+  /* Copy row buffer data to statement */
+  iprec->concise_type= get_sql_data_type(stmt, field, 0);
+  aprec->concise_type= SQL_C_CHAR;
+
+  if (row_data && *row_data)
+  {
+    aprec->data_ptr= (SQLPOINTER) *row_data;
+    length= strlen(*row_data);
+
+    aprec->octet_length_ptr= &length;
+    aprec->indicator_ptr= &length;
+
+    if (!SQL_SUCCEEDED(insert_param(stmt, (uchar *) &to, stmt->apd,
+                                    aprec, iprec, 0)))
+      return 1;
+    if (!(to= (unsigned char *) add_to_buffer(net, (char *) to, " AND ", 5)))
     {
-        aprec->data_ptr= (SQLPOINTER) *row_data;
-        length= strlen(*row_data);
-
-        aprec->octet_length_ptr= &length;
-        aprec->indicator_ptr= &length;
-
-        if (!SQL_SUCCEEDED(insert_param(stmt, (char **) &to, stmt->apd,
-                                        aprec, iprec, 0)))
-          return 1;
-        if (!(to= (unsigned char *) add_to_buffer(net, (char *) to, " AND ", 5)))
-          return (my_bool)set_error(stmt, MYERR_S1001, NULL, 4001);
-
-        length= (uint) ((char *)to - (char*) net->buff);
-        dynstr_append_mem(dynQuery, (char*) net->buff, length);
+      return (my_bool)set_error(stmt, MYERR_S1001, NULL, 4001);
     }
-    else
-    {
-        --dynQuery->length;
-        dynstr_append_mem(dynQuery, " IS NULL AND ",13);
-    }
-    return 0;
+
+    length= (uint) ((char *)to - (char*) net->buff);
+    dynstr_append_mem(dynQuery, (char*) net->buff, length);
+  }
+  else
+  {
+    --dynQuery->length;
+    dynstr_append_mem(dynQuery, " IS NULL AND ",13);
+  }
+  return 0;
 }
 
 
@@ -500,7 +538,7 @@ static my_bool insert_field(STMT FAR *stmt, MYSQL_RES *result,
   if it is, copy that data to query, else we can't find the right row
 */
 
-static SQLRETURN insert_pk_fields(STMT FAR *stmt, DYNAMIC_STRING *dynQuery)
+static SQLRETURN insert_pk_fields(STMT *stmt, DYNAMIC_STRING *dynQuery)
 {
     MYSQL_RES    *result= stmt->result;
     MYSQL_FIELD  *field;
@@ -547,7 +585,7 @@ static SQLRETURN insert_pk_fields(STMT FAR *stmt, DYNAMIC_STRING *dynQuery)
   @purpose : generate a WHERE clause based on the fields in the result set
 */
 
-static SQLRETURN append_all_fields(STMT FAR *stmt, DYNAMIC_STRING *dynQuery)
+static SQLRETURN append_all_fields(STMT *stmt, DYNAMIC_STRING *dynQuery)
 {
   MYSQL_RES    *result= stmt->result;
   MYSQL_RES    *presultAllColumns;
@@ -651,7 +689,7 @@ static SQLRETURN append_all_fields(STMT FAR *stmt, DYNAMIC_STRING *dynQuery)
   @purpose : build the where clause
 */
 
-static SQLRETURN build_where_clause( STMT FAR *       pStmt, 
+static SQLRETURN build_where_clause( STMT * pStmt, 
                                      DYNAMIC_STRING * dynQuery,
                                      SQLUSMALLINT     irow )
 {
@@ -703,7 +741,7 @@ static SQLRETURN build_where_clause( STMT FAR *       pStmt,
   @purpose : set clause building..
 */
 
-static SQLRETURN build_set_clause(STMT FAR *stmt, SQLULEN irow,
+static SQLRETURN build_set_clause(STMT *stmt, SQLULEN irow,
                                   DYNAMIC_STRING *dynQuery)
 {
     DESCREC aprec_, iprec_;
@@ -780,8 +818,8 @@ static SQLRETURN build_set_clause(STMT FAR *stmt, SQLULEN irow,
         iprec->concise_type= get_sql_data_type(stmt, field, NULL);
         aprec->concise_type= arrec->concise_type;
 	/* copy prec and scale - needed for SQL_NUMERIC values */
-	iprec->precision= arrec->precision;
-	iprec->scale= arrec->scale;
+        iprec->precision= arrec->precision;
+        iprec->scale= arrec->scale;
         if (stmt->dae_type && aprec->par.is_dae)
           aprec->data_ptr= aprec->par.value;
         else
@@ -818,7 +856,7 @@ static SQLRETURN build_set_clause(STMT FAR *stmt, SQLULEN irow,
   @purpose : deletes the positioned cursor row
 */
 
-SQLRETURN my_pos_delete(STMT FAR *stmt, STMT FAR *stmtParam,
+SQLRETURN my_pos_delete(STMT *stmt, STMT *stmtParam,
                         SQLUSMALLINT irow, DYNAMIC_STRING *dynQuery)
 {
     SQLRETURN nReturn;
@@ -844,14 +882,14 @@ SQLRETURN my_pos_delete(STMT FAR *stmt, STMT FAR *stmtParam,
   @purpose : updates the positioned cursor row
 */
 
-SQLRETURN my_pos_update( STMT FAR *         pStmtCursor, 
-                         STMT FAR *         pStmt,
+SQLRETURN my_pos_update( STMT *             pStmtCursor, 
+                         STMT *             pStmt,
                          SQLUSMALLINT       nRow, 
                          DYNAMIC_STRING *   dynQuery )
 {
     SQLRETURN   rc;
     SQLHSTMT    hStmtTemp;
-    STMT FAR  * pStmtTemp;
+    STMT        * pStmtTemp;
 
     rc = build_where_clause( pStmtCursor, dynQuery, nRow );
     if ( !SQL_SUCCEEDED( rc ) )
@@ -866,7 +904,7 @@ SQLRETURN my_pos_update( STMT FAR *         pStmtCursor,
         return set_stmt_error( pStmt, "HY000", "my_SQLAllocStmt() failed.", 0 );
     }
 
-    pStmtTemp = (STMT FAR*)hStmtTemp;
+    pStmtTemp = (STMT *)hStmtTemp;
 
     if (my_SQLPrepare(pStmtTemp, (SQLCHAR *)dynQuery->str, dynQuery->length,
                       FALSE) != SQL_SUCCESS)
@@ -895,6 +933,7 @@ SQLRETURN my_pos_update( STMT FAR *         pStmtCursor,
       /*
         Re-prepare the statement, which will leave us with a prepared
         statement that is a non-positioned update.
+        To check: do we really need that?
       */
       if (my_SQLPrepare(pStmt, (SQLCHAR *)dynQuery->str, dynQuery->length,
                         FALSE) != SQL_SUCCESS)
@@ -913,129 +952,143 @@ SQLRETURN my_pos_update( STMT FAR *         pStmtCursor,
   @purpose : deletes the positioned cursor row - will del all rows in rowset if irow = 0
 */
 
-static SQLRETURN setpos_delete(STMT FAR *stmt, SQLUSMALLINT irow,
+static SQLRETURN setpos_delete(STMT *stmt, SQLUSMALLINT irow,
                                DYNAMIC_STRING *dynQuery)
 {
-    SQLUINTEGER  rowset_pos,rowset_end;
-    my_ulonglong affected_rows= 0;
-    SQLRETURN    nReturn= SQL_SUCCESS;
-    ulong        query_length;
-    const char   *table_name;
+  SQLUINTEGER  rowset_pos,rowset_end;
+  my_ulonglong affected_rows= 0;
+  SQLRETURN    nReturn= SQL_SUCCESS;
+  ulong        query_length;
+  const char   *table_name;
 
-    /* we want to work with base table name - we expect call to fail if more than one base table involved */
-    if ( !(table_name= find_used_table(stmt)) )
-        return SQL_ERROR;
+  /* we want to work with base table name - we expect call to fail if more than one base table involved */
+  if (!(table_name= find_used_table(stmt)))
+  {
+    return SQL_ERROR;
+  }
 
-    /* appened our table name to our DELETE statement */
-    dynstr_append_quoted_name(dynQuery,table_name);
-    query_length= dynQuery->length;
+  /* appened our table name to our DELETE statement */
+  dynstr_append_quoted_name(dynQuery,table_name);
+  query_length= dynQuery->length;
 
-    /* IF irow == 0 THEN delete all rows in the current rowset ELSE specific (as in one) row */
-    if ( irow == 0 )
+  /* IF irow == 0 THEN delete all rows in the current rowset ELSE specific (as in one) row */
+  if ( irow == 0 )
+  {
+    rowset_pos= 1;
+    rowset_end= stmt->rows_found_in_set;
+  }
+  else
+  {
+    rowset_pos= rowset_end= irow;
+  }
+
+  /* process all desired rows in the rowset - we assume rowset_pos is valid */
+  do
+  {
+    dynQuery->length= query_length;
+
+    /* append our WHERE clause to our DELETE statement */
+    nReturn = build_where_clause( stmt, dynQuery, (SQLUSMALLINT)rowset_pos );
+    if (!SQL_SUCCEEDED( nReturn ))
     {
-        rowset_pos= 1;
-        rowset_end= stmt->rows_found_in_set;
+      return nReturn;
     }
-    else
-        rowset_pos= rowset_end= irow;
 
-    /* process all desired rows in the rowset - we assume rowset_pos is valid */
-    do
+    /* execute our DELETE statement */
+    if ( !(nReturn= exec_stmt_query(stmt, dynQuery->str, dynQuery->length)) )
     {
-        dynQuery->length= query_length;
+      affected_rows+= stmt->dbc->mysql.affected_rows;
+    }
 
-        /* append our WHERE clause to our DELETE statement */
-        nReturn = build_where_clause( stmt, dynQuery, (SQLUSMALLINT)rowset_pos );
-        if ( !SQL_SUCCEEDED( nReturn ) )
-            return nReturn;
+  } while ( ++rowset_pos <= rowset_end );
 
-        /* execute our DELETE statement */
-        if ( !(nReturn= exec_stmt_query(stmt, dynQuery->str, dynQuery->length)) )
-            affected_rows+= stmt->dbc->mysql.affected_rows;
+  if (nReturn == SQL_SUCCESS)
+  {
+    nReturn= update_setpos_status(stmt, irow, affected_rows, SQL_ROW_DELETED);
+  }
 
-    } while ( ++rowset_pos <= rowset_end );
+  /* fix-up so fetching next rowset is correct */
+  if (if_dynamic_cursor(stmt))
+  {
+    stmt->rows_found_in_set-= (uint) affected_rows;
+  }
 
-    if ( nReturn == SQL_SUCCESS )
-        nReturn= update_setpos_status(stmt,irow,affected_rows,SQL_ROW_DELETED);
-
-    /* fix-up so fetching next rowset is correct */
-    if (if_dynamic_cursor(stmt))
-      stmt->rows_found_in_set-= (uint) affected_rows;
-
-    return nReturn;
+  return nReturn;
 }
 
 
 /*
-  @type    : myodbc3 internal
-  @purpose : updates the positioned cursor row.
+@type    : myodbc3 internal
+@purpose : updates the positioned cursor row.
 */
 
-static SQLRETURN setpos_update(STMT FAR *stmt, SQLUSMALLINT irow,
-                               DYNAMIC_STRING *dynQuery)
+static SQLRETURN setpos_update(STMT *stmt, SQLUSMALLINT irow,
+                             DYNAMIC_STRING *dynQuery)
 {
-    SQLUINTEGER  rowset_pos,rowset_end;
-    my_ulonglong affected_rows= 0;
-    SQLRETURN    nReturn= SQL_SUCCESS;
-    ulong        query_length;
-    const char   *table_name;
+  SQLUINTEGER  rowset_pos,rowset_end;
+  my_ulonglong affected= 0;
+  SQLRETURN    nReturn= SQL_SUCCESS;
+  ulong        query_length;
+  const char   *table_name;
 
-    if ( !(table_name= find_used_table(stmt)) )
+  if ( !(table_name= find_used_table(stmt)) )
+      return SQL_ERROR;
+
+  dynstr_append_quoted_name(dynQuery,table_name);
+  query_length= dynQuery->length;
+
+  if ( !irow )
+  {
+      /*
+        If irow == 0, then update all rows in the current rowset
+      */
+      rowset_pos= 1;
+      rowset_end= stmt->rows_found_in_set;
+  }
+  else
+      rowset_pos= rowset_end= irow;
+
+  do /* UPDATE, irow from current row set */
+  {
+      dynQuery->length= query_length;
+      nReturn= build_set_clause(stmt,rowset_pos,dynQuery);
+      if (nReturn == ER_ALL_COLUMNS_IGNORED)
+      {
+        /*
+          If we're updating more than one row, having all columns ignored
+          is fine. If it's just one row, that's an error.
+        */
+        if (!irow)
+        {
+          nReturn= SQL_SUCCESS;
+          continue;
+        }
+        else
+        {
+          set_stmt_error(stmt, "21S02",
+                         "Degree of derived table does not match column list",
+                         0);
+          return SQL_ERROR;
+        }
+      }
+      else if (nReturn == SQL_ERROR)
         return SQL_ERROR;
 
-    dynstr_append_quoted_name(dynQuery,table_name);
-    query_length= dynQuery->length;
+      nReturn= build_where_clause(stmt, dynQuery, (SQLUSMALLINT)rowset_pos);
+      if (!SQL_SUCCEEDED(nReturn))
+        return nReturn;
 
-    if ( !irow )
-    {
-        /*
-          If irow == 0, then update all rows in the current rowset
-        */
-        rowset_pos= 1;
-        rowset_end= stmt->rows_found_in_set;
-    }
-    else
-        rowset_pos= rowset_end= irow;
+      if ( !(nReturn= exec_stmt_query(stmt, dynQuery->str, dynQuery->length)) )
+      {
+        affected+= mysql_affected_rows(&stmt->dbc->mysql);
+      }
 
-    do /* UPDATE, irow from current row set */
-    {
-        dynQuery->length= query_length;
-        nReturn= build_set_clause(stmt,rowset_pos,dynQuery);
-        if (nReturn == ER_ALL_COLUMNS_IGNORED)
-        {
-          /*
-            If we're updating more than one row, having all columns ignored
-            is fine. If it's just one row, that's an error.
-          */
-          if (!irow)
-          {
-            nReturn= SQL_SUCCESS;
-            continue;
-          }
-          else
-          {
-            set_stmt_error(stmt, "21S02",
-                           "Degree of derived table does not match column list",
-                           0);
-            return SQL_ERROR;
-          }
-        }
-        else if (nReturn == SQL_ERROR)
-          return SQL_ERROR;
+  } while ( ++rowset_pos <= rowset_end );
 
-        nReturn= build_where_clause(stmt, dynQuery, (SQLUSMALLINT)rowset_pos);
-        if (!SQL_SUCCEEDED(nReturn))
-          return nReturn;
+  if (nReturn == SQL_SUCCESS)
+      nReturn= update_setpos_status(stmt, irow, affected, SQL_ROW_UPDATED);
 
-        if ( !(nReturn= exec_stmt_query(stmt, dynQuery->str, dynQuery->length)) )
-            affected_rows+= stmt->dbc->mysql.affected_rows;
-
-    } while ( ++rowset_pos <= rowset_end );
-
-    if ( nReturn == SQL_SUCCESS )
-        nReturn= update_setpos_status(stmt,irow,affected_rows,SQL_ROW_UPDATED);
-
-    return nReturn;
+  return nReturn;
 }
 
 
@@ -1061,7 +1114,7 @@ static SQLRETURN setpos_update(STMT FAR *stmt, SQLUSMALLINT irow,
     \retval SQL_SUCCESS     Success!
 */
 
-static SQLRETURN batch_insert( STMT FAR *stmt, SQLULEN irow, DYNAMIC_STRING *ext_query )
+static SQLRETURN batch_insert( STMT *stmt, SQLULEN irow, DYNAMIC_STRING *ext_query )
 {
     MYSQL_RES    *result= stmt->result;     /* result set we are working with */
     SQLULEN      insert_count= 1;           /* num rows to insert - will be real value when row is 0 (all)  */
@@ -1195,8 +1248,7 @@ static SQLRETURN batch_insert( STMT FAR *stmt, SQLULEN irow, DYNAMIC_STRING *ext
 
     } while ( break_insert && count < insert_count );
 
-    /* get rows affected count */
-    stmt->affected_rows= stmt->dbc->mysql.affected_rows= insert_count;
+    global_set_affected_rows(stmt, insert_count);
 
     /* update row status pointer(s) */
     if (stmt->ird->array_status_ptr)
@@ -1279,7 +1331,7 @@ static const char *alloc_error= "Driver Failed to set the internal dynamic resul
 SQLRETURN SQL_API my_SQLSetPos(SQLHSTMT hstmt, SQLSETPOSIROW irow,
                                SQLUSMALLINT fOption, SQLUSMALLINT fLock)
 {
-    STMT FAR  *stmt= (STMT FAR*) hstmt;
+    STMT      *stmt= (STMT *) hstmt;
     SQLRETURN sqlRet= SQL_SUCCESS;
     MYSQL_RES *result= stmt->result;
     SQLRETURN rc;
@@ -1314,7 +1366,7 @@ SQLRETURN SQL_API my_SQLSetPos(SQLHSTMT hstmt, SQLSETPOSIROW irow,
 
     /* If irow > maximum rows in the resultset. for forwrd only row_count is 0
      */
-    if ( fOption != SQL_ADD && irow > result->row_count )
+    if ( fOption != SQL_ADD && irow > num_rows(stmt))
         return set_error(stmt,MYERR_S1107,NULL,0);
 
     /* Not a valid lock type ..*/
@@ -1341,8 +1393,8 @@ SQLRETURN SQL_API my_SQLSetPos(SQLHSTMT hstmt, SQLSETPOSIROW irow,
                 --irow;
                 sqlRet= SQL_SUCCESS;
                 stmt->cursor_row= (long)(stmt->current_row+irow);
-                mysql_data_seek(stmt->result,(my_ulonglong)stmt->cursor_row);
-                stmt->current_values= mysql_fetch_row(stmt->result);
+                data_seek(stmt, (my_ulonglong)stmt->cursor_row);
+                stmt->current_values= fetch_row(stmt);
                 reset_getdata_position(stmt);
                 if ( stmt->fix_fields )
                     stmt->current_values= (*stmt->fix_fields)(stmt,stmt->current_values);
@@ -1351,7 +1403,7 @@ SQLRETURN SQL_API my_SQLSetPos(SQLHSTMT hstmt, SQLSETPOSIROW irow,
                  cursor, but we don't want that. We seek back to this row
                  so the MYSQL_RES is in the state we expect.
                 */
-                mysql_data_seek(stmt->result,(my_ulonglong)stmt->cursor_row);
+                data_seek(stmt, (my_ulonglong)stmt->cursor_row);
                 pthread_mutex_unlock(&stmt->dbc->lock);
                 break;
             }
