@@ -60,9 +60,9 @@ void myodbc_thread_key_create()
        its list
 */
 
-SQLRETURN SQL_API my_SQLAllocEnv(SQLHENV FAR *phenv)
+SQLRETURN SQL_API my_SQLAllocEnv(SQLHENV *phenv)
 {
-  ENV FAR **env= (ENV FAR**) phenv;
+  ENV **env= (ENV **) phenv;
 #ifdef _UNIX_
   /* Init thread key just once for all threads */
   pthread_once(&myodbc_thread_key_inited, myodbc_thread_key_create);
@@ -96,7 +96,7 @@ SQLRETURN SQL_API my_SQLAllocEnv(SQLHENV FAR *phenv)
   @purpose : to allocate the environment handle
 */
 
-SQLRETURN SQL_API SQLAllocEnv(SQLHENV FAR *phenv)
+SQLRETURN SQL_API SQLAllocEnv(SQLHENV *phenv)
 {
   SQLRETURN rc;
 
@@ -105,9 +105,9 @@ SQLRETURN SQL_API SQLAllocEnv(SQLHENV FAR *phenv)
   {
 /* --- if OS=WIN32, set default env option for SQL_ATTR_ODBC_VERSION */
 #ifdef WIN32
-    ((ENV FAR*) *phenv)->odbc_ver= SQL_OV_ODBC3;
+    ((ENV *) *phenv)->odbc_ver= SQL_OV_ODBC3;
 #else
-    ((ENV FAR*) *phenv)->odbc_ver= SQL_OV_ODBC2;
+    ((ENV *) *phenv)->odbc_ver= SQL_OV_ODBC2;
 #endif /* WIN32 */
   }
 
@@ -122,7 +122,7 @@ SQLRETURN SQL_API SQLAllocEnv(SQLHENV FAR *phenv)
 
 SQLRETURN SQL_API my_SQLFreeEnv(SQLHENV henv)
 {
-    ENV FAR *env= (ENV FAR*) henv;
+    ENV *env= (ENV *) henv;
     pthread_mutex_destroy(&env->lock);
 #ifndef _UNIX_
     GlobalUnlock(GlobalHandle((HGLOBAL) henv));
@@ -171,10 +171,10 @@ SQLRETURN my_GetLastError(ENV *henv)
        maintain the connection list
 */
 
-SQLRETURN SQL_API my_SQLAllocConnect(SQLHENV henv, SQLHDBC FAR *phdbc)
+SQLRETURN SQL_API my_SQLAllocConnect(SQLHENV henv, SQLHDBC *phdbc)
 {
-    DBC FAR *dbc;
-    ENV FAR *penv= (ENV FAR*) henv;
+    DBC *dbc;
+    ENV *penv= (ENV *) henv;
 
 #ifdef _UNIX_
     long *thread_count;
@@ -239,7 +239,7 @@ SQLRETURN SQL_API my_SQLAllocConnect(SQLHENV henv, SQLHDBC FAR *phdbc)
 /* penv->odbc_ver= SQL_OV_ODBC3; */
 #endif /* WIN32 */
 
-    dbc= (DBC FAR*) *phdbc;
+    dbc= (DBC *) *phdbc;
     dbc->mysql.net.vio= 0;     /* Marker if open */
     dbc->commit_flag= 0;
     dbc->stmt_options.max_rows= dbc->stmt_options.max_length= 0L;
@@ -270,7 +270,7 @@ SQLRETURN SQL_API my_SQLAllocConnect(SQLHENV henv, SQLHDBC FAR *phdbc)
        maintain the connection list
 */
 
-SQLRETURN SQL_API SQLAllocConnect(SQLHENV henv, SQLHDBC FAR *phdbc)
+SQLRETURN SQL_API SQLAllocConnect(SQLHENV henv, SQLHDBC *phdbc)
 {
     return my_SQLAllocConnect(henv, phdbc);
 }
@@ -284,7 +284,7 @@ SQLRETURN SQL_API SQLAllocConnect(SQLHENV henv, SQLHDBC FAR *phdbc)
 
 SQLRETURN SQL_API my_SQLFreeConnect(SQLHDBC hdbc)
 {
-    DBC FAR *dbc= (DBC FAR*) hdbc;
+    DBC *dbc= (DBC *) hdbc;
     LIST *ldesc;
     LIST *next;
 
@@ -348,19 +348,59 @@ SQLRETURN SQL_API SQLFreeConnect(SQLHDBC hdbc)
     return my_SQLFreeConnect(hdbc);
 }
 
+/* allocates dynamic array for param bind.
+   returns TRUE on allocation errors */
+BOOL allocate_param_bind(DYNAMIC_ARRAY **param_bind, uint elements)
+{
+  if (*param_bind == NULL)
+  {
+    *param_bind= my_malloc(sizeof(DYNAMIC_ARRAY), MYF(0));
+
+    if (*param_bind == NULL)
+    {
+      return TRUE;
+    }
+  }
+
+  my_init_dynamic_array(*param_bind, sizeof(MYSQL_BIND), elements, 10);
+  memset((*param_bind)->buffer, 0, sizeof(MYSQL_BIND) *
+											(*param_bind)->max_element);
+
+  return FALSE;
+}
+
+
+void delete_param_bind(DYNAMIC_ARRAY *param_bind)
+{
+  if (param_bind != NULL)
+  {
+    uint i;
+    for (i=0; i < param_bind->max_element; ++i)
+    {
+      MYSQL_BIND *bind= (MYSQL_BIND *)param_bind->buffer + i;
+
+      if (bind != NULL)
+      {
+        x_free(bind->buffer);
+      }
+    }
+    delete_dynamic(param_bind);
+    x_free(param_bind);
+  }
+}
+
 
 /*
   @type    : myodbc3 internal
   @purpose : allocates the statement handle
 */
-
-SQLRETURN SQL_API my_SQLAllocStmt(SQLHDBC hdbc,SQLHSTMT FAR *phstmt)
+SQLRETURN SQL_API my_SQLAllocStmt(SQLHDBC hdbc,SQLHSTMT *phstmt)
 {
 #ifndef _UNIX_
     HGLOBAL  hstmt;
 #endif
-    STMT FAR *stmt;
-    DBC FAR *dbc= (DBC FAR*) hdbc;
+    STMT  *stmt;
+    DBC   *dbc= (DBC*) hdbc;
 
 #ifndef _UNIX_
     hstmt= GlobalAlloc(GMEM_MOVEABLE | GMEM_ZEROINIT, sizeof(STMT));
@@ -375,9 +415,8 @@ SQLRETURN SQL_API my_SQLAllocStmt(SQLHDBC hdbc,SQLHSTMT FAR *phstmt)
     if (*phstmt == SQL_NULL_HSTMT)
         goto error;
 
-    stmt= (STMT FAR*) *phstmt;
+    stmt= (STMT *) *phstmt;
     stmt->dbc= dbc;
-    stmt->ssps= NULL;;
 
     pthread_mutex_lock(&stmt->dbc->lock);
     dbc->statements= list_add(dbc->statements,&stmt->list);
@@ -387,7 +426,13 @@ SQLRETURN SQL_API my_SQLAllocStmt(SQLHDBC hdbc,SQLHSTMT FAR *phstmt)
     stmt->state= ST_UNKNOWN;
     stmt->dummy_state= ST_DUMMY_UNKNOWN;
     strmov(stmt->error.sqlstate, "00000");
-    my_init_dynamic_array(&stmt->param_pos, sizeof(char *), 0, 0);
+    init_parsed_query(&stmt->query);
+    init_parsed_query(&stmt->orig_query);
+
+    if (!dbc->ds->no_ssps && allocate_param_bind(&stmt->param_bind, 10))
+    {
+      goto error;
+    }
 
     if (!(stmt->ard= desc_alloc(stmt, SQL_DESC_ALLOC_AUTO,
                                 DESC_APP, DESC_ROW)))
@@ -405,11 +450,16 @@ SQLRETURN SQL_API my_SQLAllocStmt(SQLHDBC hdbc,SQLHSTMT FAR *phstmt)
     stmt->imp_apd= stmt->apd;
 
     return SQL_SUCCESS;
+
 error:
     x_free(stmt->ard);
     x_free(stmt->ird);
     x_free(stmt->apd);
     x_free(stmt->ipd);
+    delete_parsed_query(&stmt->query);
+    delete_parsed_query(&stmt->orig_query);
+    delete_param_bind(stmt->param_bind);
+
     return set_dbc_error(dbc, "HY001", "Memory allocation error", MYERR_S1001);
 }
 
@@ -419,7 +469,7 @@ error:
   @purpose : allocates the statement handle
 */
 
-SQLRETURN SQL_API SQLAllocStmt(SQLHDBC hdbc,SQLHSTMT FAR *phstmt)
+SQLRETURN SQL_API SQLAllocStmt(SQLHDBC hdbc,SQLHSTMT *phstmt)
 {
     return my_SQLAllocStmt(hdbc,phstmt);
 }
@@ -463,7 +513,7 @@ SQLRETURN SQL_API my_SQLFreeStmt(SQLHSTMT hstmt,SQLUSMALLINT fOption)
 SQLRETURN SQL_API my_SQLFreeStmtExtended(SQLHSTMT hstmt,SQLUSMALLINT fOption,
                                          uint clearAllResults)
 {
-    STMT FAR *stmt= (STMT FAR*) hstmt;
+    STMT *stmt= (STMT *) hstmt;
     uint i;
 
     if (fOption == SQL_UNBIND)
@@ -473,6 +523,8 @@ SQLRETURN SQL_API my_SQLFreeStmtExtended(SQLHSTMT hstmt,SQLUSMALLINT fOption,
       return SQL_SUCCESS;
     }
 
+    stmt->out_params_state= 0;
+
     desc_free_paramdata(stmt->apd);
     /* reset data-at-exec state */
     stmt->dae_type= 0;
@@ -481,11 +533,14 @@ SQLRETURN SQL_API my_SQLFreeStmtExtended(SQLHSTMT hstmt,SQLUSMALLINT fOption,
 
     if (fOption == SQL_RESET_PARAMS)
     {
-      /* with current use of ps we need to close, if we prepare on server, we will have
-         to change this to mysql_stmt_reset
-         http://dev.mysql.com/doc/refman/5.5/en/mysql-stmt-reset.html
-       */
-      ssps_close(stmt);
+      if (stmt->param_bind != NULL)
+      {
+        reset_dynamic(stmt->param_bind);
+      }
+      if (ssps_used(stmt))
+      {
+        mysql_stmt_reset(stmt->ssps);
+      }
       /* remove all params and reset count to 0 (per spec) */
       /* http://msdn2.microsoft.com/en-us/library/ms709284.aspx */
       stmt->apd->count= 0;
@@ -494,18 +549,13 @@ SQLRETURN SQL_API my_SQLFreeStmtExtended(SQLHSTMT hstmt,SQLUSMALLINT fOption,
 
     if (!stmt->fake_result)
     {
-      free_current_result(stmt);
-      /* check if there are more resultsets */
       if (clearAllResults)
       {
-        /* For ps _close() will free all pending results */
-        while (mysql_more_results(&stmt->dbc->mysql))
+        /* We seiously CLOSEing statement for preparing handle object for
+           new query */
+        while (!next_result(stmt))
         {
-          if (!mysql_next_result(&stmt->dbc->mysql))
-          {
-            stmt->result= mysql_store_result(&stmt->dbc->mysql);
-            mysql_free_result(stmt->result);
-          }
+          get_result_metadata(stmt, TRUE);
         }
       }
     }
@@ -515,29 +565,33 @@ SQLRETURN SQL_API my_SQLFreeStmtExtended(SQLHSTMT hstmt,SQLUSMALLINT fOption,
       {
         free_root(&stmt->result->field_alloc, MYF(0));
       }
-
       x_free(stmt->result);
     }
 
     x_free(stmt->fields);
-    x_free(stmt->array);
     x_free(stmt->result_array);
     x_free(stmt->lengths);
     stmt->result= 0;
     stmt->fake_result= 0;
     stmt->fields= 0;
-    stmt->array= 0;
     stmt->result_array= 0;
     stmt->lengths= 0;
     stmt->current_values= 0;   /* For SQLGetData */
     stmt->fix_fields= 0;
     stmt->affected_rows= 0;
-    stmt->current_row= stmt->cursor_row= stmt->rows_found_in_set= 0;
+    stmt->current_row= stmt->rows_found_in_set= 0;
+    stmt->cursor_row= -1;
     stmt->dae_type= 0;
     stmt->ird->count= 0;
 
     if (fOption == MYSQL_RESET_BUFFERS)
-        return SQL_SUCCESS;
+    {
+      free_result_bind(stmt);
+      x_free(stmt->array);
+      stmt->array= 0;
+
+      return SQL_SUCCESS;
+    }
 
     stmt->state= ST_UNKNOWN;
 
@@ -557,15 +611,25 @@ SQLRETURN SQL_API my_SQLFreeStmtExtended(SQLHSTMT hstmt,SQLUSMALLINT fOption,
     }
     stmt->cursor.pk_count= 0;
 
-    ssps_close(stmt);
+    if (clearAllResults)
+    {
+      x_free(stmt->array);
+      stmt->array= 0;
+      ssps_close(stmt);
+    }
 
     if (fOption == SQL_CLOSE)
         return SQL_SUCCESS;
 
     /* At this point, only MYSQL_RESET and SQL_DROP left out */
-    x_free(stmt->query);
-    x_free(stmt->orig_query);
-    stmt->query= stmt->orig_query= 0;
+    reset_parsed_query(&stmt->orig_query, NULL, NULL, NULL);
+    reset_parsed_query(&stmt->query, NULL, NULL, NULL);
+
+    if (stmt->param_bind != NULL)
+    {
+      reset_dynamic(stmt->param_bind);
+    }
+
     stmt->param_count= 0;
 
     reset_ptr(stmt->apd->rows_processed_ptr);
@@ -591,7 +655,10 @@ SQLRETURN SQL_API my_SQLFreeStmtExtended(SQLHSTMT hstmt,SQLUSMALLINT fOption,
 
     x_free(stmt->cursor.name);
 
-    delete_dynamic(&stmt->param_pos);
+    delete_parsed_query(&stmt->query);
+    delete_parsed_query(&stmt->orig_query);
+    delete_param_bind(stmt->param_bind);
+
     pthread_mutex_lock(&stmt->dbc->lock);
     stmt->dbc->statements= list_delete(stmt->dbc->statements,&stmt->list);
     pthread_mutex_unlock(&stmt->dbc->lock);
@@ -618,7 +685,6 @@ SQLRETURN my_SQLAllocDesc(SQLHDBC hdbc, SQLHANDLE *pdesc)
     return set_dbc_error(dbc, "HY001", "Memory allocation error", MYERR_S1001);
 
   desc->exp.dbc= dbc;
-  pthread_mutex_init(&desc->lock, NULL);
 
   /* add to this connection's list of explicit descriptors */
   e= (LIST *) my_malloc(sizeof(LIST), MYF(0));
@@ -675,7 +741,6 @@ SQLRETURN my_SQLFreeDesc(SQLHANDLE hdesc)
     x_free(lstmt);
   }
 
-  pthread_mutex_destroy(&desc->lock);
   desc_free(desc);
   return SQL_SUCCESS;
 }
@@ -756,4 +821,3 @@ SQLRETURN SQL_API SQLFreeHandle(SQLSMALLINT HandleType,
     return error;
 }
 
-
