@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2000, 2011, Oracle and/or its affiliates. All rights reserved.
+  Copyright (c) 2000, 2013, Oracle and/or its affiliates. All rights reserved.
 
   The MySQL Connector/ODBC is licensed under the terms of the GPLv2
   <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
@@ -244,12 +244,19 @@ MySQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT fInfoType,
     MYINFO_SET_STR("N");
 
   case SQL_DRIVER_NAME:
-#ifdef WIN32
-    MYINFO_SET_STR("myodbc5.dll");
+#ifdef MYODBC_UNICODEDRIVER
+# ifdef WIN32
+    MYINFO_SET_STR("myodbc5w.dll");
+# else
+    MYINFO_SET_STR("libmyodbc5w.so");
+# endif
 #else
-    MYINFO_SET_STR("libmyodbc5.so");
+# ifdef WIN32
+    MYINFO_SET_STR("myodbc5a.dll");
+# else
+    MYINFO_SET_STR("libmyodbc5a.so");
+# endif
 #endif
-
   case SQL_DRIVER_ODBC_VER:
     MYINFO_SET_STR("03.51");               /* What standard we implement */
 
