@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2012, Oracle and/or its affiliates. All rights reserved.
+  Copyright (c) 2012-2013, Oracle and/or its affiliates. All rights reserved.
 
   The MySQL Connector/ODBC is licensed under the terms of the GPLv2
   <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
@@ -58,33 +58,182 @@ static char * my_f_to_a(char * buf, size_t buf_size, double a)
 /* }}} */
 
 
+/* {{{ ssps_init() -I- */
 void ssps_init(STMT *stmt)
 {
-  
   stmt->ssps= mysql_stmt_init(&stmt->dbc->mysql);
 
-  if (stmt->dbc->ds->cursor_prefetch_number > 0)
-  {
-    unsigned long type= (unsigned long) CURSOR_TYPE_READ_ONLY;
-
-    /* TODO check return value */
-    mysql_stmt_attr_set(stmt->ssps, STMT_ATTR_CURSOR_TYPE, (void*) &type);
-    mysql_stmt_attr_set(stmt->ssps, STMT_ATTR_PREFETCH_ROWS,
-                      (void*) &stmt->dbc->ds->cursor_prefetch_number);
-  }
-
   stmt->result_bind= 0;
 }
+/* }}} */
 
 
-static void free_result_bind(STMT *stmt)
+char * numeric2binary(char * dst, long long src, unsigned int byte_count)
+{
+  char byte;
+
+  while (byte_count)
+  {
+    byte= src & 0xff;
+    *(dst+(--byte_count))= byte;
+    src= src >> 8;
+  }
+
+  return dst;
+}
+
+
+SQLRETURN SQL_API
+sql_get_data(STMT *stmt, SQLSMALLINT fCType, uint column_number,
+             SQLPOINTER rgbValue, SQLLEN cbValueMax, SQLLEN *pcbValue,
+             char *value, ulong length, DESCREC *arrec);
+
+/**
+  @returns TRUE if the resultset is SP OUT params
+  Basically it makes sense with prepared statements only
+  */
+BOOL ssps_get_out_params(STMT *stmt)
+{
+    /* If we use prepared statement, and the query is CALL and we have any
+     user's parameter described as INOUT or OUT and that is only result */
+  if (is_call_procedure(&stmt->query))
+  {
+    MYSQL_ROW values= NULL;
+    DESCREC   *iprec, *aprec;
+    uint      counter= 0;
+    int       i;
+
+    /*Since OUT parameters can be completely different - we have to free current
+      bind and bind new */
+
+    free_result_bind(stmt);
+    /* Thus function interface has to be changed */
+    if (ssps_bind_result(stmt) == 0)
+    {
+      values= fetch_row(stmt);
+
+      if (stmt->fix_fields)
+      {
+        values= (*stmt->fix_fields)(stmt,values);
+      }
+    }
+
+    assert(values);
+
+    /* Then current result is out params */
+    stmt->out_params_state= 2;
+
+    if (values != NULL && got_out_parameters(stmt))
+    {
+      for (i= 0; i < myodbc_min(stmt->ipd->count, stmt->apd->count); ++i)
+      {
+        /* Making bit field look "normally" */
+        if (stmt->result_bind[counter].buffer_type == MYSQL_TYPE_BIT)
+        {
+          MYSQL_FIELD *field= mysql_fetch_field_direct(stmt->result, counter);
+          unsigned long long numeric;
+
+          assert(field->type == MYSQL_TYPE_BIT);
+          /* terminating with NULL */
+          values[counter][*stmt->result_bind[counter].length]= '\0';
+          numeric= strtoull(values[counter], NULL, 10);
+
+          *stmt->result_bind[counter].length= (field->length+7)/8;
+          numeric2binary(values[counter], numeric,
+                        *stmt->result_bind[counter].length);
+
+        }
+
+        iprec= desc_get_rec(stmt->ipd, i, FALSE);
+        aprec= desc_get_rec(stmt->apd, i, FALSE);
+        assert(iprec && aprec);
+
+        if ((iprec->parameter_type == SQL_PARAM_INPUT_OUTPUT
+        || iprec->parameter_type == SQL_PARAM_OUTPUT))
+        {
+          if (aprec->data_ptr)
+          {
+            unsigned long length= *stmt->result_bind[counter].length;
+            char *target= NULL;
+            SQLLEN *octet_length_ptr= NULL;
+            SQLLEN *indicator_ptr= NULL;
+            SQLINTEGER default_size;
+
+            if (aprec->octet_length_ptr)
+            {
+              octet_length_ptr= ptr_offset_adjust(aprec->octet_length_ptr,
+                                            stmt->apd->bind_offset_ptr,
+                                            stmt->apd->bind_type,
+                                            sizeof(SQLLEN), 0);
+            }
+
+            indicator_ptr= ptr_offset_adjust(aprec->indicator_ptr,
+                                         stmt->apd->bind_offset_ptr,
+                                         stmt->apd->bind_type,
+                                         sizeof(SQLLEN), 0);
+
+            default_size= bind_length(aprec->concise_type,
+                                      aprec->octet_length);
+            target= ptr_offset_adjust(aprec->data_ptr, stmt->apd->bind_offset_ptr,
+                                  stmt->apd->bind_type, default_size, 0);
+
+            reset_getdata_position(stmt);
+
+            sql_get_data(stmt, aprec->concise_type, counter,
+                         target, aprec->octet_length, indicator_ptr,
+                         values[counter], length, aprec);
+
+            /* TODO: solve that globally */
+            if (octet_length_ptr != NULL && indicator_ptr != NULL
+              && octet_length_ptr != indicator_ptr
+              && *indicator_ptr != SQL_NULL_DATA)
+            {
+              *octet_length_ptr= *indicator_ptr;
+            }
+          }
+          ++counter;
+        }
+      }
+    }
+    /* This MAGICAL fetch is required */
+    mysql_stmt_fetch(stmt->ssps);
+
+    return TRUE;
+  }
+  return FALSE;
+}
+
+
+int ssps_get_result(STMT *stmt)
+{
+  if (stmt->result)
+  {
+    if (!if_forward_cache(stmt))
+    {
+      return mysql_stmt_store_result(stmt->ssps);
+    }
+
+  }
+  
+  return 0;
+}
+
+
+void free_result_bind(STMT *stmt)
 {
   if (stmt->result_bind != NULL)
   {
+    int i, field_cnt= field_count(stmt);
+
     x_free(stmt->result_bind[0].is_null);
     x_free(stmt->result_bind[0].length);
     x_free(stmt->result_bind[0].error);
-    x_free(stmt->result_bind[0].buffer);
+
+    /* buffer was allocated for each column */
+    for (i= 0; i < field_cnt; i++)
+    {
+      x_free(stmt->result_bind[i].buffer);
+    }
 
     x_free(stmt->result_bind);
     stmt->result_bind= 0;
@@ -99,10 +248,12 @@ void ssps_close(STMT *stmt)
 {
   if (stmt->ssps != NULL)
   {
-    mysql_stmt_close(stmt->ssps);
-    stmt->ssps= NULL;
-
     free_result_bind(stmt);
+
+    if (mysql_stmt_close(stmt->ssps) != '\0')
+      assert(!"Could not close stmt");
+
+    stmt->ssps= NULL;
   }
 }
 
@@ -117,7 +268,7 @@ typedef struct tagBST
 
 /* {{{ allocate_buffer_for_field() -I- */
 static st_buffer_size_type
-allocate_buffer_for_field(const MYSQL_FIELD * const field)
+allocate_buffer_for_field(const MYSQL_FIELD * const field, BOOL outparams)
 {
   st_buffer_size_type result= {NULL, 0, field->type};
 
@@ -183,7 +334,20 @@ allocate_buffer_for_field(const MYSQL_FIELD * const field)
     case MYSQL_TYPE_SET:
     #endif
     case MYSQL_TYPE_BIT:
-      result.size= 1;/*8?*/
+      result.type= MYSQL_TYPE_BIT;
+      if (outparams)
+      {
+        /* For out params we surprisingly get it as string representation of a
+           number representing those bits. Allocating buffer to accommodate
+           largest string possible - 8 byte number + NULL terminator.
+           We will need to terminate the string to convert it to a number */
+        result.size= 30;
+      }
+      else
+      {
+        result.size= (field->length + 7)/8;
+      }
+
       break;
     case MYSQL_TYPE_GEOMETRY:
     default:
@@ -221,7 +385,7 @@ static MYSQL_ROW fetch_varlength_columns(STMT *stmt, MYSQL_ROW columns)
       stmt->result_bind[i].buffer= stmt->array[i];
       stmt->result_bind[i].buffer_length= stmt->lengths[i];
 
-      mysql_stmt_fetch_column(stmt->ssps, stmt->result_bind, i, 0);
+      mysql_stmt_fetch_column(stmt->ssps, &stmt->result_bind[i], i, 0);
     }
   }
 
@@ -237,6 +401,11 @@ int ssps_bind_result(STMT *stmt)
   const unsigned int  num_fields= field_count(stmt);
   unsigned int        i;
 
+  if (num_fields == 0)
+  {
+    return 0;
+  }
+
   if (stmt->result_bind)
   {
     /* We have fields requiring to read real length first */
@@ -256,43 +425,54 @@ int ssps_bind_result(STMT *stmt)
   }
   else
   {
-    my_bool             *is_null= my_malloc(sizeof(my_bool)*num_fields, MYF(MY_ZEROFILL));
-    my_bool             *err    = my_malloc(sizeof(my_bool)*num_fields, MYF(MY_ZEROFILL));
-    unsigned long       *len= my_malloc(sizeof(unsigned long)*num_fields, MYF(MY_ZEROFILL));
+    my_bool       *is_null= my_malloc(sizeof(my_bool)*num_fields,
+                                      MYF(MY_ZEROFILL));
+    my_bool       *err=     my_malloc(sizeof(my_bool)*num_fields,
+                                      MYF(MY_ZEROFILL));
+    unsigned long *len=     my_malloc(sizeof(unsigned long)*num_fields,
+                                      MYF(MY_ZEROFILL));
 
     /*TODO care about memory allocation errors */
-    stmt->result_bind=  (MYSQL_BIND*)my_malloc(sizeof(MYSQL_BIND)*num_fields, MYF(0));
-    stmt->array=        (MYSQL_ROW)my_malloc(sizeof(char*)*num_fields, MYF(MY_ZEROFILL));
+    stmt->result_bind=  (MYSQL_BIND*)my_malloc(sizeof(MYSQL_BIND)*num_fields,
+                                              MYF(MY_ZEROFILL));
+    stmt->array=        (MYSQL_ROW)my_malloc(sizeof(char*)*num_fields,
+                                              MYF(MY_ZEROFILL));
 
     for (i= 0; i < num_fields; ++i)
     {
-      MYSQL_FIELD    *field= mysql_fetch_field(stmt->result);
-      st_buffer_size_type p= allocate_buffer_for_field(field);
+      MYSQL_FIELD    *field= mysql_fetch_field_direct(stmt->result, i);
+      st_buffer_size_type p= allocate_buffer_for_field(field,
+                                                      IS_PS_OUT_PARAMS(stmt));
 
       stmt->result_bind[i].buffer_type  = p.type;
-	  stmt->result_bind[i].buffer       = p.buffer;
-	  stmt->result_bind[i].buffer_length= (unsigned long)p.size;
-	  stmt->result_bind[i].length       = &len[i];
-	  stmt->result_bind[i].is_null      = &is_null[i];
-	  stmt->result_bind[i].error        = &err[i];
-	  stmt->result_bind[i].is_unsigned  = field->flags & UNSIGNED_FLAG;
+	    stmt->result_bind[i].buffer       = p.buffer;
+	    stmt->result_bind[i].buffer_length= (unsigned long)p.size;
+	    stmt->result_bind[i].length       = &len[i];
+	    stmt->result_bind[i].is_null      = &is_null[i];
+	    stmt->result_bind[i].error        = &err[i];
+      stmt->result_bind[i].is_unsigned  = (field->flags & UNSIGNED_FLAG)? 1: 0;
 
       stmt->array[i]= p.buffer;
 
-      /* Marking that there are columns that will require buffer (re) allocating */
+      /* Marking that there are columns that will require buffer (re) allocating
+       */
       if (  stmt->result_bind[i].buffer       == 0
         &&  stmt->result_bind[i].buffer_type  != MYSQL_TYPE_NULL)
       {
         stmt->fix_fields= fetch_varlength_columns;
         
-        stmt->lengths= my_malloc(sizeof(unsigned long)*num_fields, MYF(MY_ZEROFILL));
+        /* Need to alloc it only once*/
+        if (stmt->lengths == NULL)
+        {
+          stmt->lengths= my_malloc(sizeof(unsigned long)*num_fields, MYF(MY_ZEROFILL));
+        }
         /* Buffer of initial length? */
       }
     }
+    
+    return mysql_stmt_bind_result(stmt->ssps, stmt->result_bind);
   }
 
-  mysql_stmt_bind_result(stmt->ssps, stmt->result_bind);
-
   return 0;
 }
 
@@ -427,9 +607,11 @@ char * ssps_get_string(STMT *stmt, ulong column_number, char *value, ulong *leng
     case MYSQL_TYPE_DECIMAL:
     case MYSQL_TYPE_NEWDECIMAL:
     case MYSQL_TYPE_STRING:
+    case MYSQL_TYPE_LONG_BLOB:
     case MYSQL_TYPE_BLOB:
     case MYSQL_TYPE_VARCHAR:
     case MYSQL_TYPE_VAR_STRING:
+      *length= *col_rbind->length;
       return (char *)(col_rbind->buffer);
     default:
       break;
@@ -438,7 +620,7 @@ char * ssps_get_string(STMT *stmt, ulong column_number, char *value, ulong *leng
 
   /* Basically should be prevented by earlied tests of
     conversion possibility */
-  return NULL;
+  return col_rbind->buffer;
 }
 /* }}} */
 
@@ -582,7 +764,7 @@ long long ssps_get_int64(STMT *stmt, ulong column_number, char *value, ulong len
           }
           else
           {
-            ret = !is_it_null? *(unsigned short *)(col_rbind->buffer):0;
+            ret = !is_it_null? *(short *)(col_rbind->buffer):0;
           }
           break;
 
@@ -590,11 +772,11 @@ long long ssps_get_int64(STMT *stmt, ulong column_number, char *value, ulong len
 
           if (is_it_unsigned)
           {
-            ret =  !is_it_null? *(int *)(col_rbind->buffer):0;
+            ret =  !is_it_null? *(unsigned int *)(col_rbind->buffer):0;
           }
           else
           {
-            ret =  !is_it_null? *(unsigned int *)(col_rbind->buffer):0;
+            ret =  !is_it_null? *(int *)(col_rbind->buffer):0;
           }
           break;
 
