@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2001, 2012, Oracle and/or its affiliates. All rights reserved.
+  Copyright (c) 2001, 2013, Oracle and/or its affiliates. All rights reserved.
 
   The MySQL Connector/ODBC is licensed under the terms of the GPLv2
   <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
@@ -129,30 +129,28 @@ typedef char * DYNAMIC_ELEMENT;
 
 SQLRETURN my_SQLPrepare(SQLHSTMT hstmt, SQLCHAR *szSqlStr, SQLINTEGER cbSqlStr,
                         my_bool dupe);
-SQLRETURN my_SQLExecute(STMT FAR* stmt);
+SQLRETURN my_SQLExecute(STMT * stmt);
 SQLRETURN SQL_API my_SQLFreeStmt(SQLHSTMT hstmt,SQLUSMALLINT fOption);
 SQLRETURN SQL_API my_SQLFreeStmtExtended(SQLHSTMT hstmt,
                     SQLUSMALLINT fOption, uint clearAllResults);
-SQLRETURN SQL_API my_SQLAllocStmt(SQLHDBC hdbc,SQLHSTMT FAR *phstmt);
-SQLRETURN do_query(STMT FAR *stmt,char *query, SQLULEN query_length);
-SQLRETURN insert_params(STMT FAR *stmt, SQLULEN row, char **finalquery,
+SQLRETURN SQL_API my_SQLAllocStmt(SQLHDBC hdbc,SQLHSTMT *phstmt);
+SQLRETURN do_query(STMT *stmt,char *query, SQLULEN query_length);
+SQLRETURN insert_params(STMT *stmt, SQLULEN row, char **finalquery,
                         SQLULEN *length);
-SQLRETURN odbc_stmt(DBC FAR *dbc, const char *query);
+SQLRETURN odbc_stmt(DBC *dbc, const char *query);
 void mysql_link_fields(STMT *stmt,MYSQL_FIELD *fields,uint field_count);
 void fix_row_lengths(STMT *stmt, const long* fix_rules, uint row, uint field_count);
 void fix_result_types(STMT *stmt);
 char *fix_str(char *to,const char *from,int length);
 char *dupp_str(char *from,int length);
-SQLRETURN my_pos_delete(STMT FAR *stmt,STMT FAR *stmtParam,
+SQLRETURN my_pos_delete(STMT *stmt,STMT *stmtParam,
 			SQLUSMALLINT irow,DYNAMIC_STRING *dynStr);
-SQLRETURN my_pos_update(STMT FAR *stmt,STMT FAR *stmtParam,
+SQLRETURN my_pos_update(STMT *stmt,STMT *stmtParam,
 			SQLUSMALLINT irow,DYNAMIC_STRING *dynStr);
-char *check_if_positioned_cursor_exists(STMT FAR *stmt, STMT FAR **stmtNew);
-SQLRETURN insert_param(STMT *stmt, char **to, DESC *apd,
+char *check_if_positioned_cursor_exists(STMT *stmt, STMT **stmtNew);
+SQLRETURN insert_param(STMT *stmt, uchar *to, DESC *apd,
                        DESCREC *aprec, DESCREC *iprec, SQLULEN row);
 char *add_to_buffer(NET *net,char *to,const char *from,ulong length);
-int is_set_names_statement(SQLCHAR *query);
-int is_select_statement(SQLCHAR *query);
 
 void reset_getdata_position(STMT *stmt);
 
@@ -171,7 +169,7 @@ copy_binary_result(STMT *stmt,
                    SQLCHAR *result, SQLLEN result_bytes, SQLLEN *used_bytes,
                    MYSQL_FIELD *field, char *src, unsigned long src_bytes);
 SQLRETURN copy_binhex_result(STMT *stmt,
-			     SQLCHAR FAR *rgbValue, SQLINTEGER cbValueMax,
+			     SQLCHAR *rgbValue, SQLINTEGER cbValueMax,
 			     SQLLEN *pcbValue, MYSQL_FIELD *field, char *src,
 			     ulong src_length);
 SQLRETURN copy_wchar_result(STMT *stmt,
@@ -179,15 +177,15 @@ SQLRETURN copy_wchar_result(STMT *stmt,
                             SQLLEN *pcbValue, MYSQL_FIELD *field, char *src,
                             long src_length);
 
-SQLRETURN set_dbc_error(DBC FAR *dbc, char *state,const char *message,uint errcode);
+SQLRETURN set_dbc_error(DBC *dbc, char *state,const char *message,uint errcode);
 #define set_stmt_error myodbc_set_stmt_error
-SQLRETURN set_stmt_error(STMT *stmt, char *state,const char *message,uint errcode);
+SQLRETURN set_stmt_error(STMT *stmt, const char *state, const char *message, uint errcode);
 SQLRETURN set_desc_error(DESC *desc, char *state,
                          const char *message, uint errcode);
 SQLRETURN handle_connection_error(STMT *stmt);
 my_bool is_connection_lost(uint errcode);
 void set_mem_error(MYSQL *mysql);
-void translate_error(char *save_state,myodbc_errid errid,uint mysql_err);
+void translate_error(char *save_state, myodbc_errid errid, uint mysql_err);
 
 SQLSMALLINT get_sql_data_type(STMT *stmt, MYSQL_FIELD *field, char *buff);
 SQLULEN get_column_size(STMT *stmt, MYSQL_FIELD *field);
@@ -226,10 +224,10 @@ SQLRETURN SQL_API my_SQLBindParameter(SQLHSTMT hstmt,SQLUSMALLINT ipar,
 				      SQLLEN *pcbValue);
 SQLRETURN SQL_API my_SQLExtendedFetch(SQLHSTMT hstmt, SQLUSMALLINT fFetchType,
 				      SQLLEN irow, SQLULEN *pcrow,
-				      SQLUSMALLINT FAR *rgfRowStatus, my_bool upd_status);
+				      SQLUSMALLINT *rgfRowStatus, my_bool upd_status);
 SQLRETURN SQL_API my_SQLSetPos(SQLHSTMT hstmt, SQLSETPOSIROW irow,
                                SQLUSMALLINT fOption, SQLUSMALLINT fLock);
-SQLRETURN copy_stmt_error(STMT FAR *src, STMT FAR *dst);
+SQLRETURN copy_stmt_error(STMT *src, STMT *dst);
 int unireg_to_c_datatype(MYSQL_FIELD *field);
 int default_c_type(int sql_data_type);
 ulong bind_length(int sql_data_type,ulong length);
@@ -244,7 +242,7 @@ void myodbc_init(void);
 void myodbc_ov_init(SQLINTEGER odbc_version);
 void myodbc_sqlstate2_init(void);
 void myodbc_sqlstate3_init(void);
-int check_if_server_is_alive(DBC FAR *dbc);
+int check_if_server_is_alive(DBC *dbc);
 my_bool dynstr_append_quoted_name(DYNAMIC_STRING *str, const char *name);
 SQLRETURN set_handle_error(SQLSMALLINT HandleType, SQLHANDLE handle,
 			   myodbc_errid errid, const char *errtext, SQLINTEGER errcode);
@@ -255,20 +253,20 @@ SQLRETURN set_conn_error(DBC *dbc,myodbc_errid errid, const char *errtext,
 SQLRETURN set_env_error(ENV * env,myodbc_errid errid, const char *errtext,
 			SQLINTEGER errcode);
 SQLRETURN copy_str_data(SQLSMALLINT HandleType, SQLHANDLE Handle,
-			SQLCHAR FAR *rgbValue, SQLSMALLINT cbValueMax,
-			SQLSMALLINT FAR *pcbValue,char FAR *src);
-SQLRETURN SQL_API my_SQLAllocEnv(SQLHENV FAR * phenv);
-SQLRETURN SQL_API my_SQLAllocConnect(SQLHENV henv, SQLHDBC FAR *phdbc);
+			SQLCHAR *rgbValue, SQLSMALLINT cbValueMax,
+			SQLSMALLINT *pcbValue,char *src);
+SQLRETURN SQL_API my_SQLAllocEnv(SQLHENV * phenv);
+SQLRETURN SQL_API my_SQLAllocConnect(SQLHENV henv, SQLHDBC *phdbc);
 SQLRETURN SQL_API my_SQLFreeConnect(SQLHDBC hdbc);
 SQLRETURN SQL_API my_SQLFreeEnv(SQLHENV henv);
 char *extend_buffer(NET *net,char *to,ulong length);
 void myodbc_end();
-my_bool set_dynamic_result(STMT FAR *stmt);
-void set_current_cursor_data(STMT FAR *stmt,SQLUINTEGER irow);
+my_bool set_dynamic_result(STMT *stmt);
+void set_current_cursor_data(STMT *stmt,SQLUINTEGER irow);
 my_bool is_minimum_version(const char *server_version,const char *version);
 int myodbc_strcasecmp(const char *s, const char *t);
 int myodbc_casecmp(const char *s, const char *t, uint len);
-my_bool reget_current_catalog(DBC FAR *dbc);
+my_bool reget_current_catalog(DBC *dbc);
 
 ulong myodbc_escape_string(MYSQL *mysql, char *to, ulong to_length,
                            const char *from, ulong length, int escape_id);
@@ -308,6 +306,8 @@ void end_query_log(FILE *query_log);
 
 LIST *list_delete_forward(LIST *elem);
 
+enum enum_field_types map_sql2mysql_type(SQLSMALLINT sql_type);
+
 /* proc_* functions - used to parse prcedures headers in SQLProcedureColumns */
 char *proc_param_tokenize(char *str, int *params_num);
 SQLCHAR *proc_get_param_type(SQLCHAR *proc, int len, SQLSMALLINT *ptype);
@@ -315,79 +315,101 @@ SQLCHAR* proc_get_param_name(SQLCHAR *proc, int len, SQLCHAR *cname);
 SQLCHAR* proc_get_param_dbtype(SQLCHAR *proc, int len, SQLCHAR *ptype);
 SQLUINTEGER proc_get_param_size(SQLCHAR *ptype, int len, int sql_type_index,
                                 SQLSMALLINT *dec);
-SQLLEN proc_get_param_octet_len(STMT *stmt, int sql_type_index, SQLULEN col_size, 
-                                SQLSMALLINT decimal_digits, unsigned int flags, char * str_buff);
+SQLLEN proc_get_param_octet_len(STMT *stmt, int sql_type_index,
+                                SQLULEN col_size, SQLSMALLINT decimal_digits,
+                                unsigned int flags, char * str_buff);
 SQLLEN proc_get_param_col_len(STMT *stmt, int sql_type_index, SQLULEN col_size, 
-                              SQLSMALLINT decimal_digits, unsigned int flags, char * str_buff);
+                              SQLSMALLINT decimal_digits, unsigned int flags,
+                              char * str_buff);
 int proc_get_param_sql_type_index(SQLCHAR *ptype, int len);
 SQLTypeMap *proc_get_param_map_by_index(int index);
 char *proc_param_next_token(char *str, char *str_end);
 
 void set_row_count(STMT * stmt, my_ulonglong rows);
-const char *get_fractional_part(const char * str, int len, BOOL dont_use_set_locale,
-                          SQLUINTEGER * fraction);
+const char *get_fractional_part(const char * str, int len,
+                                BOOL dont_use_set_locale,
+                                SQLUINTEGER * fraction);
 /* Convert MySQL timestamp to full ANSI timestamp format. */
 char * complete_timestamp(const char * value, ulong length, char buff[21]);
 long double strtold(const char *nptr, char **endptr);
 char * extend_buffer(NET *net, char *to, ulong length);
 char * add_to_buffer(NET *net,char *to,const char *from,ulong length);
-MY_LIMIT_CLAUSE find_position4limit(CHARSET_INFO* cs, char *query, char * query_end);
+MY_LIMIT_CLAUSE find_position4limit(CHARSET_INFO* cs, char *query,
+                                    char * query_end);
 BOOL myodbc_isspace(CHARSET_INFO* cs, const char * begin, const char *end);
+BOOL got_out_parameters(STMT *stmt);
 
+/* handle.c*/
+BOOL          allocate_param_bind(DYNAMIC_ARRAY **param_bind, uint elements);
 /*results.c*/
-long long binary2numeric(long long *dst, char *src, uint srcLen);
-BOOL isStatementForRead(STMT FAR *stmt);
-void fill_ird_data_lengths(DESC *ird, ulong *lengths, uint fields);
+long long     binary2numeric(long long *dst, char *src, uint srcLen);
+void          fill_ird_data_lengths(DESC *ird, ulong *lengths, uint fields);
 
 /* Functions to work with prepared and regular statements  */
+
+#ifdef SERVER_PS_OUT_PARAMS
+# define IS_PS_OUT_PARAMS(_stmt) ((_stmt)->dbc->mysql.server_status & SERVER_PS_OUT_PARAMS)
+#else
+/* In case if driver is built against old libmysl. In fact is not quite
+   correct */
+# define IS_PS_OUT_PARAMS(_stmt) (ssps_used(_stmt) && is_call_procedure(&_stmt->query) && !mysql_more_results(&(_stmt)->dbc->mysql))
+#endif
+
 /* my_stmt.c */
-BOOL          ssps_used(STMT *stmt);
-BOOL          returned_result(STMT *stmt);
-my_bool       free_current_result(STMT *stmt);
-MYSQL_RES *   get_result(STMT *stmt);
-unsigned int  field_count(STMT *stmt);
-my_ulonglong  affected_rows(STMT *stmt);
-my_ulonglong  update_affected_rows(STMT *stmt);
-my_ulonglong  num_rows(STMT *stmt);
-MYSQL_ROW     fetch_row(STMT *stmt);
-unsigned long*fetch_lengths(STMT *stmt);
-int           get_int(STMT *stmt, ulong column_number, char *value,
-                      ulong length);
-long long     get_int64(STMT *stmt, ulong column_number, char *value,
-                        ulong length);
-char *        get_string(STMT *stmt, ulong column_number, char *value,
-                         ulong *length, char * buffer);
-long double   get_double(STMT *stmt, ulong column_number, char *value,
-                         ulong length);
-BOOL          is_null(STMT *stmt, ulong column_number, char *value);
-void          scroller_reset(STMT *stmt);
+BOOL              ssps_used           (STMT *stmt);
+BOOL              returned_result     (STMT *stmt);
+my_bool           free_current_result (STMT *stmt);
+MYSQL_RES *       get_result_metadata (STMT *stmt, BOOL force_use);
+int               bind_result         (STMT *stmt);
+int               get_result          (STMT *stmt);
+unsigned int      field_count         (STMT *stmt);
+my_ulonglong      affected_rows       (STMT *stmt);
+my_ulonglong      update_affected_rows(STMT *stmt);
+my_ulonglong      num_rows            (STMT *stmt);
+MYSQL_ROW         fetch_row           (STMT *stmt);
+unsigned long*    fetch_lengths       (STMT *stmt);
+MYSQL_ROW_OFFSET  row_seek            (STMT *stmt, MYSQL_ROW_OFFSET offset);
+void              data_seek           (STMT *stmt, my_ulonglong offset);
+MYSQL_ROW_OFFSET  row_tell            (STMT *stmt);
+int               next_result         (STMT *stmt);
+
+int           get_int     (STMT *stmt, ulong column_number, char *value,
+                          ulong length);
+long long     get_int64   (STMT *stmt, ulong column_number, char *value,
+                          ulong length);
+char *        get_string  (STMT *stmt, ulong column_number, char *value,
+                          ulong *length, char * buffer);
+long double   get_double  (STMT *stmt, ulong column_number, char *value,
+                          ulong length);
+BOOL          is_null     (STMT *stmt, ulong column_number, char *value);
+SQLRETURN     prepare     (STMT *stmt, char * query, SQLINTEGER query_length);
+
+/* scroller-related functions */
+void          scroller_reset      (STMT *stmt);
 unsigned int  calc_prefetch_number(unsigned int selected, SQLULEN app_fetchs,
                                    SQLULEN max_rows);
-BOOL          scroller_exists(STMT * stmt);
-void          scroller_create(STMT * stmt, char *query, SQLULEN len);
-unsigned long long  scroller_move(STMT * stmt);
-SQLRETURN     scroller_prefetch(STMT * stmt);
-BOOL          scrollable(STMT * stmt, char * query, char * query_end);
+BOOL          scroller_exists     (STMT * stmt);
+void          scroller_create     (STMT * stmt, char *query, SQLULEN len);
+
+unsigned long long  scroller_move (STMT * stmt);
+
+SQLRETURN     scroller_prefetch   (STMT * stmt);
+BOOL          scrollable          (STMT * stmt, char * query, char * query_end);
 
 /* my_prepared_stmt.c */
-void  ssps_init(STMT *stmt);
-void  ssps_close(STMT *stmt);
-int   ssps_bind_result(STMT *stmt);
-BOOL  ssps_0buffers_truncated_only(STMT *stmt);
-long long   ssps_get_int64 (STMT *stmt, ulong column_number, char *value,
-                            ulong length);
-long double ssps_get_double(STMT *stmt, ulong column_number, char *value,
-                            ulong length);
-char *      ssps_get_string(STMT *stmt, ulong column_number, char *value,
-                            ulong *length, char * buffer);
-
-/* parse.c */
-const char *mystr_get_prev_token(CHARSET_INFO *charset,
-                                        const char **query, const char *start);
-const char *mystr_get_next_token(CHARSET_INFO *charset,
-                                        const char **query, const char *end);
-const char *find_token(CHARSET_INFO *charset, const char * begin,
-                       const char * end, const char * target);
+void        ssps_init             (STMT *stmt);
+BOOL        ssps_get_out_params   (STMT *stmt);
+int         ssps_get_result       (STMT *stmt);
+void        ssps_close            (STMT *stmt);
+int         ssps_bind_result      (STMT *stmt);
+void        free_result_bind      (STMT *stmt);
+BOOL        ssps_0buffers_truncated_only(STMT *stmt);
+long long   ssps_get_int64        (STMT *stmt, ulong column_number, char *value,
+                                  ulong length);
+long double ssps_get_double       (STMT *stmt, ulong column_number, char *value,
+                                  ulong length);
+char *      ssps_get_string       (STMT *stmt, ulong column_number, char *value,
+                                  ulong *length, char * buffer);
 
 #ifdef __WIN__
 #define cmp_database(A,B) myodbc_strcasecmp((const char *)(A),(const char *)(B))
