@@ -192,7 +192,7 @@ my_bool odbc_supported_conversion(SQLSMALLINT sqlType, SQLSMALLINT cType)
   @param[in]  length      Length of value
   @param[in]  arrec       ARD record for this column (can be NULL)
 */
-static SQLRETURN SQL_API
+SQLRETURN SQL_API
 sql_get_data(STMT *stmt, SQLSMALLINT fCType, uint column_number,
              SQLPOINTER rgbValue, SQLLEN cbValueMax, SQLLEN *pcbValue,
              char *value, ulong length, DESCREC *arrec)
@@ -273,8 +273,8 @@ sql_get_data(STMT *stmt, SQLSMALLINT fCType, uint column_number,
     {
       if (is_binary_ctype(fCType))
       {
-        return copy_binary_result(stmt, (SQLCHAR *)rgbValue, cbValueMax
-          , pcbValue, field , value , length);
+        return copy_binary_result(stmt, (SQLCHAR *)rgbValue, cbValueMax,
+                                  pcbValue, field, value, length);
       }
       else
       {
@@ -573,6 +573,13 @@ sql_get_data(STMT *stmt, SQLSMALLINT fCType, uint column_number,
             (SQL_TIMESTAMP_STRUCT *)rgbValue;
           time_t sec_time= time(NULL);
           struct tm cur_tm;
+
+          if (ts.hour > 23)
+          {
+            sec_time+= (ts.hour/24)*24*60*60;
+            ts.hour= ts.hour%24;
+          }
+
           localtime_r(&sec_time, &cur_tm);
 
           /* I wornder if that hasn't to be server current date*/
@@ -582,7 +589,8 @@ sql_get_data(STMT *stmt, SQLSMALLINT fCType, uint column_number,
           timestamp_info->hour=   ts.hour;
           timestamp_info->minute= ts.minute;
           timestamp_info->second= ts.second;
-          get_fractional_part(tmp, SQL_NTS, TRUE, &timestamp_info->fraction);
+          /* Fractional seconds must be 0 no matter what is actually in the field */
+          timestamp_info->fraction= 0;
           *pcbValue= sizeof(SQL_TIMESTAMP_STRUCT);
         }
       }
@@ -666,59 +674,6 @@ sql_get_data(STMT *stmt, SQLSMALLINT fCType, uint column_number,
 }
 
 
-/*!
-    \brief  Returns true if we are dealing with a statement which
-            is likely to result in reading only (SELECT || SHOW).
-
-            Some ODBC calls require knowledge about a statement
-            which we can not determine until we have executed 
-            the statement. This is because we do not parse the SQL
-            - the server does.
-
-            However if we silently execute a pending statement we
-            may insert rows.
-
-            So we do a very crude check of the SQL here to reduce 
-            the chance of a problem.
-
-    \sa     BUG 5778            
-*/
-BOOL isStatementForRead(STMT *stmt)
-{
-    char *pCursor;
-    int n = 0;
-    char szToken[55];
-
-    if ( !stmt )
-        return FALSE;
-
-    if ( !stmt->query )
-        return FALSE;
-
-    /* eat up any space */
-    for ( pCursor = stmt->query; pCursor != stmt->query_end && isspace( *pCursor ); )
-    {
-        ++pCursor; 
-    }
-
-    /* continue while alpha-numeric */
-    for ( ; pCursor != stmt->query_end && !isspace( *pCursor ) && n < 50; )
-    {
-        szToken[n] = toupper( *pCursor );
-        ++pCursor; 
-        ++n;
-    }
-
-    szToken[n] = '\0';
-
-    /* Took from Jess' "ssps rsmd" patch - "CALL" was missing here */
-    if (!strcmp(szToken, "SELECT") || !strcmp(szToken, "SHOW") ||
-        !strcmp(szToken, "CALL"))
-        return TRUE;
-
-    return FALSE;
-}
-
 /*
   @type    : myodbc3 internal
   @purpose : execute the query if it is only prepared. This is needed
@@ -726,39 +681,40 @@ BOOL isStatementForRead(STMT *stmt)
   before SQLExecute().
 */
 
-static SQLRETURN check_result(STMT FAR *stmt )
+static SQLRETURN check_result(STMT *stmt)
 {
-    SQLRETURN error= 0;
+  SQLRETURN error= 0;
 
-    switch ( stmt->state )
-    {
-        case ST_UNKNOWN:
-            error= set_stmt_error(stmt,"24000","Invalid cursor state",0);
-            break;
-        case ST_PREPARED:
-            if ( isStatementForRead( stmt ) )
-            {
-                SQLULEN real_max_rows= stmt->stmt_options.max_rows;
-                stmt->stmt_options.max_rows= 1;
-                /* select limit will be restored back to max_rows before real execution */
-                if ( (error= my_SQLExecute(stmt)) == SQL_SUCCESS )
-                {
-                    stmt->state= ST_PRE_EXECUTED;  /* mark for execute */
-                }
-                else
-                {
-                    set_sql_select_limit(stmt->dbc, real_max_rows);
-                }
-                stmt->stmt_options.max_rows= real_max_rows;
-            }
-            else
-                error = SQL_SUCCESS;
-            break;
-        case ST_PRE_EXECUTED:
-        case ST_EXECUTED:
-            error= SQL_SUCCESS;
-    }
-    return(error);
+  switch (stmt->state)
+  {
+    case ST_UNKNOWN:
+      error= set_stmt_error(stmt,"24000","Invalid cursor state",0);
+      break;
+    case ST_PREPARED:
+      /*TODO: introduce state for statements prepared on the server side */
+      if (!ssps_used(stmt) && stmt_returns_result(&stmt->query))
+      {
+        SQLULEN real_max_rows= stmt->stmt_options.max_rows;
+        stmt->stmt_options.max_rows= 1;
+        /* select limit will be restored back to max_rows before real execution */
+        if ( (error= my_SQLExecute(stmt)) == SQL_SUCCESS )
+        {
+          stmt->state= ST_PRE_EXECUTED;  /* mark for execute */
+        }
+        else
+        {
+          set_sql_select_limit(stmt->dbc, real_max_rows);
+        }
+        stmt->stmt_options.max_rows= real_max_rows;
+      }
+      else
+        error = SQL_SUCCESS;
+      break;
+    case ST_PRE_EXECUTED:
+    case ST_EXECUTED:
+      error= SQL_SUCCESS;
+  }
+  return(error);
 }
 
 /*
@@ -769,7 +725,7 @@ static SQLRETURN check_result(STMT FAR *stmt )
 SQLRETURN do_dummy_parambind(SQLHSTMT hstmt)
 {
     SQLRETURN rc;
-    STMT FAR *stmt= (STMT FAR *)hstmt;
+    STMT *stmt= (STMT *)hstmt;
     uint     nparam;
 
     for ( nparam= 0; nparam < stmt->param_count; ++nparam )
@@ -798,26 +754,30 @@ SQLRETURN do_dummy_parambind(SQLHSTMT hstmt)
   @purpose : returns the number of columns in a result set
 */
 
-SQLRETURN SQL_API SQLNumResultCols(SQLHSTMT  hstmt, SQLSMALLINT FAR *pccol)
+SQLRETURN SQL_API SQLNumResultCols(SQLHSTMT  hstmt, SQLSMALLINT *pccol)
 {
-    SQLRETURN error;
-    STMT FAR *stmt= (STMT FAR*) hstmt;
+  SQLRETURN error;
+  STMT *stmt= (STMT *) hstmt;
 
-    if ( stmt->param_count > 0 && stmt->dummy_state == ST_DUMMY_UNKNOWN &&
-         (stmt->state != ST_PRE_EXECUTED || stmt->state != ST_EXECUTED) )
+  if (!ssps_used(stmt))
+  {
+    if (stmt->param_count > 0 && stmt->dummy_state == ST_DUMMY_UNKNOWN &&
+      (stmt->state != ST_PRE_EXECUTED || stmt->state != ST_EXECUTED))
     {
-        if ( do_dummy_parambind(hstmt) != SQL_SUCCESS )
-            return SQL_ERROR;
+      if ( do_dummy_parambind(hstmt) != SQL_SUCCESS )
+      {
+        return SQL_ERROR;
+      }
     }
-    if ( (error= check_result(stmt)) != SQL_SUCCESS )
-        return error;
+    if ((error= check_result(stmt)) != SQL_SUCCESS)
+    {
+      return error;
+    }
+  }
 
-    if ( !stmt->result )
-        *pccol= 0;      /* Not a select */
-    else
-        *pccol= stmt->result->field_count;
+  *pccol= (SQLSMALLINT) stmt->ird->count;
 
-    return SQL_SUCCESS;
+  return SQL_SUCCESS;
 }
 
 
@@ -844,19 +804,22 @@ MySQLDescribeCol(SQLHSTMT hstmt, SQLUSMALLINT column,
 
   /* SQLDescribeCol can be called before SQLExecute. Thus we need make sure that
      all parameters have been bound */
-  if ( stmt->param_count > 0 && stmt->dummy_state == ST_DUMMY_UNKNOWN &&
-    (stmt->state != ST_PRE_EXECUTED || stmt->state != ST_EXECUTED) )
+  if (!ssps_used(stmt))
   {
-    if ( do_dummy_parambind(hstmt) != SQL_SUCCESS )
-      return SQL_ERROR;
-  }
+    if ( stmt->param_count > 0 && stmt->dummy_state == ST_DUMMY_UNKNOWN &&
+      (stmt->state != ST_PRE_EXECUTED || stmt->state != ST_EXECUTED) )
+    {
+      if ( do_dummy_parambind(hstmt) != SQL_SUCCESS )
+        return SQL_ERROR;
+    }
 
-  if ((error= check_result(stmt)) != SQL_SUCCESS)
-    return error;
-  if (!stmt->result)
-    return set_stmt_error(stmt, "07005", "No result set", 0);
-  if (column == 0 || column > stmt->ird->count)
-    return set_stmt_error(stmt, "07009", "Invalid descriptor index", 0);
+    if ((error= check_result(stmt)) != SQL_SUCCESS)
+      return error;
+    if (!stmt->result)
+      return set_stmt_error(stmt, "07005", "No result set", 0);
+    if (column == 0 || column > stmt->ird->count)
+      return set_stmt_error(stmt, "07009", "Invalid descriptor index", 0);
+  }
 
   irrec= desc_get_rec(stmt->ird, column - 1, FALSE);
   assert(irrec);
@@ -918,20 +881,25 @@ MySQLColAttribute(SQLHSTMT hstmt, SQLUSMALLINT column,
   SQLRETURN error= SQL_SUCCESS;
   DESCREC *irrec;
 
-  /* MySQLColAttribute can be called before SQLExecute. Thus we need make sure that
-  all parameters have been bound */
-  if ( stmt->param_count > 0 && stmt->dummy_state == ST_DUMMY_UNKNOWN &&
-    (stmt->state != ST_PRE_EXECUTED || stmt->state != ST_EXECUTED) )
+  if (!ssps_used(stmt))
   {
-    if ( do_dummy_parambind(hstmt) != SQL_SUCCESS )
+    /* MySQLColAttribute can be called before SQLExecute. Thus we need make sure that
+    all parameters have been bound */
+    if ( stmt->param_count > 0 && stmt->dummy_state == ST_DUMMY_UNKNOWN &&
+      (stmt->state != ST_PRE_EXECUTED || stmt->state != ST_EXECUTED) )
+    {
+      if ( do_dummy_parambind(hstmt) != SQL_SUCCESS )
+        return SQL_ERROR;
+    }
+
+    if (check_result(stmt) != SQL_SUCCESS)
       return SQL_ERROR;
   }
 
-  if (check_result(stmt) != SQL_SUCCESS)
-    return SQL_ERROR;
-
   if (!stmt->result)
+  {
     return set_stmt_error(stmt, "07005", "No result set", 0);
+  }
 
   /* we report bookmark type if requested, nothing else */
   if (attrib == SQL_DESC_TYPE && column == 0)
@@ -1087,7 +1055,7 @@ SQLRETURN SQL_API SQLBindCol(SQLHSTMT      StatementHandle,
                              SQLLEN *      StrLen_or_IndPtr)
 {
   SQLRETURN rc;
-  STMT FAR *stmt= (STMT FAR*) StatementHandle;
+  STMT *stmt= (STMT *) StatementHandle;
   DESCREC *arrec;
   /* TODO if this function fails, the SQL_DESC_COUNT should be unchanged in ard */
 
@@ -1166,7 +1134,7 @@ SQLRETURN SQL_API SQLBindCol(SQLHSTMT      StatementHandle,
   @purpose : returns the latest resultset(dynamic)
 */
 
-my_bool set_dynamic_result(STMT FAR *stmt)
+my_bool set_dynamic_result(STMT *stmt)
 {
   SQLRETURN rc;
   long row= stmt->current_row;
@@ -1254,86 +1222,106 @@ SQLRETURN SQL_API SQLGetData(SQLHSTMT      StatementHandle,
 
 SQLRETURN SQL_API SQLMoreResults( SQLHSTMT hStmt )
 {
-    STMT FAR *  pStmt   = (STMT FAR*)hStmt;
-    int         nRetVal;
-    SQLRETURN   nReturn = SQL_SUCCESS;
+  STMT *  pStmt   = (STMT *)hStmt;
+  int         nRetVal;
+  SQLRETURN   nReturn = SQL_SUCCESS;
 
-    pthread_mutex_lock( &pStmt->dbc->lock );
+  pthread_mutex_lock( &pStmt->dbc->lock );
 
-    CLEAR_STMT_ERROR( pStmt );
+  CLEAR_STMT_ERROR( pStmt );
 
-    if (!mysql_more_results(&pStmt->dbc->mysql))
+  /*
+    http://msdn.microsoft.com/en-us/library/ms714673%28v=vs.85%29.aspx
+
+    For some drivers, output parameters and return values are not available
+    until all result sets and row counts have been processed. For such 
+    drivers, output parameters and return values become available when 
+    SQLMoreResults returns SQL_NO_DATA.
+  */
+  if ( pStmt->state != ST_EXECUTED )
+  {
+    nReturn = SQL_NO_DATA;
+    goto exitSQLMoreResults;
+  }
+
+  /* try to get next resultset */
+  nRetVal = next_result(pStmt);
+
+  /* call to mysql_next_result() failed */
+  if (nRetVal > 0)
+  {
+    nRetVal= mysql_errno(&pStmt->dbc->mysql);
+
+    switch ( nRetVal )
     {
-      nReturn= SQL_NO_DATA;
+      case CR_SERVER_GONE_ERROR:
+      case CR_SERVER_LOST:
+        nReturn = set_stmt_error( pStmt, "08S01", mysql_error( &pStmt->dbc->mysql ), nRetVal );
+        goto exitSQLMoreResults;
+      case CR_COMMANDS_OUT_OF_SYNC:
+      case CR_UNKNOWN_ERROR:
+        nReturn = set_stmt_error( pStmt, "HY000", mysql_error( &pStmt->dbc->mysql ), nRetVal );
+        goto exitSQLMoreResults;
+      default:
+        nReturn = set_stmt_error( pStmt, "HY000", "unhandled error from mysql_next_result()", nRetVal );
+        goto exitSQLMoreResults;
+    }
+  }
+
+  /* no more resultsets */
+  if (nRetVal < 0)
+  {
+    nReturn = SQL_NO_DATA;
+    goto exitSQLMoreResults;
+  }
+
+  /* cleanup existing resultset */
+  nReturn = my_SQLFreeStmtExtended((SQLHSTMT)pStmt,SQL_CLOSE,0);
+  if (!SQL_SUCCEEDED( nReturn ))
+  {
+    goto exitSQLMoreResults;
+  }
+
+  /* start using the new resultset */
+  pStmt->result = get_result_metadata(pStmt, FALSE);
+
+  if (!pStmt->result)
+  {
+    /* no fields means; INSERT, UPDATE or DELETE so no resultset is fine */
+    if (!field_count(pStmt))
+    {
+      pStmt->state = ST_EXECUTED;
+      pStmt->affected_rows = affected_rows(pStmt);
       goto exitSQLMoreResults;
     }
-
-    /* SQLExecute or SQLExecDirect need to be called first */
-    if ( pStmt->state != ST_EXECUTED )
+    /* we have fields but no resultset (not even an empty one) - this is bad */
+    nReturn = set_stmt_error(pStmt, "HY000", mysql_error( &pStmt->dbc->mysql ),
+                              mysql_errno(&pStmt->dbc->mysql));
+    goto exitSQLMoreResults;
+  }
+  
+  /* checking if next result is SP OUT params and fetch them if needed */
+  if (IS_PS_OUT_PARAMS(pStmt))
+  {
+    fix_result_types(pStmt);
+    /* This server status(SERVER_PS_OUT_PARAMS) can be only if we used PS */
+    ssps_get_out_params(pStmt);
+  }
+  else
+  {
+    free_result_bind(pStmt);
+    if (bind_result(pStmt) || get_result(pStmt))
     {
-        nReturn = set_stmt_error( pStmt, "HY010", NULL, 0 );
-        goto exitSQLMoreResults;
+      nReturn= set_stmt_error(pStmt, "HY000", mysql_error( &pStmt->dbc->mysql ),
+                            mysql_errno(&pStmt->dbc->mysql));
     }
 
-    /* try to get next resultset */
-    nRetVal = mysql_next_result( &pStmt->dbc->mysql );
-
-    /* call to mysql_next_result() failed */
-    if ( nRetVal > 0 )
-    {
-        nRetVal = mysql_errno( &pStmt->dbc->mysql );
-        switch ( nRetVal )
-        {
-            case CR_SERVER_GONE_ERROR:
-            case CR_SERVER_LOST:
-                nReturn = set_stmt_error( pStmt, "08S01", mysql_error( &pStmt->dbc->mysql ), nRetVal );
-                goto exitSQLMoreResults;
-            case CR_COMMANDS_OUT_OF_SYNC:
-            case CR_UNKNOWN_ERROR:
-                nReturn = set_stmt_error( pStmt, "HY000", mysql_error( &pStmt->dbc->mysql ), nRetVal );
-                goto exitSQLMoreResults;
-            default:
-                nReturn = set_stmt_error( pStmt, "HY000", "unhandled error from mysql_next_result()", nRetVal );
-                goto exitSQLMoreResults;
-        }
-    }
-
-    /* no more resultsets */
-    if ( nRetVal < 0 )
-    {
-        nReturn = SQL_NO_DATA;
-        goto exitSQLMoreResults;
-    }
-
-    /* cleanup existing resultset */
-    nReturn = my_SQLFreeStmtExtended((SQLHSTMT)pStmt,SQL_CLOSE,0);
-    if ( !SQL_SUCCEEDED( nReturn ) )
-        goto exitSQLMoreResults;
-
-    /* start using the new resultset */
-    if ( if_forward_cache( pStmt ) )
-        pStmt->result = mysql_use_result( &pStmt->dbc->mysql );
-    else
-        pStmt->result = mysql_store_result( &pStmt->dbc->mysql );
-
-    if ( !pStmt->result )
-    {
-        /* no fields means; INSERT, UPDATE or DELETE so no resultset is fine */
-        if ( !mysql_field_count( &pStmt->dbc->mysql ) )
-        {
-            pStmt->state = ST_EXECUTED;
-            pStmt->affected_rows = mysql_affected_rows( &pStmt->dbc->mysql );
-            goto exitSQLMoreResults;
-        }
-        /* we have fields but no resultset (not even an empty one) - this is bad */
-        nReturn = set_stmt_error( pStmt, "HY000", mysql_error( &pStmt->dbc->mysql ), mysql_errno( &pStmt->dbc->mysql ) );
-        goto exitSQLMoreResults;
-    }
-    fix_result_types( pStmt );
+    fix_result_types(pStmt);
+  }
 
 exitSQLMoreResults:
-    pthread_mutex_unlock( &pStmt->dbc->lock );
-    return nReturn;
+  pthread_mutex_unlock( &pStmt->dbc->lock );
+  return nReturn;
 }
 
 
@@ -1348,11 +1336,11 @@ exitSQLMoreResults:
 SQLRETURN SQL_API SQLRowCount( SQLHSTMT hstmt, 
                                SQLLEN * pcrow )
 {
-    STMT FAR *stmt= (STMT FAR*) hstmt;
+    STMT *stmt= (STMT *) hstmt;
 
     if ( stmt->result )
     {
-      /* for SetPos operations result is defined */
+      /* for SetPos operations result is defined and they use direct execution */
         *pcrow= (SQLLEN) affected_rows(stmt);
     }
     else
@@ -1379,7 +1367,7 @@ void fill_ird_data_lengths(DESC *ird, ulong *lengths, uint fields)
 
   /* This will be NULL for catalog functions with "fake" results */
   if (!lengths)
-  {	
+  {
     return;
   }
 
@@ -1387,7 +1375,8 @@ void fill_ird_data_lengths(DESC *ird, ulong *lengths, uint fields)
   {
     irrec= desc_get_rec(ird, i, FALSE);
     assert(irrec);
-	irrec->row.datalen= lengths[i];
+
+    irrec->row.datalen= lengths[i];
   }
 }
 
@@ -1406,7 +1395,7 @@ fill_fetch_buffers(STMT *stmt, MYSQL_ROW values, uint rownum)
   int i;
   ulong length= 0;
   DESCREC *irrec, *arrec;
-
+ 
   for (i= 0; i < myodbc_min(stmt->ird->count, stmt->ard->count); ++i, ++values)
   {
     irrec= desc_get_rec(stmt->ird, i, FALSE);
@@ -1481,14 +1470,14 @@ SQLRETURN SQL_API my_SQLExtendedFetch( SQLHSTMT             hstmt,
                                        SQLUSMALLINT         fFetchType,
                                        SQLLEN               irow,
                                        SQLULEN             *pcrow,
-                                       SQLUSMALLINT FAR    *rgfRowStatus,
+                                       SQLUSMALLINT        *rgfRowStatus,
                                        my_bool              upd_status )
 {
     SQLULEN           rows_to_fetch;
     long              cur_row, max_row;
     SQLULEN           i;
     SQLRETURN         row_res, res;
-    STMT FAR          *stmt= (STMT FAR*) hstmt;
+    STMT              *stmt= (STMT *) hstmt;
     MYSQL_ROW         values= 0;
     MYSQL_ROW_OFFSET  save_position;
     SQLULEN           dummy_pcrow;
@@ -1497,23 +1486,36 @@ SQLRETURN SQL_API my_SQLExtendedFetch( SQLHSTMT             hstmt,
     LINT_INIT(save_position);
 
     if ( !stmt->result )
-        return set_stmt_error(stmt, "24000", "Fetch without a SELECT", 0);
+      return set_stmt_error(stmt, "24000", "Fetch without a SELECT", 0);
+
+    if (stmt->out_params_state > 0)
+    {
+      switch(--stmt->out_params_state)
+      {
+      case 0:
+        return SQL_NO_DATA_FOUND;
+      default:
+        /* TODO: Need to remember real fetch' result*/
+        /* just in case...*/
+        stmt->out_params_state= 1;
+      }
+    }
 
     cur_row = stmt->current_row;
 
     if ( stmt->stmt_options.cursor_type == SQL_CURSOR_FORWARD_ONLY )
     {
-        if ( fFetchType != SQL_FETCH_NEXT && !stmt->dbc->ds->safe )
-            return  set_error(stmt,MYERR_S1106,
-                              "Wrong fetchtype with FORWARD ONLY cursor", 0);
+      if ( fFetchType != SQL_FETCH_NEXT && !stmt->dbc->ds->safe )
+        return  set_error(stmt,MYERR_S1106,
+                          "Wrong fetchtype with FORWARD ONLY cursor", 0);
     }
 
     if ( if_dynamic_cursor(stmt) && set_dynamic_result(stmt) )
-        return set_error(stmt,MYERR_S1000,
-                         "Driver Failed to set the internal dynamic result", 0);
+      return set_error(stmt,MYERR_S1000,
+                       "Driver Failed to set the internal dynamic result", 0);
 
     if ( !pcrow )
-        pcrow= &dummy_pcrow;
+      pcrow= &dummy_pcrow;
 
     /* for scrollable cursor("scroller") max_row is max row for currently
        fetched part of resultset */
@@ -1523,56 +1525,58 @@ SQLRETURN SQL_API my_SQLExtendedFetch( SQLHSTMT             hstmt,
 
     switch ( fFetchType )
     {
-        case SQL_FETCH_NEXT:
-            cur_row= (stmt->current_row < 0 ? 0 :
-                      stmt->current_row+stmt->rows_found_in_set);
-            break;
-        case SQL_FETCH_PRIOR:
-            cur_row= (stmt->current_row <= 0 ? -1 :
-                      (long)(stmt->current_row - stmt->ard->array_size));
-            break;
-        case SQL_FETCH_FIRST:
-            cur_row= 0L;
-            break;
-        case SQL_FETCH_LAST:
-            cur_row= max_row - stmt->ard->array_size;
-            break;
-        case SQL_FETCH_ABSOLUTE:
-            if ( irow < 0 )
-            {
-                /* Fetch from end of result set */
-                if ( max_row+irow < 0 && -irow <= (long) stmt->ard->array_size )
-                {
-                    /*
-                      | FetchOffset | > LastResultRow AND
-                      | FetchOffset | <= RowsetSize
-                    */
-                    cur_row= 0;     /* Return from beginning */
-                }
-                else
-                    cur_row= max_row+irow;     /* Ok if max_row <= -irow */
-            }
-            else
-                cur_row= (long) irow-1;
-            break;
+      case SQL_FETCH_NEXT:
+        cur_row= (stmt->current_row < 0 ? 0 :
+                  stmt->current_row+stmt->rows_found_in_set);
+        break;
+      case SQL_FETCH_PRIOR:
+        cur_row= (stmt->current_row <= 0 ? -1 :
+                  (long)(stmt->current_row - stmt->ard->array_size));
+        break;
+      case SQL_FETCH_FIRST:
+        cur_row= 0L;
+        break;
+      case SQL_FETCH_LAST:
+        cur_row= max_row - stmt->ard->array_size;
+        break;
+      case SQL_FETCH_ABSOLUTE:
+        if (irow < 0)
+        {
+          /* Fetch from end of result set */
+          if ( max_row+irow < 0 && -irow <= (long) stmt->ard->array_size )
+          {
+            /*
+              | FetchOffset | > LastResultRow AND
+              | FetchOffset | <= RowsetSize
+            */
+            cur_row= 0;     /* Return from beginning */
+          }
+          else
+            cur_row= max_row+irow;     /* Ok if max_row <= -irow */
+        }
+        else
+          cur_row= (long) irow-1;
+        break;
 
-        case SQL_FETCH_RELATIVE:
-            cur_row= stmt->current_row + irow;
-            if ( stmt->current_row > 0 && cur_row < 0 &&
-                 (long)-irow <= (long)stmt->ard->array_size )
-                cur_row= 0;
-            break;
+      case SQL_FETCH_RELATIVE:
+        cur_row= stmt->current_row + irow;
+        if (stmt->current_row > 0 && cur_row < 0 &&
+           (long)-irow <= (long)stmt->ard->array_size)
+        {
+          cur_row= 0;
+        }
+        break;
 
-        default:
-            return set_error(stmt, MYERR_S1106, "Fetch type out of range", 0);
+      default:
+          return set_error(stmt, MYERR_S1106, "Fetch type out of range", 0);
     }
 
     if ( cur_row < 0 )
     {
-        stmt->current_row= -1;  /* Before first row */
-        stmt->rows_found_in_set= 0;
-        mysql_data_seek(stmt->result,0L);
-        return SQL_NO_DATA_FOUND;
+      stmt->current_row= -1;  /* Before first row */
+      stmt->rows_found_in_set= 0;
+      data_seek(stmt, 0L);
+      return SQL_NO_DATA_FOUND;
     }
     if ( cur_row > max_row )
     {
@@ -1603,9 +1607,9 @@ SQLRETURN SQL_API my_SQLExtendedFetch( SQLHSTMT             hstmt,
         if ( stmt->stmt_options.cursor_type != SQL_CURSOR_DYNAMIC &&
              cur_row && cur_row == (long)(stmt->current_row +
                                           stmt->rows_found_in_set) )
-            mysql_row_seek(stmt->result,stmt->end_of_set);
+            row_seek(stmt, stmt->end_of_set);
         else
-            mysql_data_seek(stmt->result,cur_row);
+            data_seek(stmt, cur_row);
     }
     stmt->current_row= cur_row;
 
@@ -1620,15 +1624,23 @@ SQLRETURN SQL_API my_SQLExtendedFetch( SQLHSTMT             hstmt,
                                 (long)stmt->ard->array_size);
     }
 
-    if ( !rows_to_fetch )
+    /* out params has been silently fetched */
+    if (rows_to_fetch == 0)
     {
+      if (stmt->out_params_state > 0)
+      {
+        rows_to_fetch= 1;
+      }
+      else
+      {
         *pcrow= 0;
         stmt->rows_found_in_set= 0;
         if ( upd_status && stmt->ird->rows_processed_ptr )
         {
-            *stmt->ird->rows_processed_ptr= 0;
+          *stmt->ird->rows_processed_ptr= 0;
         }
         return SQL_NO_DATA_FOUND;
+      }
     }
 
     if (!stmt->dbc->ds->dont_use_set_locale)
@@ -1639,153 +1651,165 @@ SQLRETURN SQL_API my_SQLExtendedFetch( SQLHSTMT             hstmt,
     res= SQL_SUCCESS;
     for (i= 0 ; i < rows_to_fetch ; ++i)
     {
-        if ( stmt->result_array )
+      if ( stmt->result_array )
+      {
+        values= stmt->result_array + cur_row*stmt->result->field_count;
+        if ( i == 0 )
         {
-            values= stmt->result_array+cur_row*stmt->result->field_count;
-            if ( i == 0 )
+          stmt->current_values= values;
+        }
+      }
+      else
+      {
+        /* This code will ensure that values is always set */
+        if ( i == 0 )
+        {
+            save_position= row_tell(stmt);
+        }
+        /* - Actual fetching happens here - */
+        if ( stmt->out_params_state == 0
+          && !(values= fetch_row(stmt)) )
+        {
+          if (scroller_exists(stmt))
+          {
+            scroller_move(stmt);
+
+            row_res= scroller_prefetch(stmt);
+
+            if (row_res != SQL_SUCCESS)
             {
-                stmt->current_values= values;
+              break;
             }
+
+            if ( !(values= fetch_row(stmt)) )
+            {
+              break;
+            }
+
+            /* Not sure that is right, but see it better than nothing */
+            save_position= row_tell(stmt);
+          }
+          else
+          {
+            break;
+          }
+        }
+
+        if (stmt->out_params_state)
+        {
+          values= stmt->array;
+        }
+
+        if ( stmt->fix_fields )
+        {
+            values= (*stmt->fix_fields)(stmt,values);
+        }
+
+        stmt->current_values= values;
+      }
+
+      if (!stmt->fix_fields)
+      {
+        /* lengths contains lengths for all rows. Alternate use could be
+           filling ird buffers in the (fix_fields) function. In this case
+           lengths could contain just one array with rules for lengths
+           calculating(it can work out in many cases like in catalog functions
+           there some fields from results of auxiliary query are simply mixed
+           somehow and constant fields added ).
+           Another approach could be using of "array" and "order" arrays
+           and special fix_fields callback, that will fix array and set
+           lengths in ird*/
+        if (stmt->lengths)
+        {
+          fill_ird_data_lengths(stmt->ird, stmt->lengths + cur_row*stmt->result->field_count,
+                                stmt->result->field_count);
         }
         else
         {
-            /* This code will ensure that values is always set */
-            if ( i == 0 )
-            {
-                save_position= mysql_row_tell(stmt->result);
-            }
-            if ( !(values= fetch_row(stmt)) )
-            {
-              if (scroller_exists(stmt))
-              {
-                scroller_move(stmt);
-
-                row_res= scroller_prefetch(stmt);
-
-                if (row_res != SQL_SUCCESS)
-                {
-                  break;
-                }
-
-                if ( !(values= fetch_row(stmt)) )
-                {
-                  break;
-                }
-
-                /* Not sure that is right, but see it better than nothing */
-                save_position= mysql_row_tell(stmt->result);
-              }
-              else
-              {
-                break;
-              }
-
-            }
-
-            if ( stmt->fix_fields )
-            {
-                values= (*stmt->fix_fields)(stmt,values);
-            }
-
-            stmt->current_values= values;
+          fill_ird_data_lengths(stmt->ird, fetch_lengths(stmt),
+                                stmt->result->field_count);
         }
+      }
 
-        if (!stmt->fix_fields)
+      row_res= fill_fetch_buffers(stmt, values, i);
+
+      /* For SQL_SUCCESS we need all rows to be SQL_SUCCESS */
+      if (res != row_res)
+      {
+        /* Any successful row makes overall result SQL_SUCCESS_WITH_INFO */
+        if (SQL_SUCCEEDED(row_res))
         {
-          /* lengths contains lengths for all rows. Alternate use could be
-             filling ird buffers in the (fix_fields) function. In this case
-             lengths could contain just one array with rules for lengths
-             calculating(it can work out in many cases like in catalog functions
-             there some fields from results of auxiliary query are simply mixed
-             somehow and constant fields added ).
-             Another approach could be using of "array" and "order" arrays
-             and special fix_fields callback, that will fix array and set
-             lengths in ird*/
-          if (stmt->lengths)
-            fill_ird_data_lengths(stmt->ird, stmt->lengths + cur_row*stmt->result->field_count,
-                                  stmt->result->field_count);
-          else
-            fill_ird_data_lengths(stmt->ird, fetch_lengths(stmt),
-                                  stmt->result->field_count);
+          res= SQL_SUCCESS_WITH_INFO;
         }
-        row_res= fill_fetch_buffers(stmt, values, i);
-
-        /* For SQL_SUCCESS we need all rows to be SQL_SUCCESS */
-        if (res != row_res)
+        /* Else error */
+        else if (i == 0)
         {
-          /* Any successful row makes overall result SQL_SUCCESS_WITH_INFO */
-          if (SQL_SUCCEEDED(row_res))
-          {
-            res= SQL_SUCCESS_WITH_INFO;
-          }
-          /* Else error */
-          else if (i == 0)
-          {
-            /* SQL_ERROR only if all rows fail */
-            res= SQL_ERROR;
-          }
-          else
-          {
-            res= SQL_SUCCESS_WITH_INFO;
-          }
+          /* SQL_ERROR only if all rows fail */
+          res= SQL_ERROR;
         }
-
-        /* "Fetching" includes buffers filling. I think errors in that 
-           have to affect row status */
-
-        if (rgfRowStatus)
+        else
         {
-          rgfRowStatus[i]= sqlreturn2row_status(row_res);
-        }
-        /*
-          No need to update rowStatusPtr_ex, it's the same as rgfRowStatus.
-        */
-        if (upd_status && stmt->ird->array_status_ptr)
-        {
-          stmt->ird->array_status_ptr[i]= sqlreturn2row_status(row_res);
+          res= SQL_SUCCESS_WITH_INFO;
         }
+      }
 
-        ++cur_row;
+      /* "Fetching" includes buffers filling. I think errors in that 
+         have to affect row status */
+
+      if (rgfRowStatus)
+      {
+        rgfRowStatus[i]= sqlreturn2row_status(row_res);
+      }
+      /*
+        No need to update rowStatusPtr_ex, it's the same as rgfRowStatus.
+      */
+      if (upd_status && stmt->ird->array_status_ptr)
+      {
+        stmt->ird->array_status_ptr[i]= sqlreturn2row_status(row_res);
+      }
+
+      ++cur_row;
     }   /* fetching cycle end*/
 
     stmt->rows_found_in_set= i;
     *pcrow= i;
 
     disconnected= is_connection_lost(mysql_errno(&stmt->dbc->mysql))
-        && handle_connection_error(stmt);
+      && handle_connection_error(stmt);
 
     if ( upd_status && stmt->ird->rows_processed_ptr )
     {
-        *stmt->ird->rows_processed_ptr= i;
+      *stmt->ird->rows_processed_ptr= i;
     }
 
     /* It is possible that both rgfRowStatus and array_status_ptr are set
     (and upp_status is TRUE) */
     for ( ; i < stmt->ard->array_size ; ++i )
     {
-        if ( rgfRowStatus )
-        {
-          rgfRowStatus[i]= disconnected ? SQL_ROW_ERROR : SQL_ROW_NOROW;
-        }
-        /*
-          No need to update rowStatusPtr_ex, it's the same as rgfRowStatus.
-        */
-        if ( upd_status && stmt->ird->array_status_ptr )
-        {
-          stmt->ird->array_status_ptr[i]= disconnected? SQL_ROW_ERROR
-                                                      : SQL_ROW_NOROW;
-        }
+      if ( rgfRowStatus )
+      {
+        rgfRowStatus[i]= disconnected ? SQL_ROW_ERROR : SQL_ROW_NOROW;
+      }
+      /*
+        No need to update rowStatusPtr_ex, it's the same as rgfRowStatus.
+      */
+      if ( upd_status && stmt->ird->array_status_ptr )
+      {
+        stmt->ird->array_status_ptr[i]= disconnected? SQL_ROW_ERROR
+                                                    : SQL_ROW_NOROW;
+      }
     }
 
     if (SQL_SUCCEEDED(res) && !stmt->result_array && !if_forward_cache(stmt))
     {
-        /* reset result position */
-        stmt->end_of_set= mysql_row_seek(stmt->result,save_position);
+      /* reset result position */
+      stmt->end_of_set= row_seek(stmt, save_position);
     }
 
     if (!stmt->dbc->ds->dont_use_set_locale)
-
-        setlocale(LC_NUMERIC,default_locale);
+    {
+      setlocale(LC_NUMERIC,default_locale);
+    }
 
     if (SQL_SUCCEEDED(res)
       && stmt->rows_found_in_set < stmt->ard->array_size)
@@ -1815,11 +1839,11 @@ SQLRETURN SQL_API SQLExtendedFetch( SQLHSTMT        hstmt,
                                     SQLUSMALLINT    fFetchType,
                                     SQLLEN          irow,
                                     SQLULEN        *pcrow,
-                                    SQLUSMALLINT FAR *rgfRowStatus )
+                                    SQLUSMALLINT   *rgfRowStatus )
 {
     SQLRETURN rc;
     SQLULEN rows;
-    STMT_OPTIONS *options= &((STMT FAR *)hstmt)->stmt_options;
+    STMT_OPTIONS *options= &((STMT *)hstmt)->stmt_options;
 
     options->rowStatusPtr_ex= rgfRowStatus;
 
