@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
+  Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
 
   The MySQL Connector/ODBC is licensed under the terms of the GPLv2
   <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
@@ -941,6 +941,15 @@ SQLSetConnectAttrWImpl(SQLHDBC hdbc, SQLINTEGER attribute,
   if (attribute == SQL_ATTR_CURRENT_CATALOG)
   {
     uint errors= 0;
+
+    /* Verifying for incorrect val_len */
+    if (value_len < 0 && value_len != SQL_NTS)
+    {
+      return set_dbc_error(dbc, "HY090",
+                  " StringLength argument was less "
+                  "than 0 but was not SQL_NTS " , 0);
+    }
+
     if (is_connected(dbc))
       value= sqlwchar_as_sqlchar(dbc->cxn_charset_info,
                                  value, &len, &errors);
@@ -1199,15 +1208,16 @@ SQLSetDescRecW(SQLHDESC hdesc, SQLSMALLINT record, SQLSMALLINT type,
 }
 
 
-#ifdef NOT_IMPLEMENTED_YET
 SQLRETURN SQL_API
 SQLBrowseConnectW(SQLHDBC hdbc, SQLWCHAR *in, SQLSMALLINT in_len,
                   SQLWCHAR *out, SQLSMALLINT out_max, SQLSMALLINT *out_len)
 {
-  NOT_IMPLEMENTED;
+  return set_conn_error(hdbc,MYERR_S1000,
+                        "Driver does not support this API", 0);
 }
 
 
+#ifdef NOT_IMPLEMENTED_YET
 //SQLDataSourcesW
 
 
