@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2000, 2012, Oracle and/or its affiliates. All rights reserved.
+  Copyright (c) 2000, 2013, Oracle and/or its affiliates. All rights reserved.
 
   The MySQL Connector/ODBC is licensed under the terms of the GPLv2
   <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
@@ -43,7 +43,7 @@ const SQLULEN sql_select_unlimited= (SQLULEN)-1;
   @param[in] dbc   The database connection
   @param[in] query The query to execute
 */
-SQLRETURN odbc_stmt(DBC FAR *dbc, const char *query)
+SQLRETURN odbc_stmt(DBC *dbc, const char *query)
 {
     SQLRETURN result= SQL_SUCCESS;
 
@@ -357,9 +357,9 @@ char *dupp_str(char *from,int length)
 */
 
 SQLRETURN copy_str_data(SQLSMALLINT HandleType, SQLHANDLE Handle,
-                        SQLCHAR FAR *rgbValue,
+                        SQLCHAR *rgbValue,
                         SQLSMALLINT cbValueMax,
-                        SQLSMALLINT FAR *pcbValue,char FAR *src)
+                        SQLSMALLINT *pcbValue, char *src)
 {
     SQLSMALLINT dummy;
 
@@ -1131,7 +1131,7 @@ SQLSMALLINT get_sql_data_type(STMT *stmt, MYSQL_FIELD *field, char *buff)
       (void)strmov(buff, field_is_binary ? "binary" : "char");
 
     return field_is_binary ? SQL_BINARY :
-      (field->charsetnr != stmt->dbc->ansi_charset_info->number ?
+      (stmt->dbc->unicode && field->charsetnr != stmt->dbc->ansi_charset_info->number ?
        SQL_WCHAR : SQL_CHAR);
 
   /*
@@ -1144,7 +1144,7 @@ SQLSMALLINT get_sql_data_type(STMT *stmt, MYSQL_FIELD *field, char *buff)
       (void)strmov(buff, field_is_binary ? "varbinary" : "varchar");
 
     return field_is_binary ? SQL_VARBINARY :
-      (field->charsetnr != stmt->dbc->ansi_charset_info->number ?
+      (stmt->dbc->unicode && field->charsetnr != stmt->dbc->ansi_charset_info->number ?
        SQL_WVARCHAR : SQL_VARCHAR);
 
   case MYSQL_TYPE_TINY_BLOB:
@@ -1152,7 +1152,7 @@ SQLSMALLINT get_sql_data_type(STMT *stmt, MYSQL_FIELD *field, char *buff)
       (void)strmov(buff, field_is_binary ? "tinyblob" : "tinytext");
 
     return field_is_binary ? SQL_LONGVARBINARY :
-      (field->charsetnr != stmt->dbc->ansi_charset_info->number ?
+      (stmt->dbc->unicode && field->charsetnr != stmt->dbc->ansi_charset_info->number ?
        SQL_WLONGVARCHAR : SQL_LONGVARCHAR);
 
   case MYSQL_TYPE_BLOB:
@@ -1160,7 +1160,7 @@ SQLSMALLINT get_sql_data_type(STMT *stmt, MYSQL_FIELD *field, char *buff)
       (void)strmov(buff, field_is_binary ? "blob" : "text");
 
     return field_is_binary ? SQL_LONGVARBINARY :
-      (field->charsetnr != stmt->dbc->ansi_charset_info->number ?
+      (stmt->dbc->unicode && field->charsetnr != stmt->dbc->ansi_charset_info->number ?
        SQL_WLONGVARCHAR : SQL_LONGVARCHAR);
 
   case MYSQL_TYPE_MEDIUM_BLOB:
@@ -1168,7 +1168,7 @@ SQLSMALLINT get_sql_data_type(STMT *stmt, MYSQL_FIELD *field, char *buff)
       (void)strmov(buff, field_is_binary ? "mediumblob" : "mediumtext");
 
     return field_is_binary ? SQL_LONGVARBINARY :
-      (field->charsetnr != stmt->dbc->ansi_charset_info->number ?
+      (stmt->dbc->unicode && field->charsetnr != stmt->dbc->ansi_charset_info->number ?
        SQL_WLONGVARCHAR : SQL_LONGVARCHAR);
 
   case MYSQL_TYPE_LONG_BLOB:
@@ -1176,7 +1176,7 @@ SQLSMALLINT get_sql_data_type(STMT *stmt, MYSQL_FIELD *field, char *buff)
       (void)strmov(buff, field_is_binary ? "longblob" : "longtext");
 
     return field_is_binary ? SQL_LONGVARBINARY :
-      (field->charsetnr != stmt->dbc->ansi_charset_info->number ?
+      (stmt->dbc->unicode && field->charsetnr != stmt->dbc->ansi_charset_info->number ?
        SQL_WLONGVARCHAR : SQL_LONGVARCHAR);
 
   case MYSQL_TYPE_ENUM:
@@ -2014,21 +2014,59 @@ int str_to_ts(SQL_TIMESTAMP_STRUCT *ts, const char *str, int len, int zeroToMin,
 
 my_bool str_to_time_st(SQL_TIME_STRUCT *ts, const char *str)
 { 
-    char buff[12],*to;
+    char buff[24],*to, *tokens[3] = {0, 0, 0};
+    int num= 0, int_hour=0, int_min= 0, int_sec= 0;
     SQL_TIME_STRUCT tmp_time;
 
     if ( !ts )
         ts= (SQL_TIME_STRUCT *) &tmp_time;
 
+    /* remember the position of the first numeric string */
+    tokens[0]= buff;
+
     for ( to= buff ; *str && to < buff+sizeof(buff)-1 ; ++str )
     {
         if (isdigit(*str))
             *to++= *str;
+        else if (num < 2)
+        {
+          /* 
+            terminate the string and remember the beginning of the 
+            new one only if the time component number is not out of
+            range
+          */
+          *to++= 0;
+          tokens[++num]= to;
+        }
+        else
+          /* We can leave the loop now */
+          break;
+    }
+    /* Put the final termination character */
+    *to= 0;
+
+    int_hour= tokens[0] ? atoi(tokens[0]) : 0;
+    int_min=  tokens[1] ? atoi(tokens[1]) : 0;
+    int_sec=  tokens[2] ? atoi(tokens[2]) : 0;
+
+    /* Convert seconds into minutes if necessary */
+    if (int_sec > 59)
+    {
+      int_min+= int_sec / 60;
+      int_sec= int_sec % 60;
     }
 
-    ts->hour=   digit(buff[0])*10+digit(buff[1]);
-    ts->minute= digit(buff[2])*10+digit(buff[3]);
-    ts->second= digit(buff[4])*10+digit(buff[5]);
+    /* Convert minutes into hours if necessary */
+    if (int_min > 59)
+    {
+      int_hour+= int_min / 60;
+      int_min= int_min % 60;
+    }
+
+    ts->hour   = (SQLUSMALLINT)(int_hour < 65536 ? int_hour : 65535);
+    ts->minute = (SQLUSMALLINT)int_min;
+    ts->second = (SQLUSMALLINT)int_sec;
+
     return 0;
 }
 
@@ -2143,7 +2181,7 @@ ulong str_to_time_as_long(const char *str, uint length)
   the server is up with mysql_ping (to force a reconnect)
 */
 
-int check_if_server_is_alive( DBC FAR *dbc )
+int check_if_server_is_alive( DBC *dbc )
 {
     time_t seconds= (time_t) time( (time_t*)0 );
     int result= 0;
@@ -2203,7 +2241,7 @@ my_bool dynstr_append_quoted_name(DYNAMIC_STRING *str, const char *name)
   @purpose : reset the db name to current_database()
 */
 
-my_bool reget_current_catalog(DBC FAR *dbc)
+my_bool reget_current_catalog(DBC *dbc)
 {
     x_free(dbc->database);
     dbc->database= NULL;
@@ -2336,11 +2374,11 @@ FILE *init_query_log(void)
 
 void end_query_log(FILE *query_log)
 {
-    if ( query_log )
-    {
-        fclose(query_log);
-        query_log= 0;
-    }
+  if ( query_log )
+  {
+      fclose(query_log);
+      query_log= 0;
+  }
 }
 
 
@@ -2714,7 +2752,7 @@ end:
                           MUST use numbegin to read the result string.
                           This should point to the LAST byte available.
                           (We fill in digits backwards.)
-  @param[in] numbegin     String pointer that will be set to the start of
+  @param[in,out] numbegin String pointer that will be set to the start of
                           the result string.
   @param[in] reqprec      Requested precision
   @param[in] reqscale     Requested scale
@@ -2810,7 +2848,9 @@ void sqlnum_to_str(SQL_NUMERIC_STRUCT *sqlnum, SQLCHAR *numstr,
       goto end;
     }
     if (*end == '.')
-      *end--= 0;
+    {
+      *end--= '\0';
+    }
     else
     {
       /* move the dec pt-- ??? */
@@ -2844,7 +2884,9 @@ void sqlnum_to_str(SQL_NUMERIC_STRUCT *sqlnum, SQLCHAR *numstr,
 
   /* finish up, handle auxilary fix-ups */
   if (!sqlnum->sign)
+  {
     *numstr--= '-';
+  }
   ++numstr;
   *numbegin= numstr;
 
@@ -2854,30 +2896,6 @@ end:
 }
 
 
-/**
- Detect if a statement is a SET NAMES statement.
-*/
-int is_set_names_statement(SQLCHAR *query)
-{
-  /* Skip leading spaces */
-  while (query && isspace(*query))
-    ++query;
-  return myodbc_casecmp((char *)query, "SET NAMES", 9) == 0;
-}
-
-
-/**
-Detect if a statement is a SELECT statement.
-*/
-int is_select_statement(SQLCHAR *query)
-{
-  /* Skip leading spaces */
-  while (query && isspace(*query))
-    ++query;
-  return myodbc_casecmp((char *)query, "SELECT", 6) == 0;
-}
-
-
 /**
   Adjust a pointer based on bind offset and bind type.
 
@@ -2919,7 +2937,7 @@ void *ptr_offset_adjust(void *ptr, SQLULEN *bind_offset_ptr,
 
   Returns new_value if operation was successful, -1 otherwise
  */
-SQLRETURN set_sql_select_limit(DBC FAR *dbc, SQLULEN new_value)
+SQLRETURN set_sql_select_limit(DBC *dbc, SQLULEN new_value)
 {
   char query[44];
   SQLRETURN rc;
@@ -3127,6 +3145,21 @@ SQLTypeMap SQL_TYPE_MAP_values[TYPE_MAP_SIZE]=
 
 };
 
+
+enum enum_field_types map_sql2mysql_type(SQLSMALLINT sql_type)
+{
+  int i;
+  for (i= 0; i < TYPE_MAP_SIZE; ++i)
+  {
+    if (SQL_TYPE_MAP_values[i].sql_type == sql_type)
+    {
+      return SQL_TYPE_MAP_values[i].mysql_type;
+    }
+  }
+
+  return MYSQL_TYPE_BLOB;
+}
+
 /**
   Gets the parameter index in the type map array
 
@@ -3310,9 +3343,12 @@ SQLUINTEGER proc_get_param_size(SQLCHAR *ptype, int len, int sql_type_index, SQL
       }
 
       break;
-
-    case MYSQL_TYPE_DATETIME:
     case MYSQL_TYPE_BIT:
+      param_size= proc_parse_sizes(start_pos, end_pos - start_pos, dec);
+
+      /* fall through*/
+    case MYSQL_TYPE_DATETIME:
+    
     case MYSQL_TYPE_TINY:
     case MYSQL_TYPE_SHORT:
     case MYSQL_TYPE_INT24:
@@ -3753,7 +3789,6 @@ MY_LIMIT_CLAUSE find_position4limit(CHARSET_INFO* cs, char *query, char * query_
   {
     result.begin= result.end= query_end;
   }
-  
 
   return result;
 }
@@ -3766,3 +3801,23 @@ BOOL myodbc_isspace(CHARSET_INFO* cs, const char * begin, const char *end)
 
   return ctype & _MY_SPC;
 }
+
+
+BOOL got_out_parameters(STMT *stmt)
+{
+  uint i;
+  DESCREC *iprec;
+
+  for(i= 0; i < stmt->param_count; ++i)
+  {
+    iprec= desc_get_rec(stmt->ipd, i, '\0');
+
+    if (iprec &&  (iprec->parameter_type == SQL_PARAM_INPUT_OUTPUT
+                || iprec->parameter_type == SQL_PARAM_OUTPUT))
+    {
+      return TRUE;
+    }
+  }
+
+  return FALSE;
+}
\ No newline at end of file
