@@ -1,4 +1,4 @@
-# Copyright (c) 2000, 2012, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2000, 2013, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 
 ##############################################################################
 #
-# mysql-connector-odbc 5.1 RPM specification
+# mysql-connector-odbc 5.2 RPM specification
 #
 ##############################################################################
 
@@ -48,7 +48,7 @@ Name:       mysql-connector-odbc-commercial
 %else
 Name:       mysql-connector-odbc
 %endif
-Summary:    An ODBC 5.1 driver for MySQL - driver package
+Summary:    An ODBC 5.2 driver for MySQL - driver package
 Group:      Applications/Databases
 Version:    @NUMERIC_VERSION@
 Release:    1
@@ -64,7 +64,7 @@ BuildRoot:  %{_tmppath}/%{name}-@VERSION@-build
 %if %{no_odbc_gui}
 %else
 %package setup
-Summary:    An ODBC 5.1 driver for MySQL - setup library
+Summary:    An ODBC 5.2 driver for MySQL - setup library
 Group:      Application/Databases
 Requires:   mysqlodbcrpmpack
 %endif
@@ -76,15 +76,14 @@ Requires:   mysqlodbcrpmpack
 ##############################################################################
 
 %description
-mysql-connector-odbc is an ODBC (3.50) level 0 (with level 1 and level
-2 features) driver for connecting an ODBC-aware application to MySQL.
-mysql-connector-odbc  works on Windows NT/2000/XP/2003, and most Unix
-platforms (incl. OSX and Linux). MySQL is a trademark of
-%{mysql_vendor}
+mysql-connector-odbc is an ODBC (3.51) driver for connecting an ODBC-aware
+application to MySQL. mysql-connector-odbc works on Windows XP/Vista/7/8
+Windows Server 2003/2008/2012, and most Unix platforms (incl. OSX and Linux).
+MySQL is a trademark of %{mysql_vendor}
 
-mysql-connector-odbc 5.1 is an enhanced version of MyODBC 2.50 to meet
-ODBC 3.5 specification. The driver is commonly referred to as
-'MySQL ODBC 5.1 Driver'.
+mysql-connector-odbc 5.2 is an enhanced version of mysql-connector-odbc 5.1.
+The driver comes in 2 flavours - ANSI and Unicode and commonly referred to as
+'MySQL ODBC 5.2 ANSI Driver' or 'MySQL ODBC 5.2 Unicode Driver' respectively.
 
 The MySQL software has Dual Licensing, which means you can use the MySQL
 software free of charge under the GNU General Public License
@@ -169,13 +168,16 @@ rm -vfr $RPM_BUILD_ROOT%{_prefix}/test
 # ----------------------------------------------------------------------
 
 %post 
-myodbc-installer -a -d -n "MySQL ODBC 5.1 Driver" -t "DRIVER=%{_libdir}/libmyodbc5.so"
+myodbc-installer -a -d -n "MySQL ODBC 5.2 Unicode Driver" -t "DRIVER=%{_libdir}/libmyodbc5w.so"
+myodbc-installer -a -d -n "MySQL ODBC 5.2 ANSI Driver"    -t "DRIVER=%{_libdir}/libmyodbc5a.so"
 
 %if %{no_odbc_gui}
 %else
 %post setup
-myodbc-installer -r -d -n "MySQL ODBC 5.1 Driver"
-myodbc-installer -a -d -n "MySQL ODBC 5.1 Driver" -t "DRIVER=%{_libdir}/libmyodbc5.so;SETUP=%{_libdir}/libmyodbc3S.so"
+myodbc-installer -r -d -n "MySQL ODBC 5.2 Unicode Driver"
+myodbc-installer -r -d -n "MySQL ODBC 5.2 ANSI Driver"
+myodbc-installer -a -d -n "MySQL ODBC 5.2 Unicode Driver" -t "DRIVER=%{_libdir}/libmyodbc5w.so;SETUP=%{_libdir}/libmyodbc3S.so"
+myodbc-installer -a -d -n "MySQL ODBC 5.2 ANSI Driver"    -t "DRIVER=%{_libdir}/libmyodbc5a.so;SETUP=%{_libdir}/libmyodbc3S.so"
 %endif
 
 # ----------------------------------------------------------------------
@@ -184,14 +186,17 @@ myodbc-installer -a -d -n "MySQL ODBC 5.1 Driver" -t "DRIVER=%{_libdir}/libmyodb
 
 # Removing the driver package, we simply orphan any related DSNs
 %preun
-myodbc-installer -r -d -n "MySQL ODBC 5.1 Driver"
+myodbc-installer -r -d -n "MySQL ODBC 5.2 Unicode Driver"
+myodbc-installer -r -d -n "MySQL ODBC 5.2 ANSI Driver"
 
 # Removing the setup RPM, downgrade the registration
 %if %{no_odbc_gui}
 %else
 %preun setup
-myodbc-installer -r -d -n "MySQL ODBC 5.1 Driver"
-myodbc-installer -a -d -n "MySQL ODBC 5.1 Driver" -t "DRIVER=%{_libdir}/libmyodbc5.so"
+myodbc-installer -r -d -n "MySQL ODBC 5.2 Unicode Driver"
+myodbc-installer -r -d -n "MySQL ODBC 5.2 ANSI Driver"
+myodbc-installer -a -d -n "MySQL ODBC 5.2 Unicode Driver" -t "DRIVER=%{_libdir}/libmyodbc5w.so"
+myodbc-installer -a -d -n "MySQL ODBC 5.2 ANSI Driver"    -t "DRIVER=%{_libdir}/libmyodbc5a.so"
 %endif
 
 ##############################################################################
@@ -203,8 +208,8 @@ myodbc-installer -a -d -n "MySQL ODBC 5.1 Driver" -t "DRIVER=%{_libdir}/libmyodb
 %files
 %defattr(-,root,root)
 %{_bindir}/myodbc-installer
-%{_libdir}/libmyodbc5.*
-%doc ChangeLog README README.debug INSTALL INSTALL.win Licenses_for_Third-Party_Components.txt
+%{_libdir}/libmyodbc5*.so
+%doc ChangeLog README README.debug INSTALL Licenses_for_Third-Party_Components.txt
 %if %{com_lic}
 %doc LICENSE.mysql
 %else
