@@ -29,20 +29,20 @@
 *********************************************************/
 DECLARE_TEST(my_init_table)
 {
-    SQLRETURN   rc;
+  SQLRETURN   rc;
 
-    ok_sql(hstmt, "DROP TABLE if exists my_demo_param");
+  ok_sql(hstmt, "DROP TABLE if exists my_demo_param");
 
-    /* commit the transaction */
-    rc = SQLEndTran(SQL_HANDLE_DBC, hdbc, SQL_COMMIT);
-    mycon(hdbc,rc);
+  /* commit the transaction */
+  rc = SQLEndTran(SQL_HANDLE_DBC, hdbc, SQL_COMMIT);
+  mycon(hdbc,rc);
 
-    /* create the table 'my_demo_param' */
-    ok_sql(hstmt, "CREATE TABLE my_demo_param(\
-                              id   int,\
-                              auto int primary key auto_increment,\
-                              name varchar(20),\
-                              timestamp timestamp)");
+  /* create the table 'my_demo_param' */
+  ok_sql(hstmt, "CREATE TABLE my_demo_param(\
+                            id   int,\
+                            auto int primary key auto_increment,\
+                            name varchar(20),\
+                            timestamp timestamp)");
 
   return OK;
 }
@@ -50,50 +50,50 @@ DECLARE_TEST(my_init_table)
 
 DECLARE_TEST(my_param_insert)
 {
-    SQLRETURN   rc;
-    SQLINTEGER  id;
-    char        name[50];
+  SQLRETURN   rc;
+  SQLINTEGER  id;
+  char        name[50];
 
-    /* prepare the insert statement with parameters */
-    rc = SQLPrepare(hstmt, (SQLCHAR *)"INSERT INTO my_demo_param(id,name) VALUES(?,?)",SQL_NTS);
-    mystmt(hstmt,rc);
+  /* prepare the insert statement with parameters */
+  rc = SQLPrepare(hstmt, (SQLCHAR *)"INSERT INTO my_demo_param(id,name) VALUES(?,?)",SQL_NTS);
+  mystmt(hstmt,rc);
 
-    /* now supply data to parameter 1 and 2 */
-    rc = SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT,
-                          SQL_C_LONG, SQL_INTEGER, 0,0,
-                          &id, 0, NULL);
-    mystmt(hstmt,rc);
+  /* now supply data to parameter 1 and 2 */
+  rc = SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT,
+                        SQL_C_LONG, SQL_INTEGER, 0,0,
+                        &id, 0, NULL);
+  mystmt(hstmt,rc);
 
-    rc = SQLBindParameter(hstmt, 2, SQL_PARAM_INPUT,
-                          SQL_C_CHAR, SQL_CHAR, 0,0,
-                          name, sizeof(name), NULL);
-    mystmt(hstmt,rc);
+  rc = SQLBindParameter(hstmt, 2, SQL_PARAM_INPUT,
+                        SQL_C_CHAR, SQL_CHAR, 0,0,
+                        name, sizeof(name), NULL);
+  mystmt(hstmt,rc);
 
-    /* now insert 10 rows of data */
-    for (id = 0; id < 10; id++)
-    {
-        sprintf(name,"MySQL%d",id);
+  /* now insert 10 rows of data */
+  for (id = 0; id < 10; id++)
+  {
+      sprintf(name,"MySQL%d",id);
 
-        rc = SQLExecute(hstmt);
-        mystmt(hstmt,rc);
-    }
+      rc = SQLExecute(hstmt);
+      mystmt(hstmt,rc);
+  }
 
-    /* Free statement param resorces */
-    rc = SQLFreeStmt(hstmt, SQL_RESET_PARAMS);
-    mystmt(hstmt,rc);
+  /* Free statement param resorces */
+  rc = SQLFreeStmt(hstmt, SQL_RESET_PARAMS);
+  mystmt(hstmt,rc);
 
-    /* Free statement cursor resorces */
-    rc = SQLFreeStmt(hstmt, SQL_CLOSE);
-    mystmt(hstmt,rc);
+  /* Free statement cursor resorces */
+  rc = SQLFreeStmt(hstmt, SQL_CLOSE);
+  mystmt(hstmt,rc);
 
-    /* commit the transaction */
-    rc = SQLEndTran(SQL_HANDLE_DBC, hdbc, SQL_COMMIT);
-    mycon(hdbc,rc);
+  /* commit the transaction */
+  rc = SQLEndTran(SQL_HANDLE_DBC, hdbc, SQL_COMMIT);
+  mycon(hdbc,rc);
 
-    /* Now fetch and verify the data */
-    ok_sql(hstmt, "SELECT * FROM my_demo_param");
+  /* Now fetch and verify the data */
+  ok_sql(hstmt, "SELECT * FROM my_demo_param");
 
-    is(10 == myresult(hstmt));
+  is(10 == myresult(hstmt));
 
   return OK;
 }
@@ -220,6 +220,7 @@ DECLARE_TEST(my_param_delete)
 }
 
 
+/*I really wonder what is this test about */
 DECLARE_TEST(tmysql_fix)
 {
   SQLRETURN rc;
@@ -257,59 +258,61 @@ DECLARE_TEST(tmysql_fix)
                   ('0000-00-00','0','0','0','0','0','0'),('2001-08-29','FIX','SQLT2',\
                   'ins1',NULL,NULL,'Error.  SQL cmd %s is not terminated or too long.')");
 
-    /* trace based */
+  /* trace based */
+  {
+    SQLSMALLINT pcpar,pccol,pfSqlType,pibScale,pfNullable;
+    SQLSMALLINT index;
+    SQLCHAR     td[30]="20010830163225";
+    SQLCHAR     node[30]="FIX";
+    SQLCHAR     tag[30]="SQLT2";
+    SQLCHAR     sqlname[30]="ins1";
+    SQLCHAR     sqlerr[30]="error";
+    SQLCHAR     fixerr[30]= "fixerr";
+    SQLCHAR     progerr[30]="progerr";
+    SQLULEN     pcbParamDef;
+
+    SQLFreeStmt(hstmt,SQL_CLOSE);
+    rc = SQLPrepare(hstmt,
+      (SQLCHAR *)"insert into tmysql_err (TD, NODE, TAG, SQLNAME, SQL_ERR,"
+                 "FIX_ERR, PROG_ERR) values (?, ?, ?, ?, ?, ?, ?)", 103);
+    mystmt(hstmt,rc);
+
+    rc = SQLNumParams(hstmt,&pcpar);
+    mystmt(hstmt,rc);
+
+    rc = SQLNumResultCols(hstmt,&pccol);
+    mystmt(hstmt,rc);
+
+    for (index=1; index <= pcpar; index++)
     {
-        SQLSMALLINT pcpar,pccol,pfSqlType,pibScale,pfNullable;
-        SQLSMALLINT index;
-        SQLCHAR     td[30]="20010830163225";
-        SQLCHAR     node[30]="FIX";
-        SQLCHAR     tag[30]="SQLT2";
-        SQLCHAR     sqlname[30]="ins1";
-        SQLCHAR     sqlerr[30]="error";
-        SQLCHAR     fixerr[30]= "fixerr";
-        SQLCHAR     progerr[30]="progerr";
-        SQLULEN     pcbParamDef;
+      rc = SQLDescribeParam(hstmt,index,&pfSqlType,&pcbParamDef,&pibScale,&pfNullable);
+      mystmt(hstmt,rc);
 
-        SQLFreeStmt(hstmt,SQL_CLOSE);
-        rc = SQLPrepare(hstmt, (SQLCHAR *)"insert into tmysql_err (TD, NODE, TAG, SQLNAME, SQL_ERR, FIX_ERR, PROG_ERR)\
-                         values (?, ?, ?, ?, ?, ?, ?)",200);
-        mystmt(hstmt,rc);
-
-        rc = SQLNumParams(hstmt,&pcpar);
-        mystmt(hstmt,rc);
-
-        rc = SQLNumResultCols(hstmt,&pccol);
-        mystmt(hstmt,rc);
-
-        for (index=1; index <= pcpar; index++)
-        {
-            rc = SQLDescribeParam(hstmt,index,&pfSqlType,&pcbParamDef,&pibScale,&pfNullable);
-            mystmt(hstmt,rc);
-
-            printMessage("descparam[%d]:%d,%d,%d,%d\n",index,pfSqlType,pcbParamDef,pibScale,pfNullable);
-        }
-
-        rc = SQLBindParameter(hstmt,1,SQL_PARAM_INPUT,11,12,0,0,td,100,0);
-        mystmt(hstmt,rc);
-
-        rc = SQLBindParameter(hstmt,2,SQL_PARAM_INPUT,1,12,0,0,node,100,0);
-        mystmt(hstmt,rc);
-
-        rc = SQLBindParameter(hstmt,3,SQL_PARAM_INPUT,1,12,0,0,tag,100,0);
-        mystmt(hstmt,rc);
-        rc = SQLBindParameter(hstmt,4,SQL_PARAM_INPUT,1,12,0,0,sqlname,100,0);
-        mystmt(hstmt,rc);
-        rc = SQLBindParameter(hstmt,5,SQL_PARAM_INPUT,1,12,0,0,sqlerr,0,0);
-        mystmt(hstmt,rc);
-        rc = SQLBindParameter(hstmt,6,SQL_PARAM_INPUT,1,12,0,0,fixerr,0,0);
-        mystmt(hstmt,rc);
-        rc = SQLBindParameter(hstmt,7,SQL_PARAM_INPUT,1,12,0,0,progerr,0,0);
-        mystmt(hstmt,rc);
-
-        rc = SQLExecute(hstmt);
-        mystmt(hstmt,rc);
+      printMessage("descparam[%d]:%d,%d,%d,%d\n",index,pfSqlType,pcbParamDef,pibScale,pfNullable);
     }
 
+    /* TODO: C and SQL types as numeric consts. Splendid.*/
+    rc = SQLBindParameter(hstmt,1,SQL_PARAM_INPUT,11,12,0,0,td,100,0);
+    mystmt(hstmt,rc);
+
+    rc = SQLBindParameter(hstmt,2,SQL_PARAM_INPUT,1,12,0,0,node,100,0);
+    mystmt(hstmt,rc);
+
+    rc = SQLBindParameter(hstmt,3,SQL_PARAM_INPUT,1,12,0,0,tag,100,0);
+    mystmt(hstmt,rc);
+    rc = SQLBindParameter(hstmt,4,SQL_PARAM_INPUT,1,12,0,0,sqlname,100,0);
+    mystmt(hstmt,rc);
+    rc = SQLBindParameter(hstmt,5,SQL_PARAM_INPUT,1,12,0,0,sqlerr,0,0);
+    mystmt(hstmt,rc);
+    rc = SQLBindParameter(hstmt,6,SQL_PARAM_INPUT,1,12,0,0,fixerr,0,0);
+    mystmt(hstmt,rc);
+    rc = SQLBindParameter(hstmt,7,SQL_PARAM_INPUT,1,12,0,0,progerr,0,0);
+    mystmt(hstmt,rc);
+
+    rc = SQLExecute(hstmt);
+    mystmt(hstmt,rc);
+  }
+
   ok_sql(hstmt, "DROP TABLE IF EXISTS tmysql_err");
 
   return OK;
@@ -934,6 +937,455 @@ DECLARE_TEST(t_bug59772)
 }
 
 
+DECLARE_TEST(t_odbcoutparams)
+{
+  SQLSMALLINT ncol, i;
+  SQLINTEGER  par[3]= {10, 20, 30}, val;
+  SQLLEN      len;
+  SQLSMALLINT type[]= {SQL_PARAM_INPUT, SQL_PARAM_OUTPUT, SQL_PARAM_INPUT_OUTPUT};
+  SQLCHAR     str[20]= "initial value", buff[20];
+
+  ok_sql(hstmt, "DROP PROCEDURE IF EXISTS t_odbcoutparams");
+  ok_sql(hstmt, "CREATE PROCEDURE t_odbcoutparams("
+                "  IN p_in INT, "
+                "  OUT p_out INT, "
+                "  INOUT p_inout INT) "
+                "BEGIN "
+                "  SET p_in = p_in*10, p_out = (p_in+p_inout)*10, p_inout = p_inout*10; "
+                "END");
+
+
+
+  for (i=0; i < sizeof(par)/sizeof(SQLINTEGER); ++i)
+  {
+    ok_stmt(hstmt, SQLBindParameter(hstmt, i+1, type[i], SQL_C_LONG, SQL_INTEGER, 0,
+      0, &par[i], 0, NULL));
+  }
+
+  ok_sql(hstmt, "CALL t_odbcoutparams(?, ?, ?)");
+
+  ok_stmt(hstmt, SQLNumResultCols(hstmt,&ncol));
+  is_num(ncol, 2);
+
+  is_num(par[1], 1300);
+  is_num(par[2], 300);
+  
+  /* Only 1 row always - we still can get them as a result */
+  ok_stmt(hstmt, SQLFetch(hstmt));
+  is_num(my_fetch_int(hstmt, 1), 1300);
+  is_num(my_fetch_int(hstmt, 2), 300);
+  expect_stmt(hstmt, SQLFetch(hstmt), SQL_NO_DATA);
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  ok_sql(hstmt, "DROP PROCEDURE t_odbcoutparams");
+  ok_sql(hstmt, "CREATE PROCEDURE t_odbcoutparams("
+                "  IN p_in INT, "
+                "  OUT p_out INT, "
+                "  INOUT p_inout INT) "
+                "BEGIN "
+                "  SELECT p_in, p_out, p_inout; "
+                "  SET p_in = 300, p_out = 100, p_inout = 200; "
+                "END");
+  ok_sql(hstmt, "CALL t_odbcoutparams(?, ?, ?)");
+  /* rs-1 */
+  ok_stmt(hstmt, SQLFetch(hstmt));
+  ok_stmt(hstmt, SQLNumResultCols(hstmt,&ncol));
+  is_num(ncol, 3);
+
+  is_num(my_fetch_int(hstmt, 1), 10);
+  /* p_out does not have value at the moment */
+  ok_stmt(hstmt, SQLGetData(hstmt, 2, SQL_INTEGER, &val, 0, &len));
+  is_num(len, SQL_NULL_DATA);
+  is_num(my_fetch_int(hstmt, 3), 300);
+
+  expect_stmt(hstmt, SQLFetch(hstmt), SQL_NO_DATA);
+  ok_stmt(hstmt, SQLMoreResults(hstmt));
+
+  is_num(par[1], 100);
+  is_num(par[2], 200);
+
+  /* SP execution status */
+  ok_stmt(hstmt, SQLMoreResults(hstmt));
+
+  expect_stmt(hstmt, SQLMoreResults(hstmt), SQL_NO_DATA);
+
+  ok_sql(hstmt, "DROP PROCEDURE t_odbcoutparams");
+  ok_sql(hstmt, "CREATE PROCEDURE t_odbcoutparams("
+                "  OUT p_out VARCHAR(19), "
+                "  IN p_in INT, "
+                "  INOUT p_inout INT) "
+                "BEGIN "
+                "  SET p_in = 300, p_out := 'This is OUT param', p_inout = 200; "
+                "  SELECT p_inout, p_in, substring(p_out, 9);"
+                "END");
+  
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 1, SQL_PARAM_OUTPUT, SQL_C_CHAR, SQL_VARCHAR, 0,
+      0, str, sizeof(str)/sizeof(SQLCHAR), NULL));
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 2, SQL_PARAM_INPUT, SQL_C_LONG, SQL_INTEGER, 0,
+      0, &par[0], 0, NULL));
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 3, SQL_PARAM_INPUT_OUTPUT, SQL_C_LONG,
+      SQL_INTEGER, 0, 0, &par[1], 0, NULL));
+
+  ok_sql(hstmt, "CALL t_odbcoutparams(?, ?, ?)");
+  /* rs-1 */
+  ok_stmt(hstmt, SQLNumResultCols(hstmt,&ncol));
+  is_num(ncol, 3);
+  
+  ok_stmt(hstmt, SQLFetch(hstmt));
+  is_num(my_fetch_int(hstmt, 1), 200);
+  is_num(my_fetch_int(hstmt, 2), 300);
+  is_str(my_fetch_str(hstmt, buff, 3), "OUT param", 10);
+
+  ok_stmt(hstmt, SQLMoreResults(hstmt));
+  is_str(str, "This is OUT param", 18);
+  is_num(par[1], 200);
+
+  ok_stmt(hstmt, SQLFetch(hstmt));
+  is_str(my_fetch_str(hstmt, buff, 1), "This is OUT param", 18);
+  is_num(my_fetch_int(hstmt, 2), 200);
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  ok_sql(hstmt, "DROP PROCEDURE t_odbcoutparams");
+
+  return OK;
+}
+
+
+DECLARE_TEST(t_bug14501952)
+{
+  SQLSMALLINT ncol;
+  SQLLEN      len= 0;
+  SQLCHAR     blobValue[50]= "initial value", buff[100];
+
+  ok_sql(hstmt, "DROP PROCEDURE IF EXISTS bug14501952");
+  ok_sql(hstmt, "CREATE PROCEDURE bug14501952 (INOUT param1 BLOB)\
+                  BEGIN\
+                    SET param1= 'this is blob value from SP ';\
+                  END;");
+
+
+
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT_OUTPUT,
+    SQL_C_BINARY, SQL_LONGVARBINARY, 50, 0, &blobValue, sizeof(blobValue),
+    &len));
+
+  ok_sql(hstmt, "CALL bug14501952(?)");
+
+  is_str(blobValue, "this is blob value from SP ", 27);
+  ok_stmt(hstmt, SQLNumResultCols(hstmt,&ncol));
+  is_num(ncol, 1);
+
+  /* Only 1 row always - we still can get them as a result */
+  ok_stmt(hstmt, SQLFetch(hstmt));
+  ok_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_BINARY, buff, sizeof(buff),
+                            &len));
+  is_str(buff, blobValue, 27);
+  
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  ok_sql(hstmt, "DROP PROCEDURE bug14501952");
+
+  return OK;
+}
+
+
+/* Bug#14563386 More than one BLOB(or any big data types) OUT param caused crash
+ */
+DECLARE_TEST(t_bug14563386)
+{
+  SQLSMALLINT ncol;
+  SQLLEN      len= 0, len1= 0;
+  SQLCHAR     blobValue[50]= "initial value", buff[100],
+              binValue[50]= "varbinary init value";
+
+  ok_sql(hstmt, "DROP PROCEDURE IF EXISTS b14563386");
+  ok_sql(hstmt, "CREATE PROCEDURE b14563386 (INOUT blob_param \
+                        BLOB, INOUT bin_param LONG VARBINARY)\
+                  BEGIN\
+                    SET blob_param = ' BLOB! ';\
+                    SET bin_param = ' LONG VARBINARY ';\
+                  END;");
+
+
+
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT_OUTPUT,
+    SQL_C_BINARY, SQL_LONGVARBINARY, 50, 0, &blobValue, sizeof(blobValue),
+    &len));
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 2, SQL_PARAM_INPUT_OUTPUT,
+    SQL_C_BINARY, SQL_LONGVARBINARY, 50, 0, &binValue, sizeof(binValue),
+    &len1));
+  ok_sql(hstmt, "CALL b14563386(?, ?)");
+
+  is_str(blobValue, " BLOB! ", 7);
+  is_str(binValue, " LONG VARBINARY ", 16);
+  ok_stmt(hstmt, SQLNumResultCols(hstmt,&ncol));
+  is_num(ncol, 2);
+
+  /* Only 1 row always - we still can get them as a result */
+  ok_stmt(hstmt, SQLFetch(hstmt));
+  ok_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_BINARY, buff, sizeof(buff),
+                            &len));
+  is_str(buff, blobValue, 7);
+  ok_stmt(hstmt, SQLGetData(hstmt, 2, SQL_C_BINARY, buff, sizeof(buff),
+                            &len));
+  is_str(buff, binValue, 16);
+  
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  ok_sql(hstmt, "DROP PROCEDURE b14563386");
+
+  return OK;
+}
+
+
+/* Bug#14551229(could not repeat) Procedure with signed out parameter */
+DECLARE_TEST(t_bug14551229)
+{
+  SQLINTEGER param, value;
+
+  ok_sql(hstmt, "DROP PROCEDURE IF EXISTS b14551229");
+  ok_sql(hstmt, "CREATE PROCEDURE b14551229 (OUT param INT)\
+                  BEGIN\
+                    SELECT -1 into param from dual;\
+                  END;");
+
+
+
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 1, SQL_PARAM_OUTPUT,
+    SQL_C_SLONG, SQL_INTEGER, 50, 0, &param, 0, 0));
+
+  ok_sql(hstmt, "CALL b14551229(?)");
+
+  is_num(param, -1);
+
+  /* Only 1 row always - we still can get them as a result */
+  ok_stmt(hstmt, SQLFetch(hstmt));
+  ok_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_SLONG, &value, 0, 0));
+  is_num(value, -1);
+  
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  ok_sql(hstmt, "DROP PROCEDURE b14551229");
+
+  return OK;
+}
+
+
+/* Bug#14560916 ASSERT for INOUT parameter of BIT(10) type */
+DECLARE_TEST(t_bug14560916)
+{
+  char        param[2]={1,1};
+  SQLINTEGER  value;
+  SQLLEN      len= 0;
+
+  ok_sql(hstmt, "DROP PROCEDURE IF EXISTS b14560916");
+
+  ok_sql(hstmt, "DROP TABLE IF EXISTS bug14560916");
+  ok_sql(hstmt, "CREATE TABLE bug14560916 (a BIT(10))");
+  ok_sql(hstmt, "INSERT INTO bug14560916  values(b'1001000001')");
+
+  ok_sql(hstmt, "CREATE PROCEDURE b14560916 (INOUT param bit(10))\
+                  BEGIN\
+                    SELECT a INTO param FROM bug14560916;\
+                  END;");
+
+
+
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT_OUTPUT,
+    SQL_C_CHAR, SQL_CHAR, 0, 0, &param, sizeof(param), &len));
+
+  /* Parameter is used to make sure that ssps will be used */
+  ok_sql(hstmt, "select a from bug14560916 where ? OR 1");
+  ok_stmt(hstmt, SQLFetch(hstmt));
+  ok_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_SLONG, &value, 0, 0));
+  is_num(value, 577);
+  ok_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_BINARY, &param, sizeof(param),
+                            &len));
+  is_num(len, 2);
+  is_str(param, "\2A", 2);
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+  
+
+  ok_sql(hstmt, "CALL b14560916(?)");
+
+  is_num(len, 2);
+  is_str(param, "\2A", 2);
+
+  /* Only 1 row always - we still can get them as a result */
+  ok_stmt(hstmt, SQLFetch(hstmt));
+  ok_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_SLONG, &value, 0, 0));
+  is_num(value, 577);
+  param[0]= param[1]= 1;
+  ok_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_BINARY, &param, sizeof(param),
+                            &len));
+  is_num(len, 2);
+  is_str(param, "\2A", 2);
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  ok_sql(hstmt, "DROP PROCEDURE b14560916");
+
+  return OK;
+}
+
+
+/* Bug#14586094 Crash while executing SP having blob and varchar OUT parameters
+  (could not repeat)
+ */
+DECLARE_TEST(t_bug14586094)
+{
+  SQLSMALLINT ncol;
+  SQLLEN      len= SQL_NTS, len1= SQL_NTS;
+  SQLCHAR     blobValue[50]= {0}/*"initial value"*/, buff[101],
+              vcValue[101]= "varchar init value";
+
+  ok_sql(hstmt, "DROP PROCEDURE IF EXISTS b14586094");
+  ok_sql(hstmt, "CREATE PROCEDURE b14586094 (INOUT blob_param \
+                    BLOB(50), INOUT vc_param VARCHAR(100))\
+                  BEGIN\
+                    SET blob_param = ' BLOB! ';\
+                    SET vc_param = 'varchar';\
+                  END;");
+
+
+
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT_OUTPUT,
+    SQL_C_BINARY, SQL_LONGVARBINARY, 50, 0, &blobValue, sizeof(blobValue),
+    &len));
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 2, SQL_PARAM_INPUT_OUTPUT,
+    SQL_C_CHAR, SQL_VARCHAR, 50, 0, &vcValue, sizeof(vcValue), &len1));
+  ok_sql(hstmt, "CALL b14586094(?, ?)");
+
+  is_str(blobValue, " BLOB! ", 7);
+  is_str(vcValue, "varchar", 9);
+  ok_stmt(hstmt, SQLNumResultCols(hstmt,&ncol));
+  is_num(ncol, 2);
+
+  /* Only 1 row always - we still can get them as a result */
+  ok_stmt(hstmt, SQLFetch(hstmt));
+  ok_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_BINARY, buff, sizeof(buff),
+                            &len));
+  is_str(buff, blobValue, 7);
+  ok_stmt(hstmt, SQLGetData(hstmt, 2, SQL_C_CHAR, buff, sizeof(buff),
+                            &len));
+  is_str(buff, vcValue, 9);
+  
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  ok_sql(hstmt, "DROP PROCEDURE b14586094");
+
+  return OK;
+}
+
+
+DECLARE_TEST(t_longtextoutparam)
+{
+  SQLSMALLINT ncol;
+  SQLLEN      len= 0;
+  SQLCHAR     blobValue[50]= "initial value", buff[100];
+
+  ok_sql(hstmt, "DROP PROCEDURE IF EXISTS t_longtextoutparam");
+  ok_sql(hstmt, "CREATE PROCEDURE t_longtextoutparam (INOUT param1 LONGTEXT)\
+                  BEGIN\
+                    SET param1= 'this is LONGTEXT value from SP ';\
+                  END;");
+
+
+
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT_OUTPUT,
+    SQL_C_BINARY, SQL_LONGVARBINARY, 50, 0, &blobValue, sizeof(blobValue),
+    &len));
+
+  ok_sql(hstmt, "CALL t_longtextoutparam(?)");
+
+  is_str(blobValue, "this is LONGTEXT value from SP ", 32);
+  ok_stmt(hstmt, SQLNumResultCols(hstmt,&ncol));
+  is_num(ncol, 1);
+
+  /* Only 1 row always - we still can get them as a result */
+  ok_stmt(hstmt, SQLFetch(hstmt));
+  ok_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_CHAR, buff, sizeof(buff),
+                            &len));
+  is_str(buff, blobValue, 32);
+  
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  ok_sql(hstmt, "DROP PROCEDURE t_longtextoutparam");
+
+  return OK;
+}
+
+
+/* 
+  Bug# 16613308/53891 ODBC driver not parsing comments correctly
+  TODO: fix for NO_SSPS=1
+ */
+DECLARE_TEST(t_bug53891)
+{
+  int c1= 2;
+  
+  DECLARE_BASIC_HANDLES(henv1, hdbc1, hstmt1);
+
+  /* Connect with SSPS enabled */
+  alloc_basic_handles_with_opt(&henv1, &hdbc1, &hstmt1, NULL, NULL, NULL, 
+                               NULL, "NO_SSPS=0");
+  /* Try a simple query without parameters */
+  ok_stmt(hstmt1, SQLPrepare(hstmt1, 
+                             "/* a question mark ? must be ignored */"\
+                             " SELECT 1", SQL_NTS));
+
+  ok_stmt(hstmt1, SQLExecute(hstmt1));
+  ok_stmt(hstmt1, SQLFetch(hstmt1));
+  is_num(my_fetch_int(hstmt1, 1), 1);
+
+  ok_stmt(hstmt1, SQLFreeStmt(hstmt1, SQL_CLOSE));
+
+  /* Try a query with parameters */
+  ok_stmt(hstmt1, SQLPrepare(hstmt1, 
+                             "/* a question mark ? must be ignored */"\
+                             " SELECT ?", SQL_NTS));
+
+  ok_stmt(hstmt1, SQLBindParameter(hstmt1, 1, SQL_PARAM_INPUT, SQL_C_LONG,
+                                  SQL_INTEGER, 0, 0, &c1, 0, NULL));
+  ok_stmt(hstmt1, SQLExecute(hstmt1));
+  ok_stmt(hstmt1, SQLFetch(hstmt1));
+  is_num(my_fetch_int(hstmt1, 1), c1);
+
+  free_basic_handles(&henv1, &hdbc1, &hstmt1);
+
+  /* Connect with SSPS disabled */
+  alloc_basic_handles_with_opt(&henv1, &hdbc1, &hstmt1, NULL, NULL, NULL, 
+                               NULL, "NO_SSPS=1");
+  /* Try a simple query without parameters */
+  ok_stmt(hstmt1, SQLPrepare(hstmt1, 
+                             "/* a question mark ? must be ignored */"\
+                             " SELECT 1", SQL_NTS));
+
+  ok_stmt(hstmt1, SQLExecute(hstmt1));
+  ok_stmt(hstmt1, SQLFetch(hstmt1));
+  is_num(my_fetch_int(hstmt1, 1), 1);
+
+  ok_stmt(hstmt1, SQLFreeStmt(hstmt1, SQL_CLOSE));
+
+  /* Try a query with parameters */
+  ok_stmt(hstmt1, SQLPrepare(hstmt1, 
+                             "/* a question mark ? must be ignored */"\
+                             " SELECT ?", SQL_NTS));
+
+  ok_stmt(hstmt1, SQLBindParameter(hstmt1, 1, SQL_PARAM_INPUT, SQL_C_LONG,
+                                  SQL_INTEGER, 0, 0, &c1, 0, NULL));
+  ok_stmt(hstmt1, SQLExecute(hstmt1));
+  ok_stmt(hstmt1, SQLFetch(hstmt1));
+  is_num(my_fetch_int(hstmt1, 1), c1);
+
+  free_basic_handles(&henv1, &hdbc1, &hstmt1);
+  return OK;
+}
+
+
 BEGIN_TESTS
   ADD_TEST(my_init_table)
   ADD_TEST(my_param_insert)
@@ -948,6 +1400,14 @@ BEGIN_TESTS
   ADD_TEST(t_bug49029)
   ADD_TEST(t_bug56804)
   ADD_TEST(t_bug59772)
+  ADD_TEST(t_odbcoutparams)
+  ADD_TEST(t_bug14501952)
+  ADD_TEST(t_bug14563386)
+  ADD_TEST(t_bug14551229)
+  ADD_TEST(t_bug14560916)
+  ADD_TEST(t_bug14586094)
+  ADD_TEST(t_longtextoutparam)
+  ADD_TEST(t_bug53891)
 END_TESTS
 
 
