@@ -1,29 +1,25 @@
 /*
-  Copyright 2003-2007 MySQL AB
-
-  The MySQL Connector/ODBC is licensed under the terms of the
-  GPL, like most MySQL Connectors. There are special exceptions
-  to the terms and conditions of the GPL as it is applied to
-  this software, see the FLOSS License Exception available on
-  mysql.com.
+  Copyright (c) 2003, 2013, Oracle and/or its affiliates. All rights reserved.
 
+  The MySQL Connector/ODBC is licensed under the terms of the GPLv2
+  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
+  MySQL Connectors. There are special exceptions to the terms and
+  conditions of the GPLv2 as it is applied to this software, see the
+  FLOSS License Exception
+  <http://www.mysql.com/about/legal/licensing/foss-exception.html>.
+  
   This program is free software; you can redistribute it and/or modify
-  it under the terms of version 2 of the GNU General Public License as
-  published by the Free Software Foundation.
-
-  There are special exceptions to the terms and conditions of the GPL
-  as it is applied to this software. View the full text of the exception
-  in file LICENSE.exceptions in the top-level directory of this software
-  distribution.
-
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+  it under the terms of the GNU General Public License as published
+  by the Free Software Foundation; version 2 of the License.
+  
+  This program is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
+  for more details.
+  
+  You should have received a copy of the GNU General Public License along
+  with this program; if not, write to the Free Software Foundation, Inc.,
+  51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
 #include "odbctap.h"
@@ -317,11 +313,11 @@ DECLARE_TEST(t_relative2)
 
   ok_sql(hstmt, "DROP TABLE IF EXISTS t_relative2");
 
-    rc = SQLExecDirect(hstmt,"create table t_relative2(id int)",SQL_NTS);
-    mystmt(hstmt,rc);
+  ok_sql(hstmt, "create table t_relative2(id int)");
 
-    rc = SQLPrepare(hstmt,"insert into t_relative2 values(?)",SQL_NTS);
-    mystmt(hstmt,rc);
+  ok_stmt(hstmt, SQLPrepare(hstmt,
+                            (SQLCHAR *) "insert into t_relative2 values(?)",
+                            SQL_NTS));
 
     rc = SQLBindParameter(hstmt,1,SQL_PARAM_INPUT, SQL_C_ULONG,
                           SQL_INTEGER,0,0,&i,0,NULL);
@@ -346,7 +342,7 @@ DECLARE_TEST(t_relative2)
     rc = SQLSetStmtAttr(hstmt,SQL_ATTR_ROWS_FETCHED_PTR,&nrows,0);
     mystmt(hstmt,rc);
 
-    rc = SQLExecDirect(hstmt,"select * from t_relative2",SQL_NTS);
+    ok_sql(hstmt, "select * from t_relative2");
     mystmt(hstmt,rc);
 
     rc = SQLBindCol(hstmt,1,SQL_C_LONG,&iarray,0,NULL);
@@ -561,26 +557,14 @@ DECLARE_TEST(t_rows_fetched_ptr)
 
   ok_sql(hstmt, "DROP TABLE IF EXISTS t_rows_fetched_ptr");
 
-    rc = SQLExecDirect(hstmt,"create table t_rows_fetched_ptr(a int)",SQL_NTS);
-    mystmt(hstmt,rc);
+  ok_sql(hstmt,"create table t_rows_fetched_ptr(a int)");
 
-    rc = SQLExecDirect(hstmt,"insert into t_rows_fetched_ptr values(0)",SQL_NTS);
-    mystmt(hstmt,rc);
-
-    rc = SQLExecDirect(hstmt,"insert into t_rows_fetched_ptr values(1)",SQL_NTS);
-    mystmt(hstmt,rc);
-
-    rc = SQLExecDirect(hstmt,"insert into t_rows_fetched_ptr values(2)",SQL_NTS);
-    mystmt(hstmt,rc);
-
-    rc = SQLExecDirect(hstmt,"insert into t_rows_fetched_ptr values(3)",SQL_NTS);
-    mystmt(hstmt,rc);
-
-    rc = SQLExecDirect(hstmt,"insert into t_rows_fetched_ptr values(4)",SQL_NTS);
-    mystmt(hstmt,rc);
-
-    rc = SQLExecDirect(hstmt,"insert into t_rows_fetched_ptr values(5)",SQL_NTS);
-    mystmt(hstmt,rc);
+  ok_sql(hstmt,"insert into t_rows_fetched_ptr values(0)");
+  ok_sql(hstmt,"insert into t_rows_fetched_ptr values(1)");
+  ok_sql(hstmt,"insert into t_rows_fetched_ptr values(2)");
+  ok_sql(hstmt,"insert into t_rows_fetched_ptr values(3)");
+  ok_sql(hstmt,"insert into t_rows_fetched_ptr values(4)");
+  ok_sql(hstmt,"insert into t_rows_fetched_ptr values(5)");
 
     rowsSize= 1;
     rc = SQLSetStmtAttr(hstmt, SQL_ATTR_ROW_ARRAY_SIZE, (SQLPOINTER)rowsSize, 0);
@@ -589,8 +573,7 @@ DECLARE_TEST(t_rows_fetched_ptr)
     rc = SQLSetStmtAttr(hstmt, SQL_ATTR_ROWS_FETCHED_PTR, &rowsFetched, 0);
     mystmt(hstmt,rc);
 
-    rc = SQLExecDirect(hstmt, "SELECT * FROM t_rows_fetched_ptr",SQL_NTS);
-    mystmt(hstmt,rc);
+    ok_sql(hstmt, "SELECT * FROM t_rows_fetched_ptr");
 
     i= 0;
     rc = SQLFetchScroll(hstmt,SQL_FETCH_NEXT,0);
@@ -611,7 +594,7 @@ DECLARE_TEST(t_rows_fetched_ptr)
     rc = SQLSetStmtAttr(hstmt, SQL_ATTR_ROWS_FETCHED_PTR, &rowsFetched, 0);
     mystmt(hstmt,rc);
 
-    rc = SQLExecDirect(hstmt, "SELECT * FROM t_rows_fetched_ptr",SQL_NTS);
+    ok_sql(hstmt, "SELECT * FROM t_rows_fetched_ptr");
     mystmt(hstmt,rc);
 
     i= 0;
@@ -633,7 +616,7 @@ DECLARE_TEST(t_rows_fetched_ptr)
     rc = SQLSetStmtAttr(hstmt, SQL_ATTR_ROWS_FETCHED_PTR, &rowsFetched, 0);
     mystmt(hstmt,rc);
 
-    rc = SQLExecDirect(hstmt, "SELECT * FROM t_rows_fetched_ptr",SQL_NTS);
+    ok_sql(hstmt, "SELECT * FROM t_rows_fetched_ptr");
     mystmt(hstmt,rc);
 
     i= 0;
@@ -655,7 +638,7 @@ DECLARE_TEST(t_rows_fetched_ptr)
     rc = SQLSetStmtAttr(hstmt, SQL_ATTR_ROWS_FETCHED_PTR, &rowsFetched, 0);
     mystmt(hstmt,rc);
 
-    rc = SQLExecDirect(hstmt, "SELECT * FROM t_rows_fetched_ptr",SQL_NTS);
+    ok_sql(hstmt, "SELECT * FROM t_rows_fetched_ptr");
     mystmt(hstmt,rc);
 
     rc = SQLFetch(hstmt);
@@ -695,26 +678,13 @@ DECLARE_TEST(t_rows_fetched_ptr1)
 
   ok_sql(hstmt, "DROP TABLE IF EXISTS t_rows_fetched_ptr");
 
-    rc = SQLExecDirect(hstmt,"create table t_rows_fetched_ptr(a int)",SQL_NTS);
-    mystmt(hstmt,rc);
-
-    rc = SQLExecDirect(hstmt,"insert into t_rows_fetched_ptr values(0)",SQL_NTS);
-    mystmt(hstmt,rc);
-
-    rc = SQLExecDirect(hstmt,"insert into t_rows_fetched_ptr values(1)",SQL_NTS);
-    mystmt(hstmt,rc);
-
-    rc = SQLExecDirect(hstmt,"insert into t_rows_fetched_ptr values(2)",SQL_NTS);
-    mystmt(hstmt,rc);
-
-    rc = SQLExecDirect(hstmt,"insert into t_rows_fetched_ptr values(3)",SQL_NTS);
-    mystmt(hstmt,rc);
-
-    rc = SQLExecDirect(hstmt,"insert into t_rows_fetched_ptr values(4)",SQL_NTS);
-    mystmt(hstmt,rc);
-
-    rc = SQLExecDirect(hstmt,"insert into t_rows_fetched_ptr values(5)",SQL_NTS);
-    mystmt(hstmt,rc);
+  ok_sql(hstmt, "create table t_rows_fetched_ptr(a int)");
+  ok_sql(hstmt, "insert into t_rows_fetched_ptr values(0)");
+  ok_sql(hstmt, "insert into t_rows_fetched_ptr values(1)");
+  ok_sql(hstmt, "insert into t_rows_fetched_ptr values(2)");
+  ok_sql(hstmt, "insert into t_rows_fetched_ptr values(3)");
+  ok_sql(hstmt, "insert into t_rows_fetched_ptr values(4)");
+  ok_sql(hstmt, "insert into t_rows_fetched_ptr values(5)");
 
     rowsSize= 1;
     rc = SQLSetStmtAttr(hstmt, SQL_ATTR_ROW_ARRAY_SIZE, (SQLPOINTER)rowsSize, 0);
@@ -723,7 +693,7 @@ DECLARE_TEST(t_rows_fetched_ptr1)
     rc = SQLSetStmtAttr(hstmt, SQL_ATTR_ROWS_FETCHED_PTR, &rowsFetched, 0);
     mystmt(hstmt,rc);
 
-    rc = SQLExecDirect(hstmt, "SELECT * FROM t_rows_fetched_ptr",SQL_NTS);
+    ok_sql(hstmt, "SELECT * FROM t_rows_fetched_ptr");
     mystmt(hstmt,rc);
 
     i= 0;
@@ -745,7 +715,7 @@ DECLARE_TEST(t_rows_fetched_ptr1)
     rc = SQLSetStmtAttr(hstmt, SQL_ATTR_ROWS_FETCHED_PTR, &rowsFetched, 0);
     mystmt(hstmt,rc);
 
-    rc = SQLExecDirect(hstmt, "SELECT * FROM t_rows_fetched_ptr",SQL_NTS);
+    ok_sql(hstmt, "SELECT * FROM t_rows_fetched_ptr");
     mystmt(hstmt,rc);
 
     i= 0;
@@ -767,7 +737,7 @@ DECLARE_TEST(t_rows_fetched_ptr1)
     rc = SQLSetStmtAttr(hstmt, SQL_ATTR_ROWS_FETCHED_PTR, &rowsFetched, 0);
     mystmt(hstmt,rc);
 
-    rc = SQLExecDirect(hstmt, "SELECT * FROM t_rows_fetched_ptr",SQL_NTS);
+    ok_sql(hstmt, "SELECT * FROM t_rows_fetched_ptr");
     mystmt(hstmt,rc);
 
     i= 0;
@@ -789,7 +759,7 @@ DECLARE_TEST(t_rows_fetched_ptr1)
     rc = SQLSetStmtAttr(hstmt, SQL_ATTR_ROWS_FETCHED_PTR, &rowsFetched, 0);
     mystmt(hstmt,rc);
 
-    rc = SQLExecDirect(hstmt, "SELECT * FROM t_rows_fetched_ptr",SQL_NTS);
+    ok_sql(hstmt, "SELECT * FROM t_rows_fetched_ptr");
     mystmt(hstmt,rc);
 
     rc = SQLFetch(hstmt);
