@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2001, 2013, Oracle and/or its affiliates. All rights reserved.
+  Copyright (c) 2003, 2013, Oracle and/or its affiliates. All rights reserved.
 
   The MySQL Connector/ODBC is licensed under the terms of the GPLv2
   <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
@@ -24,6 +24,7 @@
 
 
 #include "odbctap.h"
+#include "../VersionInfo.h"
 
 /* result set demo */
 DECLARE_TEST(my_resultset)
@@ -501,30 +502,10 @@ DECLARE_TEST(t_max_rows)
 
   /* Testing max_rows with PREFETCH feature(client side cursor) enabled */
   {
-    HDBC  hdbc1;
-    HSTMT hstmt1;
-    SQLCHAR conn[512];
+    DECLARE_BASIC_HANDLES(henv1, hdbc1, hstmt1);
 
-    sprintf((char *)conn, "DSN=%s;UID=%s;PWD=%s;PREFETCH=5",
-          mydsn, myuid, mypwd);
-    if (mysock != NULL)
-    {
-      strcat((char *)conn, ";SOCKET=");
-      strcat((char *)conn, (char *)mysock);
-    }
-    if (myport)
-    {
-      char pbuff[20];
-      sprintf(pbuff, ";PORT=%d", myport);
-      strcat((char *)conn, pbuff);
-    }
-
-    ok_env(henv, SQLAllocHandle(SQL_HANDLE_DBC, henv, &hdbc1));
-
-    ok_con(hdbc1, SQLDriverConnect(hdbc1, NULL, conn, sizeof(conn), NULL,
-                                   0, NULL,
-                                   SQL_DRIVER_NOPROMPT));
-    ok_con(hdbc1, SQLAllocStmt(hdbc1, &hstmt1));
+    is(OK == alloc_basic_handles_with_opt(&henv1, &hdbc1, &hstmt1, NULL,
+                                        NULL, NULL, NULL, "PREFETCH=5"));
 
     /* max_rows is bigger than a prefetch, and is not divided evenly by it */
     ok_stmt(hstmt1, SQLSetStmtAttr(hstmt1,SQL_ATTR_MAX_ROWS,(SQLPOINTER)7,0));
@@ -562,9 +543,7 @@ DECLARE_TEST(t_max_rows)
 
     is_num(3, myrowcount(hstmt1));
 
-    ok_stmt(hstmt1, SQLFreeStmt(hstmt1,SQL_DROP));
-    ok_con(hdbc1, SQLDisconnect(hdbc1));
-    ok_con(hdbc1, SQLFreeHandle(SQL_HANDLE_DBC, hdbc1));
+    free_basic_handles(&henv1, &hdbc1, &hstmt1);
   }
 
   ok_sql(hstmt, "DROP TABLE IF EXISTS t_max_rows");
@@ -729,46 +708,43 @@ DECLARE_TEST(t_zerolength)
     myassert(bData1[1] == 'z');
 
     pcbValue= pcbValue1= 99;
-    rc = SQLGetData(hstmt,1,SQL_C_CHAR,szData,1,&pcbValue);
-    mystmt(hstmt,rc);
+    ok_stmt(hstmt, SQLGetData(hstmt,1,SQL_C_CHAR,szData,1,&pcbValue));
     printMessage("data: %s, length: %d", szData, pcbValue);
     myassert(pcbValue == 0);
     myassert(szData[0] == '\0');
 
     bData[0]=bData[1]='z';
-    rc = SQLGetData(hstmt,2,SQL_C_BINARY,bData,1,&pcbValue1);
-    mystmt(hstmt,rc);
+    ok_stmt(hstmt, SQLGetData(hstmt,2,SQL_C_BINARY,bData,1,&pcbValue1));
+
     printMessage("data: %s, length: %d", bData, pcbValue1);
     myassert(pcbValue1 == 0);
 
     bData1[0]=bData1[1]='z';
-    rc = SQLGetData(hstmt,3,SQL_C_CHAR,bData1,1,&pcbValue2);
-    mystmt(hstmt,rc);
+    ok_stmt(hstmt, SQLGetData(hstmt,3,SQL_C_CHAR,bData1,1,&pcbValue2));
     printMessage("data: %s, length: %d", bData1, pcbValue2);
     myassert(pcbValue2 == 0);
     myassert(bData1[0] == '\0');
     myassert(bData1[1] == 'z');
 
-    rc = SQLFetch(hstmt);
-    mystmt(hstmt,rc);
+    ok_stmt(hstmt, SQLFetch(hstmt));
 
     pcbValue= pcbValue1= 99;
     szData[0]= bData[0]= 'z';
     rc = SQLGetData(hstmt,1,SQL_C_CHAR,szData,0,&pcbValue);
-    mystmt_err(hstmt,rc == SQL_SUCCESS_WITH_INFO,rc);
+    mystmt_err(hstmt, rc == SQL_SUCCESS_WITH_INFO, rc);
     printMessage("length: %d", pcbValue);
     myassert(pcbValue == 4);
     myassert(szData[0] == 'z');
 
     rc = SQLGetData(hstmt,2,SQL_C_BINARY,bData,0,&pcbValue1);
-    mystmt_err(hstmt,rc == SQL_SUCCESS_WITH_INFO,rc);
+    mystmt_err(hstmt, rc == SQL_SUCCESS_WITH_INFO, rc);
     printMessage("length: %d", pcbValue1);
     myassert(pcbValue1 == 5);
     myassert(bData[0] == 'z');
 
     bData[0]=bData1[1]='z';
     rc = SQLGetData(hstmt,3,SQL_C_BINARY,bData1,0,&pcbValue2);
-    mystmt_err(hstmt,rc == SQL_SUCCESS_WITH_INFO,rc);
+    mystmt_err(hstmt, rc == SQL_SUCCESS_WITH_INFO, rc);
     printMessage("length: %d", pcbValue2);
     myassert(pcbValue2 == 5);
 
@@ -864,29 +840,18 @@ DECLARE_TEST(t_zerolength)
 /* Test the bug when two stmts are used with the don't cache results */
 DECLARE_TEST(t_cache_bug)
 {
-  SQLHENV    henv1;
-  SQLHDBC    hdbc1;
-  SQLHSTMT   hstmt1, hstmt2;
-  SQLCHAR    conn[MAX_NAME_LEN];
+  DECLARE_BASIC_HANDLES(henv1, hdbc1, hstmt1);
+  SQLHSTMT   hstmt2;
 
   ok_sql(hstmt, "DROP TABLE IF EXISTS t_cache");
   ok_sql(hstmt, "CREATE TABLE t_cache (id INT)");
   ok_sql(hstmt, "INSERT INTO t_cache VALUES (1),(2),(3),(4),(5)");
 
-  sprintf((char *)conn, "DSN=%s;UID=%s;PWD=%s;DATABASE=%s;OPTION=1048579",
-          mydsn, myuid, mypwd, mydb);
-  if (mysock != NULL)
-  {
-    strcat((char *)conn, ";SOCKET=");
-    strcat((char *)conn, (char *)mysock);
-  }
-  if (myport)
-  {
-    char pbuff[20];
-    sprintf(pbuff, ";PORT=%d", myport);
-    strcat((char *)conn, pbuff);
-  }
-  is(mydrvconnect(&henv1, &hdbc1, &hstmt1, conn) == OK);
+  SET_DSN_OPTION(1048579);
+
+  is(OK == alloc_basic_handles(&henv1, &hdbc1, &hstmt1));
+
+  SET_DSN_OPTION(0);
 
   ok_stmt(hstmt1, SQLSetStmtAttr(hstmt1, SQL_ATTR_CURSOR_TYPE,
                                  (SQLPOINTER)SQL_CURSOR_STATIC, 0));
@@ -916,11 +881,7 @@ DECLARE_TEST(t_cache_bug)
 
   expect_stmt(hstmt1, SQLFetch(hstmt1), SQL_NO_DATA);
 
-  ok_stmt(hstmt1, SQLFreeStmt(hstmt1, SQL_DROP));
-
-  ok_con(hdbc1, SQLDisconnect(hdbc1));
-  ok_con(hdbc1, SQLFreeConnect(hdbc1));
-  ok_env(henv1, SQLFreeEnv(henv1));
+  free_basic_handles(&henv1, &hdbc1, &hstmt1);
 
   ok_sql(hstmt, "DROP TABLE IF EXISTS t_cache");
 
@@ -931,26 +892,18 @@ DECLARE_TEST(t_cache_bug)
 /* Test the bug when two stmts are used with the don't cache results */
 DECLARE_TEST(t_non_cache_bug)
 {
-  SQLHENV    henv1;
-  SQLHDBC    hdbc1;
-  SQLHSTMT   hstmt1, hstmt2;
-  SQLCHAR    conn[MAX_NAME_LEN];
+  DECLARE_BASIC_HANDLES(henv1, hdbc1, hstmt1);
+  SQLHSTMT   hstmt2;
 
   ok_sql(hstmt, "DROP TABLE IF EXISTS t_cache");
   ok_sql(hstmt, "CREATE TABLE t_cache (id INT)");
   ok_sql(hstmt, "INSERT INTO t_cache VALUES (1),(2),(3),(4),(5)");
 
-  sprintf((char *)conn, "DSN=%s;UID=%s;PWD=%s;DATABASE=%s;OPTION=3",
-          mydsn, myuid, mypwd, mydb);
-  if (mysock != NULL)
-  {
-    strcat((char *)conn, ";SOCKET=");
-    strcat((char *)conn, (char *)mysock);
-  }
-  is(mydrvconnect(&henv1, &hdbc1, &hstmt1, conn) == OK);
+  SET_DSN_OPTION(3);
 
-  ok_stmt(hstmt1, SQLSetStmtAttr(hstmt1, SQL_ATTR_CURSOR_TYPE,
-                                 (SQLPOINTER)SQL_CURSOR_STATIC, 0));
+  is(OK == alloc_basic_handles(&henv1, &hdbc1, &hstmt1));
+
+  SET_DSN_OPTION(0);
 
   ok_sql(hstmt1, "SELECT * FROM t_cache");
 
@@ -977,11 +930,7 @@ DECLARE_TEST(t_non_cache_bug)
 
   expect_stmt(hstmt1, SQLFetch(hstmt1), SQL_NO_DATA);
 
-  ok_stmt(hstmt1, SQLFreeStmt(hstmt1, SQL_DROP));
-
-  ok_con(hdbc1, SQLDisconnect(hdbc1));
-  ok_con(hdbc1, SQLFreeConnect(hdbc1));
-  ok_env(henv1, SQLFreeEnv(henv1));
+  free_basic_handles(&henv1, &hdbc1, &hstmt1);
 
   ok_sql(hstmt, "DROP TABLE IF EXISTS t_cache");
 
@@ -1797,7 +1746,7 @@ DECLARE_TEST(t_bug16817)
   is_str(my_fetch_str(hstmt, name, 1), "Zack", 4);
   ok_stmt(hstmt, SQLMoreResults(hstmt));
 
-  ok_stmt(hstmt, SQLNumResultCols(hstmt,&ncol));
+  ok_stmt(hstmt, SQLNumResultCols(hstmt, &ncol));
   is_num(ncol, 0);
 
   ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
@@ -1832,13 +1781,14 @@ DECLARE_TEST(t_binary_collation)
       (!strncmp("5.1", (char *)server_version, 3) &&
         mysql_min_version(hdbc, "5.1.22", 6)))
   {
-    is_num(data_type, SQL_WVARCHAR);
+    is_num(data_type, unicode_driver ? SQL_WVARCHAR : SQL_VARCHAR);
   }
   else
   {
     is_num(data_type, SQL_VARBINARY);
   }
   ok_sql(hstmt, "DROP TABLE IF EXISTS t_binary_collation");
+
   return OK;
 }
 
@@ -2113,9 +2063,7 @@ DECLARE_TEST(t_bug31246)
 */
 DECLARE_TEST(t_bug13776)
 {
-  SQLHENV  henv1;
-  SQLHDBC  hdbc1;
-  SQLHSTMT hstmt1;
+  DECLARE_BASIC_HANDLES(henv1, hdbc1, hstmt1);
 
   SQLULEN     pcColSz;
   SQLCHAR     szColName[MAX_NAME_LEN];
@@ -2164,9 +2112,7 @@ DECLARE_TEST(t_bug13776)
 DECLARE_TEST(t_bug13776_auto)
 {
 #ifdef WIN32
-  SQLHENV  henv1;
-  SQLHDBC  hdbc1;
-  SQLHSTMT hstmt1;
+  DECLARE_BASIC_HANDLES(henv1, hdbc1, hstmt1);
   HMODULE  ado_dll;
 
   SQLULEN     pcColSz;
