@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2013, Oracle and/or its affiliates. All rights reserved.
+  Copyright (c) 2003, 2013, Oracle and/or its affiliates. All rights reserved.
 
   The MySQL Connector/ODBC is licensed under the terms of the GPLv2
   <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
@@ -24,6 +24,7 @@
 
 
 #include "odbctap.h"
+#include "../VersionInfo.h"
 
 
 /*
@@ -31,7 +32,7 @@
 */
 DECLARE_TEST(t_bug32420)
 {
-  SQLHANDLE henv1, hdbc1, hstmt1;
+  DECLARE_BASIC_HANDLES(henv1, hdbc1, hstmt1);
   SQLINTEGER nData[4];
   SQLCHAR szData[4][16];
   SQLUSMALLINT rgfRowStatus[4];
@@ -120,6 +121,7 @@ DECLARE_TEST(t_bug34575)
   return OK;
 }
 
+
 /*
 Bug #24131 SHOW CREATE TABLE result truncated with mysql 3.23 and ODBC driver 3.51.12.00
 */
@@ -700,30 +702,10 @@ DECLARE_TEST(t_row_status)
 
 DECLARE_TEST(t_prefetch)
 {
-    HDBC  hdbc1;
-    HSTMT hstmt1;
-    SQLCHAR conn[512];
+    DECLARE_BASIC_HANDLES(henv1, hdbc1, hstmt1);
 
-    sprintf((char *)conn, "DSN=%s;UID=%s;PWD=%s;PREFETCH=5",
-          mydsn, myuid, mypwd);
-    if (mysock != NULL)
-    {
-      strcat((char *)conn, ";SOCKET=");
-      strcat((char *)conn, (char *)mysock);
-    }
-    if (myport)
-    {
-      char pbuff[20];
-      sprintf(pbuff, ";PORT=%d", myport);
-      strcat((char *)conn, pbuff);
-    }
-
-    ok_env(henv, SQLAllocHandle(SQL_HANDLE_DBC, henv, &hdbc1));
-
-    ok_con(hdbc1, SQLDriverConnect(hdbc1, NULL, conn, sizeof(conn), NULL,
-                                   0, NULL,
-                                   SQL_DRIVER_NOPROMPT));
-    ok_con(hdbc1, SQLAllocStmt(hdbc1, &hstmt1));
+    is(OK == alloc_basic_handles_with_opt(&henv1, &hdbc1, &hstmt1, NULL,
+                                        NULL, NULL, NULL, "PREFETCH=5"));
 
     ok_sql(hstmt, "DROP table IF EXISTS b_prefecth");
     ok_sql(hstmt, "CREATE table b_prefecth(i int)");
@@ -732,16 +714,11 @@ DECLARE_TEST(t_prefetch)
 
     ok_stmt(hstmt1, SQLPrepare(hstmt1, "select* from b_prefecth;    ", SQL_NTS));
     ok_stmt(hstmt1, SQLExecute(hstmt1));
-    ok_stmt(hstmt1, SQLFreeStmt(hstmt1,SQL_DROP));
 
-    ok_con(hdbc1, SQLDisconnect(hdbc1));
+    free_basic_handles(&henv1, &hdbc1, &hstmt1);
 
-    sprintf((char *)conn+strlen(conn), ";MULTI_STATEMENTS=1");
-
-    ok_con(hdbc1, SQLDriverConnect(hdbc1, NULL, conn, sizeof(conn), NULL,
-                                   0, NULL,
-                                   SQL_DRIVER_NOPROMPT));
-    ok_con(hdbc1, SQLAllocStmt(hdbc1, &hstmt1));
+    is(OK == alloc_basic_handles_with_opt(&henv1, &hdbc1, &hstmt1, NULL,
+                                        NULL, NULL, NULL, "MULTI_STATEMENTS=1"));
 
     ok_stmt(hstmt1, SQLPrepare(hstmt1, "select* from b_prefecth;\
                                         select * from b_prefecth where i < 7; ",
@@ -757,9 +734,7 @@ DECLARE_TEST(t_prefetch)
 
     expect_stmt(hstmt1, SQLMoreResults(hstmt1), SQL_NO_DATA);
 
-    ok_stmt(hstmt1, SQLFreeStmt(hstmt1,SQL_DROP));
-    ok_con(hdbc1, SQLDisconnect(hdbc1));
-    ok_con(hdbc1, SQLFreeHandle(SQL_HANDLE_DBC, hdbc1));
+    free_basic_handles(&henv1, &hdbc1, &hstmt1);
 
     ok_sql(hstmt, "DROP table IF EXISTS b_prefecth");
 
@@ -767,6 +742,77 @@ DECLARE_TEST(t_prefetch)
 }
 
 
+DECLARE_TEST(t_outparams)
+{
+  SQLSMALLINT ncol, i;
+  SQLINTEGER par[3]= {10, 20, 30}, val, len;
+
+  ok_sql(hstmt, "DROP PROCEDURE IF EXISTS p_outparams");
+  ok_sql(hstmt, "CREATE PROCEDURE p_outparams("
+                "  IN p_in INT, "
+                "  OUT p_out INT, "
+                "  INOUT p_inout INT) "
+                "BEGIN "
+                "  SELECT p_in, p_out, p_inout; "
+                "  SET p_in = 100, p_out = 200, p_inout = 300; "
+                "  SELECT p_inout, p_in, p_out;"
+                "END");
+
+
+  for (i=0; i < sizeof(par)/sizeof(SQLINTEGER); ++i)
+  {
+    ok_stmt(hstmt, SQLBindParameter(hstmt, i+1, SQL_PARAM_INPUT, SQL_C_LONG, SQL_INTEGER, 0,
+      0, &par[i], 0, NULL));
+  }
+
+  ok_sql(hstmt, "CALL p_outparams(?, ?, ?)");
+
+  /* rs-1 */
+  ok_stmt(hstmt, SQLFetch(hstmt));
+  ok_stmt(hstmt, SQLNumResultCols(hstmt,&ncol));
+  is_num(ncol, 3);
+
+  is_num(my_fetch_int(hstmt, 1), 10);
+  /* p_out does not have value at the moment */
+  ok_stmt(hstmt, SQLGetData(hstmt, 2, SQL_INTEGER, &val, 0, &len));
+  is_num(len, SQL_NULL_DATA);
+  is_num(my_fetch_int(hstmt, 3), 30);
+
+  expect_stmt(hstmt, SQLFetch(hstmt), SQL_NO_DATA);
+
+  /* rs-2 */
+  ok_stmt(hstmt, SQLMoreResults(hstmt));
+
+  ok_stmt(hstmt, SQLNumResultCols(hstmt,&ncol));
+  is_num(ncol, 3);
+  
+  ok_stmt(hstmt, SQLFetch(hstmt));
+  is_num(my_fetch_int(hstmt, 1), 300);
+  is_num(my_fetch_int(hstmt, 2), 100);
+  is_num(my_fetch_int(hstmt, 3), 200);
+
+  /* rs-3 out params */
+  ok_stmt(hstmt, SQLMoreResults(hstmt));
+  ok_stmt(hstmt, SQLNumResultCols(hstmt,&ncol));
+  is_num(ncol, 2);
+
+  ok_stmt(hstmt, SQLFetch(hstmt));
+  is_num(my_fetch_int(hstmt, 1), 200);
+  is_num(my_fetch_int(hstmt, 2), 300);
+  /* Only 1 row always */
+  expect_stmt(hstmt, SQLFetch(hstmt), SQL_NO_DATA);
+
+  /* SP execution status */
+  ok_stmt(hstmt, SQLMoreResults(hstmt));
+  expect_stmt(hstmt, SQLMoreResults(hstmt), SQL_NO_DATA);
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  ok_sql(hstmt, "DROP PROCEDURE p_outparams");
+  return OK;
+}
+
+
 /*
   Bug #11766437: Incorrect increment(increments in multiple of SQLLEN) of 
   pointer to the length/indicator buffer(last parameter of SQLBindCol), 
@@ -888,6 +934,7 @@ BEGIN_TESTS
   ADD_TEST(t_bug62657)
   ADD_TEST(t_row_status)
   ADD_TEST(t_prefetch)
+  ADD_TOFIX(t_outparams)
   ADD_TEST(t_bug11766437)
 END_TESTS
 
