@@ -25,6 +25,7 @@
 #include <wchar.h>
 #include "odbctap.h"
 #include <sqlucode.h>
+#include "../driver/error.h"
 
 
 DECLARE_TEST(sqlconnect)
@@ -1055,11 +1056,13 @@ DECLARE_TEST(t_bug34672)
   {
     chars[0]= 0xd802;
     chars[1]= 0xdc60;
+    chars[2]= 0;
     inchars= 2;
   }
   else
   {
     chars[0]= (SQLWCHAR) 0x10860;
+    chars[1]= 0;
     inchars= 1;
   }
 
@@ -1187,7 +1190,6 @@ DECLARE_TEST(t_bug28168)
   ok_con(hdbc2, SQLGetDiagRecW(SQL_HANDLE_DBC, hdbc2, 1,
                                sqlstate, &native_error, errmsgtxt,
                                256 * sizeof(SQLWCHAR), &errmsglen));
-  
   ok_con(hdbc2, SQLFreeHandle(SQL_HANDLE_DBC, hdbc2));
 
   /* 
@@ -1197,7 +1199,6 @@ DECLARE_TEST(t_bug28168)
    wstr= sqlwchar_to_wchar_t(errmsgtxt);
    is(wcsstr(wstr,  
              L"Access denied for user '\x03A8\x0391\x03A1\x039F uid'@") != NULL);
-   
   ok_stmt(hstmt1,SQLExecDirectW(hstmt1,
     W(L"DROP USER "
     L"'\x03A8\x0391\x03A1\x039F uid'@localhost"), SQL_NTS));
@@ -1347,30 +1348,92 @@ DECLARE_TEST(t_bug14363601)
 }
 
 
+/*
+  Bug#14838690: ODBC 5.2.2 CONNECTOR BROKEN FOR ALL SQL SERVERS 
+  OLDER THAN 5.5.3
+  With utf8 byte string, Mysql server version < 5.5.3 failes with
+  error message "Server does not support 4-byte encoded UTF8 characters."
+*/
+DECLARE_TEST(t_bug14838690)
+{
+  int i;
+  SQLINTEGER col_id= 1234;
+  SQLWCHAR *col_vc= W(L"abcdefg\x30a1"), col_vc_res[30];
+
+  ok_sql(hstmt, "DROP TABLE IF EXISTS bug14838690");
+  ok_sql(hstmt, "CREATE TABLE bug14838690("
+                 "id INT, vc VARCHAR(32)"
+                 ")CHARSET=UTF8");
+
+  ok_stmt(hstmt, SQLPrepareW(hstmt, 
+		    W(L"INSERT INTO bug14838690 (id, vc) "
+                      L"VALUES (?, ?)"), SQL_NTS));
+
+  /* Bind 1st INT param */
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT, SQL_C_LONG,
+                                  SQL_INTEGER, 0, 0, &col_id, 0, NULL));
+  
+  /* Bind 2nd VARCHAR param with utf8 byte string */
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 2, SQL_PARAM_INPUT, SQL_C_WCHAR,
+                                   SQL_WVARCHAR, 10, 0, col_vc, 
+                                  10*sizeof(SQLWCHAR), NULL));
+
+  ok_stmt(hstmt, SQLExecute(hstmt));
+
+  ok_stmt(hstmt, SQLExecDirectW(hstmt, W(L"SELECT * FROM bug14838690"), 
+                                 SQL_NTS));
+
+  ok_stmt(hstmt, SQLFetch(hstmt));
+
+  is_num(my_fetch_int(hstmt, 1), col_id);
+
+  ok_stmt(hstmt, SQLGetData(hstmt, 2, SQL_C_WCHAR, col_vc_res,
+                             sizeof(col_vc_res), NULL));
+
+  /* we want to compare SQLWCHAR instead of wchar_t */
+  for (i= 0; i < 8; i++)
+  {
+    is(col_vc[i] == col_vc_res[i]);
+  }
+
+  expect_stmt(hstmt, SQLFetch(hstmt), SQL_NO_DATA_FOUND);
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  ok_sql(hstmt, "DROP TABLE IF EXISTS bug14838690");
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_DROP));
+  
+  /* OK if it has not crashed */
+  return OK;
+}
+
+
 BEGIN_TESTS
   ADD_TEST(sqlconnect)
-  ADD_TEST(sqlprepare)
+  ADD_TEST_UNICODE(sqlprepare)
   ADD_TEST(sqlprepare_ansi)
-  ADD_TEST(sqlchar)
+  ADD_TEST_UNICODE(sqlchar)
   ADD_TEST(sqldriverconnect)
   ADD_TEST(sqlnativesql)
   ADD_TEST(sqlsetcursorname)
   ADD_TEST(sqlgetcursorname)
-  ADD_TEST(sqlcolattribute)
-  ADD_TEST(sqldescribecol)
+  ADD_TEST_UNICODE(sqlcolattribute)
+  ADD_TEST_UNICODE(sqldescribecol)
   ADD_TEST(sqlgetconnectattr)
   ADD_TEST(sqlgetdiagrec)
   ADD_TEST(sqlgetdiagfield)
-  ADD_TEST(sqlcolumns)
-  ADD_TEST(sqltables)
-  ADD_TEST(sqlspecialcolumns)
+  ADD_TEST_UNICODE(sqlcolumns)
+  ADD_TEST_UNICODE(sqltables)
+  ADD_TEST_UNICODE(sqlspecialcolumns)
   ADD_TEST(sqlforeignkeys)
-  ADD_TEST(sqlprimarykeys)
-  ADD_TEST(sqlstatistics)
-  ADD_TEST(t_bug32161)
-  ADD_TEST(t_bug34672)
-  ADD_TEST(t_bug28168)
-  ADD_TEST(t_bug14363601)
+  ADD_TEST_UNICODE(sqlprimarykeys)
+  ADD_TEST_UNICODE(sqlstatistics)
+  ADD_TEST_UNICODE(t_bug32161)
+  ADD_TEST_UNICODE(t_bug34672)
+  ADD_TEST_UNICODE(t_bug28168)
+  ADD_TEST_UNICODE(t_bug14363601)
+  ADD_TEST_UNICODE(t_bug14838690)
 END_TESTS
 
 
