@@ -162,6 +162,7 @@ static SQLWCHAR W_CLIENT_INTERACTIVE[]=
   {'I','N','T','E','R','A','C','T','I','V','E',0};
 static SQLWCHAR W_NO_I_S[]= {'N','O','_','I','_','S',0};
 static SQLWCHAR W_PREFETCH[]= {'P','R','E','F','E','T','C','H',0};
+static SQLWCHAR W_NO_SSPS[]= {'N','O','_','S','S','P','S',0};
 static SQLWCHAR W_CAN_HANDLE_EXP_PWD[]=
   {'C','A','N','_','H','A','N','D','L','E','_','E','X','P','_','P','W','D',0};
 static SQLWCHAR W_ENABLE_CLEARTEXT_PLUGIN[]=
@@ -171,7 +172,7 @@ static SQLWCHAR W_ENABLE_CLEARTEXT_PLUGIN[]=
 /* externally used strings */
 const SQLWCHAR W_DRIVER_PARAM[]= {';', 'D', 'R', 'I', 'V', 'E', 'R', '=', 0};
 const SQLWCHAR W_DRIVER_NAME[]= {'M', 'y', 'S', 'Q', 'L', ' ',
-                                 'O', 'D', 'B', 'C', ' ', '5', '.', '1', ' ',
+                                 'O', 'D', 'B', 'C', ' ', '5', '.', '2', ' ',
                                  'D', 'r', 'i', 'v', 'e', 'r', 0};
 const SQLWCHAR W_INVALID_ATTR_STR[]= {'I', 'n', 'v', 'a', 'l', 'i', 'd', ' ',
                                       'a', 't', 't', 'r', 'i', 'b', 'u', 't', 'e', ' ',
@@ -194,7 +195,7 @@ SQLWCHAR *dsnparams[]= {W_DSN, W_DRIVER, W_DESCRIPTION, W_SERVER,
                         W_ZERO_DATE_TO_MIN, W_MIN_DATE_TO_ZERO,
                         W_MULTI_STATEMENTS, W_COLUMN_SIZE_S32,
                         W_NO_BINARY_RESULT, W_DFLT_BIGINT_BIND_STR,
-                        W_CLIENT_INTERACTIVE, W_NO_I_S, W_PREFETCH, 
+                        W_CLIENT_INTERACTIVE, W_NO_I_S, W_PREFETCH, W_NO_SSPS,
                         W_CAN_HANDLE_EXP_PWD, W_ENABLE_CLEARTEXT_PLUGIN};
 static const
 int dsnparamcnt= sizeof(dsnparams) / sizeof(SQLWCHAR *);
@@ -751,7 +752,6 @@ void ds_map_param(DataSource *ds, const SQLWCHAR *param,
     *intdest= &ds->clientinteractive;
   else if (!sqlwcharcasecmp(W_PREFETCH, param))
     *intdest= &ds->cursor_prefetch_number;
-
   else if (!sqlwcharcasecmp(W_FOUND_ROWS, param))
     *booldest= &ds->return_matching_rows;
   else if (!sqlwcharcasecmp(W_BIG_PACKETS, param))
@@ -810,6 +810,8 @@ void ds_map_param(DataSource *ds, const SQLWCHAR *param,
     *booldest= &ds->default_bigint_bind_str;
   else if (!sqlwcharcasecmp(W_NO_I_S, param))
     *booldest= &ds->no_information_schema;
+  else if (!sqlwcharcasecmp(W_NO_SSPS, param))
+    *booldest= &ds->no_ssps;
   else if (!sqlwcharcasecmp(W_CAN_HANDLE_EXP_PWD, param))
     *booldest= &ds->can_handle_exp_pwd;
   else if (!sqlwcharcasecmp(W_ENABLE_CLEARTEXT_PLUGIN, param))
@@ -1276,6 +1278,7 @@ int ds_add(DataSource *ds)
   if (ds_add_intprop(ds->name, W_NO_BINARY_RESULT, ds->handle_binary_as_char)) goto error;
   if (ds_add_intprop(ds->name, W_DFLT_BIGINT_BIND_STR, ds->default_bigint_bind_str)) goto error;
   if (ds_add_intprop(ds->name, W_NO_I_S, ds->no_information_schema)) goto error;
+  if (ds_add_intprop(ds->name, W_NO_SSPS, ds->no_ssps)) goto error;
   if (ds_add_intprop(ds->name, W_CAN_HANDLE_EXP_PWD, ds->can_handle_exp_pwd)) goto error;
   if (ds_add_intprop(ds->name, W_ENABLE_CLEARTEXT_PLUGIN, ds->enable_cleartext_plugin)) goto error;
   /* DS_PARAM */
