@@ -207,30 +207,47 @@ SQLCHAR *sqlwchar_as_sqlchar(CHARSET_INFO *charset_info, SQLWCHAR *str,
 
 
 /**
-  Duplicate a SQLWCHAR as a SQLCHAR encoded as UTF-8.
+  A bit extended version of sqlwchar_as_utf8 to be used by it and in other
+  places where sqlwchar_as_utf8 could not be used
 
   @param[in]      str           String to convert
   @param[in,out]  len           Pointer to length of source (in chars) or
-                                destination string (in bytes)
+                                destination string (in bytes). 
+  @param[in]      buff          Buffer to put the result string if it fits
+  @param[in]      buff_max      max size(in bytes) of the buff.
+  @param[out]     utf8mb4_used  has 4 bytes utf8 characters been used
 
   @return  Pointer to a newly allocated SQLCHAR, or @c NULL
 */
-SQLCHAR *sqlwchar_as_utf8(const SQLWCHAR *str, SQLINTEGER *len)
+SQLCHAR *sqlwchar_as_utf8_ext(const SQLWCHAR *str, SQLINTEGER *len,
+                              SQLCHAR *buff, uint buff_max, int *utf8mb4_used)
 {
   const SQLWCHAR *str_end;
   UTF8 *u8;
+  int utf8len, dummy;
   SQLINTEGER i;
 
-  if (*len == SQL_NTS)
-    *len= sqlwcharlen(str);
-  if (!str || *len == 0)
+  if (!str || *len <= 0)
   {
     *len= 0;
-    return NULL;
+    return buff;
   }
 
-  u8= (UTF8 *)my_malloc(sizeof(UTF8) * MAX_BYTES_PER_UTF8_CP * *len + 1,
+  if (utf8mb4_used == NULL)
+  {
+    utf8mb4_used= &dummy;
+  }
+
+  if (buff == NULL || buff_max < (uint)(*len * MAX_BYTES_PER_UTF8_CP))
+  {
+    u8= (UTF8 *)my_malloc(sizeof(UTF8) * MAX_BYTES_PER_UTF8_CP * *len + 1,
                         MYF(0));
+  }
+  else
+  {
+    u8= buff;
+  }
+
   if (!u8)
   {
     *len= -1;
@@ -242,7 +259,19 @@ SQLCHAR *sqlwchar_as_utf8(const SQLWCHAR *str, SQLINTEGER *len)
   if (sizeof(SQLWCHAR) == 4)
   {
     for (i= 0; str < str_end; )
-      i+= utf32toutf8((UTF32)*str++, u8 + i);
+    {
+      i+= (utf8len= utf32toutf8((UTF32)*str++, u8 + i));
+
+      /*
+        utf8mb4 is a superset of utf8, only supplemental characters
+        which require four bytes differs in storage characteristics (length)
+        between utf8 and utf8mb4.
+      */
+      if (utf8len == 4)
+      {
+        *utf8mb4_used= 1;
+      }
+    }
   }
   else
   {
@@ -250,19 +279,67 @@ SQLCHAR *sqlwchar_as_utf8(const SQLWCHAR *str, SQLINTEGER *len)
     {
       UTF32 u32;
       int consumed= utf16toutf32((UTF16 *)str, &u32);
-      str+= consumed;
       if (!consumed)
+      {
         break;
-      i+= utf32toutf8(u32, u8 + i);
+      }
+
+      str+= consumed;
+
+      i+= (utf8len= utf32toutf8(u32, u8 + i));
+      /*
+        utf8mb4 is a superset of utf8, only supplemental characters
+        which require four bytes differs in storage characteristics (length)
+        between utf8 and utf8mb4.
+      */
+      if (utf8len == 4)
+      {
+        *utf8mb4_used= 1;
+      }
     }
   }
 
   *len= i;
-  u8[i]= '\0';
   return u8;
 }
 
 
+/**
+  Duplicate a SQLWCHAR as a SQLCHAR encoded as UTF-8.
+
+  @param[in]      str           String to convert
+  @param[in,out]  len           Pointer to length of source (in chars) or
+                                destination string (in bytes)
+
+  @return  Pointer to a newly allocated SQLCHAR, or @c NULL
+*/
+SQLCHAR *sqlwchar_as_utf8(const SQLWCHAR *str, SQLINTEGER *len)
+{
+  SQLCHAR *res;
+
+  if (*len == SQL_NTS)
+  {
+    *len= sqlwcharlen(str);
+  }
+
+  if (!str || *len <= 0)
+  {
+    *len= 0;
+    return NULL;
+  }
+
+  res= sqlwchar_as_utf8_ext(str, len, NULL, 0, NULL);
+
+  /* If we could not allocate memory */
+  if (res != NULL)
+  {
+    res[*len]= '\0';
+  }
+
+  return res;
+}
+
+
 /**
   Convert a SQLCHAR encoded as UTF-8 into a SQLWCHAR.
 
@@ -670,3 +747,205 @@ char * myodbc_strlwr(char *target, size_t len)
 
   return target;
 }
+
+
+/********************************
+This charset mapping function is cut out from server repo: sql_common/client.c
+*********************************/
+
+typedef enum my_cs_match_type_enum
+{
+  /* MySQL and OS charsets are fully compatible */
+  my_cs_exact,
+  /* MySQL charset is very close to OS charset  */
+  my_cs_approx,
+  /*
+    MySQL knows this charset, but it is not supported as client character set.
+  */
+  my_cs_unsupp
+} my_cs_match_type;
+
+
+typedef struct str2str_st
+{
+  const char        *os_name;
+  const char        *my_name;
+  my_cs_match_type  param;
+} MY_CSET_OS_NAME;
+
+static const MY_CSET_OS_NAME charsets[]=
+{
+#ifdef __WIN__
+  {"cp437",          "cp850",    my_cs_approx},
+  {"cp850",          "cp850",    my_cs_exact},
+  {"cp852",          "cp852",    my_cs_exact},
+  {"cp858",          "cp850",    my_cs_approx},
+  {"cp866",          "cp866",    my_cs_exact},
+  {"cp874",          "tis620",   my_cs_approx},
+  {"cp932",          "cp932",    my_cs_exact},
+  {"cp936",          "gbk",      my_cs_approx},
+  {"cp949",          "euckr",    my_cs_approx},
+  {"cp950",          "big5",     my_cs_exact},
+  {"cp1200",         "utf16le",  my_cs_unsupp},
+  {"cp1201",         "utf16",    my_cs_unsupp},
+  {"cp1250",         "cp1250",   my_cs_exact},
+  {"cp1251",         "cp1251",   my_cs_exact},
+  {"cp1252",         "latin1",   my_cs_exact},
+  {"cp1253",         "greek",    my_cs_exact},
+  {"cp1254",         "latin5",   my_cs_exact},
+  {"cp1255",         "hebrew",   my_cs_approx},
+  {"cp1256",         "cp1256",   my_cs_exact},
+  {"cp1257",         "cp1257",   my_cs_exact},
+  {"cp10000",        "macroman", my_cs_exact},
+  {"cp10001",        "sjis",     my_cs_approx},
+  {"cp10002",        "big5",     my_cs_approx},
+  {"cp10008",        "gb2312",   my_cs_approx},
+  {"cp10021",        "tis620",   my_cs_approx},
+  {"cp10029",        "macce",    my_cs_exact},
+  {"cp12001",        "utf32",    my_cs_unsupp},
+  {"cp20107",        "swe7",     my_cs_exact},
+  {"cp20127",        "latin1",   my_cs_approx},
+  {"cp20866",        "koi8r",    my_cs_exact},
+  {"cp20932",        "ujis",     my_cs_exact},
+  {"cp20936",        "gb2312",   my_cs_approx},
+  {"cp20949",        "euckr",    my_cs_approx},
+  {"cp21866",        "koi8u",    my_cs_exact},
+  {"cp28591",        "latin1",   my_cs_approx},
+  {"cp28592",        "latin2",   my_cs_exact},
+  {"cp28597",        "greek",    my_cs_exact},
+  {"cp28598",        "hebrew",   my_cs_exact},
+  {"cp28599",        "latin5",   my_cs_exact},
+  {"cp28603",        "latin7",   my_cs_exact},
+#ifdef UNCOMMENT_THIS_WHEN_WL_4579_IS_DONE
+  {"cp28605",        "latin9",   my_cs_exact},
+#endif
+  {"cp38598",        "hebrew",   my_cs_exact},
+  {"cp51932",        "ujis",     my_cs_exact},
+  {"cp51936",        "gb2312",   my_cs_exact},
+  {"cp51949",        "euckr",    my_cs_exact},
+  {"cp51950",        "big5",     my_cs_exact},
+#ifdef UNCOMMENT_THIS_WHEN_WL_WL_4024_IS_DONE
+  {"cp54936",        "gb18030",  my_cs_exact},
+#endif
+  {"cp65001",        "utf8",     my_cs_exact},
+
+#else /* not Windows */
+
+  {"646",            "latin1",   my_cs_approx}, /* Default on Solaris */
+  {"ANSI_X3.4-1968", "latin1",   my_cs_approx},
+  {"ansi1251",       "cp1251",   my_cs_exact},
+  {"armscii8",       "armscii8", my_cs_exact},
+  {"armscii-8",      "armscii8", my_cs_exact},
+  {"ASCII",          "latin1",   my_cs_approx},
+  {"Big5",           "big5",     my_cs_exact},
+  {"cp1251",         "cp1251",   my_cs_exact},
+  {"cp1255",         "hebrew",   my_cs_approx},
+  {"CP866",          "cp866",    my_cs_exact},
+  {"eucCN",          "gb2312",   my_cs_exact},
+  {"euc-CN",         "gb2312",   my_cs_exact},
+  {"eucJP",          "ujis",     my_cs_exact},
+  {"euc-JP",         "ujis",     my_cs_exact},
+  {"eucKR",          "euckr",    my_cs_exact},
+  {"euc-KR",         "euckr",    my_cs_exact},
+#ifdef UNCOMMENT_THIS_WHEN_WL_WL_4024_IS_DONE
+  {"gb18030",        "gb18030",  my_cs_exact},
+#endif
+  {"gb2312",         "gb2312",   my_cs_exact},
+  {"gbk",            "gbk",      my_cs_exact},
+  {"georgianps",     "geostd8",  my_cs_exact},
+  {"georgian-ps",    "geostd8",  my_cs_exact},
+  {"IBM-1252",       "cp1252",   my_cs_exact},
+
+  {"iso88591",       "latin1",   my_cs_approx},
+  {"ISO_8859-1",     "latin1",   my_cs_approx},
+  {"ISO8859-1",      "latin1",   my_cs_approx},
+  {"ISO-8859-1",     "latin1",   my_cs_approx},
+
+  {"iso885913",      "latin7",   my_cs_exact},
+  {"ISO_8859-13",    "latin7",   my_cs_exact},
+  {"ISO8859-13",     "latin7",   my_cs_exact},
+  {"ISO-8859-13",    "latin7",   my_cs_exact},
+
+#ifdef UNCOMMENT_THIS_WHEN_WL_4579_IS_DONE
+  {"iso885915",      "latin9",   my_cs_exact},
+  {"ISO_8859-15",    "latin9",   my_cs_exact},
+  {"ISO8859-15",     "latin9",   my_cs_exact},
+  {"ISO-8859-15",    "latin9",   my_cs_exact},
+#endif
+
+  {"iso88592",       "latin2",   my_cs_exact},
+  {"ISO_8859-2",     "latin2",   my_cs_exact},
+  {"ISO8859-2",      "latin2",   my_cs_exact},
+  {"ISO-8859-2",     "latin2",   my_cs_exact},
+
+  {"iso88597",       "greek",    my_cs_exact},
+  {"ISO_8859-7",     "greek",    my_cs_exact},
+  {"ISO8859-7",      "greek",    my_cs_exact},
+  {"ISO-8859-7",     "greek",    my_cs_exact},
+
+  {"iso88598",       "hebrew",   my_cs_exact},
+  {"ISO_8859-8",     "hebrew",   my_cs_exact},
+  {"ISO8859-8",      "hebrew",   my_cs_exact},
+  {"ISO-8859-8",     "hebrew",   my_cs_exact},
+
+  {"iso88599",       "latin5",   my_cs_exact},
+  {"ISO_8859-9",     "latin5",   my_cs_exact},
+  {"ISO8859-9",      "latin5",   my_cs_exact},
+  {"ISO-8859-9",     "latin5",   my_cs_exact},
+
+  {"koi8r",          "koi8r",    my_cs_exact},
+  {"KOI8-R",         "koi8r",    my_cs_exact},
+  {"koi8u",          "koi8u",    my_cs_exact},
+  {"KOI8-U",         "koi8u",    my_cs_exact},
+
+  {"roman8",         "hp8",      my_cs_exact}, /* Default on HP UX */
+
+  {"Shift_JIS",      "sjis",     my_cs_exact},
+  {"SJIS",           "sjis",     my_cs_exact},
+  {"shiftjisx0213",  "sjis",     my_cs_exact},
+  
+  {"tis620",         "tis620",   my_cs_exact},
+  {"tis-620",        "tis620",   my_cs_exact},
+
+  {"ujis",           "ujis",     my_cs_exact},
+
+  {"US-ASCII",       "latin1",   my_cs_approx},
+
+  {"utf8",           "utf8",     my_cs_exact},
+  {"utf-8",          "utf8",     my_cs_exact},
+#endif
+  {NULL,             NULL,       0}
+};
+
+
+const char *
+my_os_charset_to_mysql_charset(const char *csname)
+{
+  const MY_CSET_OS_NAME *csp;
+  for (csp= charsets; csp->os_name; ++csp)
+  {
+    if (!my_strcasecmp(&my_charset_latin1, csp->os_name, csname))
+    {
+      switch (csp->param)
+      {
+      case my_cs_exact:
+        return csp->my_name;
+
+      case my_cs_approx:
+        /*
+          Maybe we should print a warning eventually:
+          character set correspondence is not exact.
+        */
+        return csp->my_name;
+
+      default:
+        goto def;
+      }
+    }
+  }
+
+def:
+  csname= MYSQL_DEFAULT_CHARSET_NAME;
+
+  return csname;
+}
